/*

  COPYRIGHT
  --------
  
    Copyright Maurizio Colucci -  9 nov 2003.
  
  COPYING
  -------
  
    This file is part of segusoLand.

    segusoLand is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    segusoLand is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with segusoLand; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 QUICK INTRO TO THE CODE
 -----------------------
* in prolog, variables start with uppercase, constants with lowercase.

  functions or procedures are called "predicates" in prolog.

  When referring to predicates, it is usual to write p/3, where
  /3 is the arity of the predicate (the number of arguments it takes).

*  If you are looking for the main() procedure, it is 
   run/1, further down this file. Actually, run/1 is called by

      int main(void)

   which is written in C++, in the file segusoLand.cc.

  
*  if you can't find the definition of some predicates in this file,
   it's because they are written in C. This was necessary in order to use GTK+.
   For example, widgetShow. See segusoLand.cc.

* Sadly, I chose inconsistent names for predicates. Keep in mind that

   'folder' and 'table' and 'feature' and 'metadata' and 'attribute'
    and 'category' are interchangeable terms.

   'view' and 'filter' and 'virtual folder' and 'derived table' are
   interchangeable terms. (views are ANDS or ORS of
   folders.)

   I plan to rename the atoms to fix these inconsistencies. I still
   haven't decided whether to use the term "category" or "folder".

*  In the code I frequently use the word "Kind". Please remember that
  
    Kind = program | file | verb | time | device | folder 

    When you see a variable named K, it is a shortcut for Kind.

* metapredicates: in prolog

     call(pred, X)

  means

     p(X).

  So why do we use call/2 sometimes? Because in prolog we can't write

    P(X).

  where P is a variable. This is called "quantifying over predicates".
  So, if we need to quantify over predicates,
  we use call/2. call/2 is a metapredicate, because it takes a predicate
  as the first argument.


* when in the code you see @@, inside a comment, it means that is a
  quick hack, dirty code, or a TODO item.


* related/3. This is one of the main predicates in the code.

  The first argument is a table name, the second is a kind, the third is an item name.

       related(document, file, 'file.pdf')

   means

      "the file 'file.pdf' is related to the idea 'good'"

   Other example:
  
    related(editor, verb, 'Emacs')

*/

% REGION declare dynamic predicates. 
% in prolog, you have to declare a predicate only if it's dynamic, i.e.
% if you plan to do assert and retract on it.
:-dynamic(selected/2). % args: Kind, object.
:-dynamic(showFoldersBelowFiles/0).
:-dynamic(showFoldersBelowPrograms/0).
:-dynamic(showFoldersBelowVerbs/0).
:-dynamic(selectedLetter/1).
:-dynamic(theQuickSearchBoxesAreVisible/0).
:-dynamic(panelStateMinimized/1).
:-dynamic(showPanelsWhenNoConceptIsSelected/0).
:-dynamic(tooltipsAreEnabled/0). 
:-dynamic(scrollingButtonsAlwaysVisible/0).
:-dynamic(autoDeselectConcepts/0).
:-dynamic(recentConcept/1).
:-dynamic(recentAction/2).
:-dynamic(file/1).
:-dynamic(idea/2).
:-dynamic(verb/1).
:-dynamic(directory/1).
:-dynamic(device/1).
:-dynamic(program/1).
:-dynamic(visibleVerb/1).
:-dynamic(visibleObject/2). % Kind, Object
:-dynamic(related/3). % see quick intro for explanation of the meaning of related/3
:-dynamic(ext/2). % example: ext(png, '/usr/doc.png').
:-dynamic(segusoDir/1). % segusoDir(D) means that D is a directory indexed by segusoLand
:-dynamic(strongNarrowing/0).
% END

% by default, we showFileFoldersBelowFiles. 
showFoldersBelowFiles.
showFoldersBelowPrograms.




% REGION remove/3: predicate that removes all occurrences of an element from a list
remove(Item, List, NewList):-
	remove(Item, List, [], NewList).
remove(_, [], CurrentList, CurrentList).
remove(Item, List, CurrentList, NewList):-
	List\=[],
	List=[Item|Tail],
	remove(Item, Tail, CurrentList, NewList).
remove(Item, List, CurrentList, NewList):-
	List\=[],
	List=[H|Tail],
	H\=Item,
	remove(Item, Tail, [H|CurrentList], NewList).
% END


% @@ remove this
selectedFiles(L):-
	findall(X, selected(file, X), L).
selectedTimes(L):-
	findall(X, selected(time, X), L).


% REGION these few lines are the very heart of segusoLand! Verb narrowing.
theVerbIsVisible(/*in*/V):-
	selected(verb, V).  % a verb is visible if it is selected...
theVerbIsVisible(/*in*/V):-
	forall( selected(idea-K, I),
		theVerbIsCompatibleWithTheIdea(V, I, K)),
	theNameMatchesFilter(verb, V), % ...and it is not filtered away...
	\+thereIsASelectedElementThatMakesNoSenseWithTheVerb(V).

% END


thereIsASelectedElementThatMakesNoSenseWithTheVerb(V):-
	selected(K, IncompatibleMember),
	IncompatibleMember \= V,
	K\=time, % times are always compatible with all verbs
	K\=idea-_,
	forall(verbRequiresAtLeastOneObjectSatisfying(V, K, P),
	       \+call(P, IncompatibleMember)).


theVerbIsCompatibleWithTheIdea(V, I, verb):-
	related(I, verb, V).
theVerbIsCompatibleWithTheIdea(V, I, K):-
	K\=verb,
	K\=idea-_,
	verbRequiresAtLeastOneObjectSatisfying(V, K, P),	
	related(I, K, O),
	call(P, O).


	       

verbRequiresAtLeastOneObjectSatisfying('Associate concepts to files', file, anyFile).

verbRequiresAtLeastOneObjectSatisfying('Associate concepts to directory (and all its files)', directory, anyDirectory).

verbRequiresAtLeastOneObjectSatisfying('Decompress APE to WAV', file, ext(ape)).

verbRequiresAtLeastOneObjectSatisfying('Decompress MP3 to WAV', file, ext(mp3)).

verbRequiresAtLeastOneObjectSatisfying('Decompress OGG to WAV', file, ext(ogg)).

verbRequiresAtLeastOneObjectSatisfying('Decompress OGM to WAV', file, ext(ogm)).

verbRequiresAtLeastOneObjectSatisfying('Compress WAV to MP3', file, ext(wav)).

verbRequiresAtLeastOneObjectSatisfying('Compress WAV to OGM', file, ext(wav)).

verbRequiresAtLeastOneObjectSatisfying('Compress WAV to FLAC', file, ext(wav)).

verbRequiresAtLeastOneObjectSatisfying('Delete directory', directory, anyDirectory).


verbRequiresAtLeastOneObjectSatisfying('Stop indexing this directory', directory, anyDirectory).

verbRequiresAtLeastOneObjectSatisfying('Browse directory', directory, anyDirectory).
verbRequiresAtLeastOneObjectSatisfying('Browse directory', program, supports('Browse directory')).

verbRequiresAtLeastOneObjectSatisfying('Rename directory (preserving concepts)', directory, anyDirectory).



verbRequiresAtLeastOneObjectSatisfying('Copy files to directory', directory, anyDirectory).
verbRequiresAtLeastOneObjectSatisfying('Copy files to directory', file, anyFile).

verbRequiresAtLeastOneObjectSatisfying('Move files to directory (preserving concepts)', directory, anyDirectory).
verbRequiresAtLeastOneObjectSatisfying('Move files to directory (preserving concepts)', file, anyFile).

verbRequiresAtLeastOneObjectSatisfying('Format partition', device, diskPartition).


verbRequiresAtLeastOneObjectSatisfying('Share and download files over the internet', program, supports('Share and download files over the internet')).

verbRequiresAtLeastOneObjectSatisfying('Rip CD', program, supports('Rip CD')).
verbRequiresAtLeastOneObjectSatisfying('Rip CD', device, cdDevice).



verbRequiresAtLeastOneObjectSatisfying('View files contained in RPM', file, ext(rpm)).

verbRequiresAtLeastOneObjectSatisfying('View files contained in DEB', file, ext(deb)).

verbRequiresAtLeastOneObjectSatisfying('View the dependencies of this RPM', file, ext(rpm)).

verbRequiresAtLeastOneObjectSatisfying('View the dependencies of this DEB', file, ext(deb)).

verbRequiresAtLeastOneObjectSatisfying('Encrypt', file, anyFile).

verbRequiresAtLeastOneObjectSatisfying('Mail files as attachments', file, anyFile).
verbRequiresAtLeastOneObjectSatisfying('Mail files as attachments', program, supports('Mail files as attachments')).


verbRequiresAtLeastOneObjectSatisfying('Extract audio track from OGM movie', file, ogmFile).

verbRequiresAtLeastOneObjectSatisfying('Extract video track from OGM movie', file, ogmFile).


verbRequiresAtLeastOneObjectSatisfying('Merge audio and video tracks', file, videoFile).
verbRequiresAtLeastOneObjectSatisfying('Merge audio and video tracks', file, audioFile).
verbRequiresAtLeastOneObjectSatisfying('Merge audio and video tracks', program, supports('Merge audio and video tracks')).

verbRequiresAtLeastOneObjectSatisfying('Install a new program', program, supports('Install a new program')).

verbRequiresAtLeastOneObjectSatisfying('Transfer files to and from an FTP site', program, supports('Transfer files to and from an FTP site')).

verbRequiresAtLeastOneObjectSatisfying('Rip DVD', program, supports('Rip DVD')).
verbRequiresAtLeastOneObjectSatisfying('Rip DVD', device, dvdDevice).

verbRequiresAtLeastOneObjectSatisfying('Close program automatically X minutes from now', program, anyProgram).



verbRequiresAtLeastOneObjectSatisfying('Browse the directory of this file', program, supports('Browse the directory of this file')).
verbRequiresAtLeastOneObjectSatisfying('Browse the directory of this file', file, anyFile).

verbRequiresAtLeastOneObjectSatisfying('Open terminal in the directory of this file', program, supports('Open terminal in the directory of this file')).
verbRequiresAtLeastOneObjectSatisfying('Open terminal in the directory of this file', file, anyFile).

verbRequiresAtLeastOneObjectSatisfying('View free space on disk', device, writableDisk).
verbRequiresAtLeastOneObjectSatisfying('View free space on disk', program, supports('View free space on disk')).

verbRequiresAtLeastOneObjectSatisfying('Partition disk', device, writableDisk).
verbRequiresAtLeastOneObjectSatisfying('Partition disk', program, supports('Partition disk')).

verbRequiresAtLeastOneObjectSatisfying('Create thumbnails', program, supports('Create thumbnails')).

verbRequiresAtLeastOneObjectSatisfying('Set mouse speed', device, mouseDevice).
verbRequiresAtLeastOneObjectSatisfying('Set mouse speed', program, supports('Set mouse speed')).

verbRequiresAtLeastOneObjectSatisfying('Configure mouse clicking behaviour', device, mouseDevice).
verbRequiresAtLeastOneObjectSatisfying('Configure mouse clicking behaviour', program, supports('Configure mouse clicking behaviour')).

verbRequiresAtLeastOneObjectSatisfying('Set mouse cursor theme', device, mouseDevice).
verbRequiresAtLeastOneObjectSatisfying('Set mouse cursor theme', program, supports('Set mouse cursor theme')).

verbRequiresAtLeastOneObjectSatisfying('Read newsgroups', program,  supports('Read newsgroups')).

verbRequiresAtLeastOneObjectSatisfying('Download a file from the internet with resume',program,  supports('Download a file from the internet with resume')).

verbRequiresAtLeastOneObjectSatisfying('Take a snapshot of your screen', program, supports('Take a snapshot of your screen')).  

verbRequiresAtLeastOneObjectSatisfying('Setup K3B', program,  supports('Setup K3B')).


verbRequiresAtLeastOneObjectSatisfying('Rename multiple files', file,  anyFile).
verbRequiresAtLeastOneObjectSatisfying('Rename multiple files', program,  supports('Rename multiple files')).

verbRequiresAtLeastOneObjectSatisfying('Install RPM', file,  ext(rpm)). % @@ slow? It scans linearly all rpms in the computer...
verbRequiresAtLeastOneObjectSatisfying('Install DEB', file,  ext(deb)).

verbRequiresAtLeastOneObjectSatisfying('View calendar and schedule', program, supports('View calendar and schedule')).

verbRequiresAtLeastOneObjectSatisfying('Eject disc',  device, ejectableDevice).

verbRequiresAtLeastOneObjectSatisfying('Create new text file',program,  supports('Create new text file')).

verbRequiresAtLeastOneObjectSatisfying('Create new PDF file', program, supports('Create new PDF file')).

verbRequiresAtLeastOneObjectSatisfying('View or terminate active processes', program, supports('View or terminate active processes')).

verbRequiresAtLeastOneObjectSatisfying('View CPU usage', program, supports('View CPU usage')).
verbRequiresAtLeastOneObjectSatisfying('View memory usage', program, supports('View CPU usage')).

verbRequiresAtLeastOneObjectSatisfying('Delete files', file,  anyFile).


verbRequiresAtLeastOneObjectSatisfying('Rename file', file,  anyFile).

verbRequiresAtLeastOneObjectSatisfying('Format disk', device, writableDisk).

verbRequiresAtLeastOneObjectSatisfying('Play video',  file, videoFile).
verbRequiresAtLeastOneObjectSatisfying('Play video', program, supports('Play video')).

verbRequiresAtLeastOneObjectSatisfying('Play movie with subtitle file', file,  videoFile).
verbRequiresAtLeastOneObjectSatisfying('Play movie with subtitle file', program, supports('Play movie with subtitle file')).
verbRequiresAtLeastOneObjectSatisfying('Play movie with subtitle file', file,  subtitleFile).

verbRequiresAtLeastOneObjectSatisfying('Play audio', file,  audioFile).
verbRequiresAtLeastOneObjectSatisfying('Play audio', program, supports('Play audio')).

verbRequiresAtLeastOneObjectSatisfying('Enqueue audio for playing', file,  audioFile).
verbRequiresAtLeastOneObjectSatisfying('Enqueue audio for playing', program, supports('Enqueue audio for playing')).

verbRequiresAtLeastOneObjectSatisfying('Enqueue video for playing', file,  videoFile).
verbRequiresAtLeastOneObjectSatisfying('Enqueue video for playing', program, supports('Enqueue video for playing')).

verbRequiresAtLeastOneObjectSatisfying('Compress files',  file, anyFile).
verbRequiresAtLeastOneObjectSatisfying('Compress files', program, supports('Compress files')).

verbRequiresAtLeastOneObjectSatisfying('Compress files separately', file,  anyFile).
verbRequiresAtLeastOneObjectSatisfying('Compress files separately', program, supports('Compress files separately')).

verbRequiresAtLeastOneObjectSatisfying('Extract archive',  file, compressedFile).
verbRequiresAtLeastOneObjectSatisfying('Extract archive', program, supports('Extract archive')).

verbRequiresAtLeastOneObjectSatisfying('View contents of archive',  file, compressedFile).
verbRequiresAtLeastOneObjectSatisfying('View contents of archive', program, supports('View contents of archive')).

verbRequiresAtLeastOneObjectSatisfying('Burn data CD', file,  anyFile).
verbRequiresAtLeastOneObjectSatisfying('Burn data CD', program, supports('Burn data CD')).
verbRequiresAtLeastOneObjectSatisfying('Burn data CD', device, cdBurningDevice).

verbRequiresAtLeastOneObjectSatisfying('Erase CD-RW', device, cdBurningDevice).
verbRequiresAtLeastOneObjectSatisfying('Erase CD-RW', program, supports('Erase CD-RW')).

verbRequiresAtLeastOneObjectSatisfying('View PDF', file,  ext(pdf)).
verbRequiresAtLeastOneObjectSatisfying('View PDF', program, supports('View PDF')).

verbRequiresAtLeastOneObjectSatisfying('Burn from ISO', program, supports('Burn from ISO')).
verbRequiresAtLeastOneObjectSatisfying('Burn from ISO', device, cdBurningDevice).
verbRequiresAtLeastOneObjectSatisfying('Burn from ISO', file,  ext(iso)).

verbRequiresAtLeastOneObjectSatisfying('Burn audio CD', file,  audioFile).
verbRequiresAtLeastOneObjectSatisfying('Burn audio CD', program, supports('Burn audio CD')).
verbRequiresAtLeastOneObjectSatisfying('Burn audio CD', device, cdBurningDevice).

verbRequiresAtLeastOneObjectSatisfying('Burn video CD',  file, videoFile).
verbRequiresAtLeastOneObjectSatisfying('Burn video CD', program, supports('Burn video CD')).
verbRequiresAtLeastOneObjectSatisfying('Burn video CD', device, cdBurningDevice).

verbRequiresAtLeastOneObjectSatisfying('Print text file',  file, plainTextFile).
verbRequiresAtLeastOneObjectSatisfying('Print text file', program, supports('Print text file')).
verbRequiresAtLeastOneObjectSatisfying('Print text file', device, printerDevice).

verbRequiresAtLeastOneObjectSatisfying('Print PDF',  file, ext(pdf)).
verbRequiresAtLeastOneObjectSatisfying('Print PDF', program, supports('Print PDF')).
verbRequiresAtLeastOneObjectSatisfying('Print PDF', device, printerDevice).

verbRequiresAtLeastOneObjectSatisfying('Burn CD from BIN and CUE files',  file, ext(cue)).
verbRequiresAtLeastOneObjectSatisfying('Burn CD from BIN and CUE files',  file, ext(bin)).
verbRequiresAtLeastOneObjectSatisfying('Burn CD from BIN and CUE files', device, cdBurningDevice).
verbRequiresAtLeastOneObjectSatisfying('Burn CD from BIN and CUE files', program, supports('Burn CD from BIN and CUE files')).

verbRequiresAtLeastOneObjectSatisfying('Burn CD from audio and CUE files',  file, ext(cue)).
verbRequiresAtLeastOneObjectSatisfying('Burn CD from audio and CUE files',  file, audioFile).
verbRequiresAtLeastOneObjectSatisfying('Burn CD from audio and CUE files', device, cdBurningDevice).
verbRequiresAtLeastOneObjectSatisfying('Burn CD from audio and CUE files', program, supports('Burn CD from audio and CUE files')).

verbRequiresAtLeastOneObjectSatisfying('Check mail', program, supports('Check mail')).

verbRequiresAtLeastOneObjectSatisfying('Compose a new email', program, supports('Compose a new email')).

verbRequiresAtLeastOneObjectSatisfying('Configure printer', device, printerDevice).

verbRequiresAtLeastOneObjectSatisfying('Change keyboard layout', device, keyboardDevice).
verbRequiresAtLeastOneObjectSatisfying('Change keyboard layout', program, supports('Change keyboard layout')).

verbRequiresAtLeastOneObjectSatisfying('Change keyboard repeat speed', device, keyboardDevice).
verbRequiresAtLeastOneObjectSatisfying('Change keyboard repeat speed', program, supports('Change keyboard repeat speed')).

verbRequiresAtLeastOneObjectSatisfying('Set sound volume', device, audioCardDevice).
verbRequiresAtLeastOneObjectSatisfying('Set sound volume', program, supports('Set sound volume')).

verbRequiresAtLeastOneObjectSatisfying('Edit plain text file', program, supports('Edit plain text file')).
verbRequiresAtLeastOneObjectSatisfying('Edit plain text file', file, plainTextFile).

verbRequiresAtLeastOneObjectSatisfying('Shutdown', device, computerDevice).
verbRequiresAtLeastOneObjectSatisfying('Login as different user', device, computerDevice).
verbRequiresAtLeastOneObjectSatisfying('Hybernate', device, computerDevice).

verbRequiresAtLeastOneObjectSatisfying('Edit DOC file', file,  ext(doc)).
verbRequiresAtLeastOneObjectSatisfying('Edit DOC file', program, supports('Edit DOC file')).

verbRequiresAtLeastOneObjectSatisfying('Edit SXW file', file,  ext(sxw)).
verbRequiresAtLeastOneObjectSatisfying('Edit SXW file', program, supports('Edit SXW file')).

verbRequiresAtLeastOneObjectSatisfying('Browse the internet', program, supports('Browse the internet')).

verbRequiresAtLeastOneObjectSatisfying('View HTML file', program, supports('View HTML file')).
verbRequiresAtLeastOneObjectSatisfying('View HTML file', file,  htmlFile).

verbRequiresAtLeastOneObjectSatisfying('View DVI file',  file, ext(dvi)).
verbRequiresAtLeastOneObjectSatisfying('View DVI file', program, supports('View DVI file')).

verbRequiresAtLeastOneObjectSatisfying('Create new SXW file', program, supports('Create new SXW file')).


verbRequiresAtLeastOneObjectSatisfying('Edit picture', program, supports('Edit picture')).
verbRequiresAtLeastOneObjectSatisfying('Edit picture', file,  pictureFile).

verbRequiresAtLeastOneObjectSatisfying('Print picture', program, supports('Print picture')).
verbRequiresAtLeastOneObjectSatisfying('Print picture',  file, pictureFile).
verbRequiresAtLeastOneObjectSatisfying('Print picture', device, printerDevice).

verbRequiresAtLeastOneObjectSatisfying('View picture', program, supports('View picture')).
verbRequiresAtLeastOneObjectSatisfying('View picture',  file, pictureFile).

verbRequiresAtLeastOneObjectSatisfying('Change resolution', device, monitorDevice).

verbRequiresAtLeastOneObjectSatisfying('Calculate size', file,  anyFile).

verbRequiresAtLeastOneObjectSatisfying('View print queue', device, printerDevice).
verbRequiresAtLeastOneObjectSatisfying('View print queue', program, supports('View print queue')).

verbRequiresAtLeastOneObjectSatisfying('Restore file from trashcan', program, supports('Restore file from trashcan')).

verbRequiresAtLeastOneObjectSatisfying('Empty trashcan', program, supports('Empty trashcan')).

verbRequiresAtLeastOneObjectSatisfying('Move to trashcan', file,  anyFile).

verbRequiresAtLeastOneObjectSatisfying('Edit TM file', program, supports('Edit TM file')).
verbRequiresAtLeastOneObjectSatisfying('Edit TM file',  file, ext(tm)).

verbRequiresAtLeastOneObjectSatisfying('Edit 3D picture', program, supports('Edit 3D picture')).
verbRequiresAtLeastOneObjectSatisfying('Edit 3D picture', file, ext(blend)).


audioFileNotWaw(X):-
	audioFile(X),
	\+ext(wav, X).


anyFile(_).
anyIdea(_).
anyVerb(_).
anyProgram(_).
anyDirectory(_).


ogmFile(X):-
	ext(ogg, X).
ogmFile(X):-
	ext(ogm, X).




subtitleFile(F) :-
     ext(sub,F).
subtitleFile(F) :-
     ext(srt,F).

pictureFile(F) :-
	ext(jpg,F).
pictureFile(F) :-
	ext(jpeg,F).
pictureFile(F) :-
	ext(	png,F).
pictureFile(F) :-
	ext(	 bmp,F).
pictureFile(F) :-
	ext(	 ppm,F).
pictureFile(F) :-
	ext(	 gif,F).
pictureFile(F) :-
	ext(	 xpm,F).

audioFile(F) :-
	ext(	 mp3,F).
audioFile(F) :-
	ext(	 ogg,F).
audioFile(F) :-
	ext(	 ogm,F).
audioFile(F) :-
	ext(	mpc,F).
audioFile(F) :-
	ext(	 flac,F).
audioFile(F) :-
	ext(	 ape,F).
audioFile(F) :-
	ext(	 rm,F).
audioFile(F) :-
	ext(	 wma,F).
audioFile(F) :-
	ext(	 wav,F).


videoFile(F):-
	 ext(avi, F).
videoFile(F):-
	 ext(mpeg, F).
videoFile(F):-
	 ext(mpg, F).
videoFile(F):-
	 ext(	 asf,F).
videoFile(F):-
	 ext(	 ogg,F).
videoFile(F):-
	 ext(	 ogm,F).
videoFile(F):-
	 ext(	 rm,F).
videoFile(F):-
	 ext(	 wmv,F).

multimediaFile(F) :-
	 ext(	 audioFile,F).
multimediaFile(F) :-
	 ext(	 videoFile,F).

compressedFile(X):-
	ext(	 zip,X).
compressedFile(X):-
	ext(	 rar,X).
compressedFile(X):-
	ext(	 gz,X).
compressedFile(X):-
	ext(	 bz2,X).
compressedFile(X):-
	ext(	 tar,X).
compressedFile(X):-
	ext(	 arj,X).
compressedFile(X):-
	ext(	 ace,X).

readableByHumans(F) :-
	 ext(plainTextFile,F).
readableByHumans(F) :-
	 ext(pdf,F).
readableByHumans(F) :-
	 ext( ps,F).
readableByHumans(F) :-
	 ext( eps,F).
readableByHumans(F) :-
	 ext( doc,F).
readableByHumans(F) :-
	 ext( sxw,F).
readableByHumans(F) :-
	 ext( dvi,F).

postscriptFile(F):-
	ext(	 ps,F).
postscriptFile(F):-
	ext(	 eps,F).

plainTextFile(F) :-
	 ext(	 txt,F).
plainTextFile(F) :-
	 ext(spec,F).
plainTextFile(F) :-
	 ext(	 tex,F).
plainTextFile(F) :-
	 ext(	 srt,F).
plainTextFile(F) :-
	 ext(	 sub,F).
plainTextFile(F) :-
	 ext(	 pl,F).
plainTextFile(F) :-
	 ext(	 pas,F).
plainTextFile(F) :-
	 ext(	 bas,F).
plainTextFile(F) :-
	 ext(	 cpp,F).
plainTextFile(F) :-
	 ext(	 h,F).
plainTextFile(F) :-
	 ext(	 hpp,F).
plainTextFile(F) :-
	 ext(	 cc,F).
plainTextFile(F) :-
	 ext(	 bas,F).
plainTextFile(F) :-
	 ext(	 c,F).
plainTextFile(F) :-
	 ext(	 htm,F).
plainTextFile(F) :-
	 ext(	 html,F).
plainTextFile(F) :-
	 ext(	 php,F).

htmlFile(F) :-
	 ext(	 htm,F).
htmlFile(F) :-
	 ext(	 html,F).

cppSourceFile(F) :-
	 ext(	 cc,F).
cppSourceFile(F) :-
	 ext(	 cpp,F).
cppSourceFile(F) :-
	ext(	 h,F).

binaryInstallPackage(F) :-
	ext(	 rpm,F).
binaryInstallPackage(F) :-
	ext(	 deb,F).

% END

supports( 'Set mouse speed', 'Kde control center').
supports( 'Change keyboard layout', 'Kde control center').
supports( 'Change keyboard repeat speed', 'Kde control center').
supports( 'Set mouse cursor theme', 'Kde control center').
supports( 'Configure mouse clicking behaviour', 'Kde control center').

supports( 'Take a snapshot of your screen', 'Ksnapshot').
supports( 'Take a snapshot of your screen', 'Gnome screenshot').

supports( 'Share and download files over the internet', 'Xmule').

supports( 'Rename multiple files', 'KRename').

supports( 'Read newsgroups', 'Knode').

supports( 'Read newsgroups', 'Pan').

supports( 'View print queue', 'KJobViewer').
supports( 'Set sound volume', 'Kmix').
supports( 'Set sound volume', 'Aumix').
supports( 'Move to trashcan', 'Trashcan').
supports( 'Empty trashcan', 'Trashcan').
supports( 'Restore file from trashcan', 'Trashcan').

supports( 'View DVI file', 'Xdvi').
supports( 'View DVI file', 'Kdvi').


supports( 'View free space on disk', 'Kdiskfree').

supports( 'Partition disk', 'Kparted').
supports( 'Partition disk', 'QTparted').
supports( 'Partition disk', 'Diskdrake').

supports( 'Install a new program', 'Rpmdrake').
supports( 'Install a new program', 'Synaptic').

supports( 'Burn from ISO', 'K3b').
supports( 'Erase CD-RW', 'K3b').
supports( 'Burn from ISO', 'K3b').
supports( 'Burn data CD', 'K3b').
supports( 'Burn audio CD', 'K3b').
supports( 'Burn video CD', 'K3b').
supports( 'Setup K3B', 'K3b').
supports( 'Burn CD from BIN and CUE files', 'K3b').
supports( 'Burn CD from audio and CUE files', 'K3b').

supports( 'Burn from ISO', 'Xcdroast').
supports( 'Burn data CD', 'Xcdroast').


supports( 'Play audio', 'Xmms').
supports( 'Enqueue audio for playing', 'Xmms').


supports('Browse the directory of this file', 'Nautilus').

supports('Open terminal in the directory of this file', 'Gnome terminal').

supports( 'Edit picture', 'The gimp').
supports( 'View picture', 'The gimp').
supports( 'Print picture', 'The gimp').

supports( 'Print picture', 'Kuickshow').
supports( 'View picture', 'Kuickshow').

supports( 'View picture', 'Eye of Gnome').
supports( 'Print picture', 'Eye of Gnome').

supports( 'View picture', 'gThumb').

supports( 'Play audio', 'Totem').
supports( 'Enqueue audio for playing', 'Totem').
supports( 'Enqueue video for playing', 'Totem').

supports( 'Play audio', 'RealPlayer').
supports( 'Play video', 'RealPlayer').


supports( 'Play audio', 'Xine').
supports( 'Enqueue audio for playing', 'Xine').
supports( 'Enqueue video for playing', 'Xine').

supports( 'Play movie with subtitle file', 'Mplayer').
supports( 'Play video', 'Mplayer').



supports( 'Play video', 'Xine').
supports( 'Play movie with subtitle file', 'Xine').

supports( 'Play video', 'Gxine').

supports( 'Play video', 'Totem').

supports( 'Download a file from the internet with resume', 'Kget').
supports( 'Download a file from the internet with resume', 'Opera').
supports( 'Download a file from the internet with resume', 'Downloader for X').

supports( 'Edit plain text file', 'Emacs').
supports( 'Create new text file', 'Emacs').
supports( 'Print text file', 'Emacs').
supports( 'Check mail', 'Emacs').
supports( 'Compose a new email', 'Emacs').

supports( 'Edit plain text file', 'Vim').
supports( 'Create new text file', 'Vim').
supports( 'Print text file', 'Vim').
supports( 'Edit plain text file', 'Gedit').
supports( 'Print text file', 'Gedit').
supports( 'Create new text file', 'Gedit').
supports( 'Edit plain text file', 'OpenOfficeWriter').
supports( 'Create new SXW file', 'OpenOfficeWriter').
supports( 'Create new PDF file', 'OpenOfficeWriter').
supports( 'Edit SXW file', 'OpenOfficeWriter').
supports( 'Edit DOC file', 'OpenOfficeWriter').
supports( 'View HTML file', 'OpenOfficeWriter').
supports( 'Edit DOC file', 'abiWord').
supports( 'View PDF', 'Xpdf').
supports( 'View PDF', 'Gpdf').
supports( 'View PDF', 'Kghostview').
supports( 'Print PDF', 'Kghostview').

supports( 'Print PDF', 'Kprinter').
supports( 'Print picture', 'Kprinter').

supports('Transfer files to and from an FTP site', 'KBear').
supports('Transfer files to and from an FTP site', 'gFTP').

supports( 'Print PDF', 'Xpdf').
supports( 'Print PDF', 'Gpdf').

supports( 'Check mail', 'Kmail').
supports( 'Compose a new email', 'Kmail').
supports( 'Mail files as attachments', 'Kmail').

supports( 'Check mail', 'Sylpheed').
supports( 'Compose a new email', 'Sylpheed').
supports( 'Mail files as attachments', 'Sylpheed').

supports( 'Check mail', 'Mozilla').
supports( 'View HTML file', 'Mozilla').

supports( 'Check mail', 'Evolution').
supports( 'Compose a new email', 'Evolution').

supports( 'Rip CD', 'Kaudiocreator').
supports( 'Rip CD', 'ripperX').


supports( 'View or terminate active processes', 'Kde system guard').
supports( 'View free space on disk', 'Gnome system monitor').
supports( 'View or terminate active processes', 'Gnome system monitor').
supports( 'View CPU usage', 'Kde system guard').
supports( 'View CPU usage', 'Gnome system monitor').

supports( 'View memory usage', 'Gnome system monitor').
supports( 'View SWAP use', 'Gnome system monitor').

supports( 'Browse the internet', 'Mozilla').
supports( 'Browse the internet', 'Mozilla Firebird').
supports( 'Browse the internet', 'Konqueror').

supports( 'Browse the internet', 'Opera').
supports( 'View HTML file', 'Opera').

supports( 'Browse the internet', 'Galeon').
supports( 'View HTML file', 'Galeon').

supports( 'Browse the internet', 'Epiphany').


supports( 'View HTML file', 'Konqueror').
supports('Browse the directory of this file', 'Konqueror').

supports( 'Extract archive', 'Ark').
supports( 'Compress files', 'Ark').
supports( 'View contents of archive', 'Ark').


supports( 'Compress files separately', 'Bzip2').
supports( 'Compress files separately', 'Zip').
supports( 'Compress files separately', 'Gzip').
supports( 'Compress files separately', 'Rar').

supports( 'Extract archive', 'FileRoller').
supports( 'Compress files', 'FileRoller').
supports( 'View contents of archive', 'FileRoller').
supports( 'View calendar and schedule', 'Korganizer').
supports( 'View calendar and schedule', 'Evolution').

supports( 'Edit TM file', 'TeXmacs').

supports( 'Edit 3D picture', 'Blender').

time('Now').
time('15 seconds from now').
time('1 minute from now').
time('2 minutes from now').
time('4 minutes from now').
time('5 minutes from now').
time('10 minutes from now').
time('15 minutes from now').
time('20 minutes from now').
time('30 minutes from now').
time('45 minutes from now').
time('1 hour from now').
time('1 hour 30 minutes from now').
time('1 hour 45 minutes from now').
time('8 hours from now').
time('8 hours 45 minutes from now').


device(X):-
	 cdBurningDevice(X).
device(X):-
	 computerDevice(X).
device(X):-
	 ejectableDevice(X).
device(X):-
	 printerDevice(X).
device(X):-
	 keyboardDevice(X).
device(X):-
	 audioCardDevice(X).
device(X):-
	 writableDisk(X).
device(X):-
	 monitorDevice(X).
device(X):-
	 mouseDevice(X).
device(X):-
	 diskPartition(X).

program( P):-
	supports(_, P).

directory(D):-
	segusoDir(D).% REGION this is a quick trick to add all verbs with a single instruction. Otherwise I would have written
%
%        verb(v1)
%        verb(v2)
%
%    etc.
verb(V):-
	verbRequiresAtLeastOneObjectSatisfying(V, _, _).
% END

% REGION But I must add the zeroary verbs explicitely, because verbRequiresAtLeastOneObjectSatisfying/3
%          is not defined for them :-)


verb( 'Refresh file list').
verb( 'Import directory and all its files').
verb( 'Import directory and all its files (recursively)').
%verb( 'Show "GO" button besides the verbs').
verb( 'Show/hide quick-search boxes').
verb( 'Toggle scrolling buttons always visible').
%verb( 'Enable/disable strong narrowing').
verb( 'Sort files by name').
verb( 'Sort files by size').
verb( 'Sort files by date').
verb( 'Show files/programs/actions/devices when no concept is selected').
verb( 'Do not show files/programs/actions/devices when no concept is selected').
verb( 'Show/hide directories below the files').
%verb( 'Deselect all'). % impossible to have this as a verb :-)
verb( 'Show/hide file sizes').
verb( 'Show/hide file dates').
verb( 'Show/hide concepts below files').
verb( 'Show/hide concepts below programs').
verb( 'Quit segusoLand').
verb( 'Save database now').
verb( 'Show/hide tooltips').
verb( 'Show/hide log').
%verb( 'Show/hide advanced controls').
verb( 'Show scheduled tasks').

verb( 'Create a new concept for organizing files').
verb( 'Delete a concept').
verb( 'Automatically deselect concepts when a nonconcept is selected').

%verb( 'Manage the directories watched by segusoLand'). %  this could be removed if I add a directories panel



%related('Networking', verb, 'Check mail').
% END

tooltipsAreEnabled. % default


theItemIsFiled(Item, Kind):-
	idea(Kind, Idea),
	related(Idea, Kind, Item).

computerDevice('My Computer').  
printerDevice('My printer').
ejectableDevice('My CD writer').
ejectableDevice('My CD reader').
ejectableDevice('Floppy Drive').
cdBurningDevice('My CD writer').
writableDisk('Floppy Drive').
writableDisk('Primary Hard disk').
writableDisk('Secondary Hard disk').
keyboardDevice('My Keyboard').
mouseDevice('My Mouse').
audioCardDevice('My sound card').
monitorDevice('My monitor').
cdDevice('My CD writer').
cdDevice('My CD reader').
dvdDevice('My CD reader').
diskPartition('Partition 0 of Primary Hard Disk').
diskPartition('Partition 1 of Primary Hard Disk').
diskPartition('Partition 2 of Primary Hard Disk').
diskPartition('Partition 0 of Secondary Hard Disk').
diskPartition('Partition 1 of Secondary Hard Disk').
diskPartition('Partition 2 of Secondary Hard Disk').

shellName('Xpdf', xpdf).
shellName('Sylpheed', sylpheed).
shellName('RealPlayer', realplay).
shellName('Galeon', galeon).
shellName('Xdvi', xdvi).
shellName('Kdvi', kdvi).
shellName('Kparted', kparted).
shellName('QTparted', qtparted).
shellName('Rpmdrake', rpmdrake).
shellName('Diskdrake', diskdrake).
shellName('Kaudiocreator', kaudiocreator).
shellName('ripperX', ripperX).
shellName('Ark', ark).
shellName('FileRoller', 'file-roller').
shellName('The gimp', gimp).
shellName('Korganizer', korganizer).
shellName('Evolution', evolution).
shellName('Kmail', kmail).
shellName('Kghostview', kghostview).
shellName('Gpdf', gpdf).
shellName('Mozilla', mozilla).
shellName('Emacs', emacs).
shellName('Vim', vim).
shellName('Gedit', gedit).
shellName('Totem', totem).
shellName('Xmms', xmms).
shellName('Xine' , xine).
shellName('Mplayer', mplayer).
shellName('Gxine', gxine).

shellName('Konqueror', konqueror).
shellName('Nautilus', nautilus).
shellName('Kprinter', kprinter).
shellName('Aumix', aumix).
shellName('Kmix', kmix).
shellName('KJobViewer', kjobviewer).
shellName('Kuickshow', kuickshow).
shellName('Eye of Gnome', eog).
shellName('gThumb', gthumb).
shellName('Epiphany', epiphany).
shellName('Gnome system monitor', 'gnome-system-monitor').
shellName('Gnome screenshot', 'gnome-panel-screenshot').
shellName('Gnome terminal', 'gnome-terminal').
shellName('gFTP', 'gftp').
shellName('Blender', 'blender').


pair(X, Y, L):-
	member(X, L),
	member(Y, L),
	X\=Y.


onVerbSelected(V/*in*/) :-  
	selected(verb, V),
	writeln('>>>ERROR: verb already selected'),
	throw(should-never-happen).
onVerbSelected(V):-
	\+selected(verb, V),
	assert(selected(verb, V)),
	(autoDeselectConcepts->deselectAllConcepts; true),
	% REGION see if I must restore the minimized panels
	(verbRequiresAtLeastOneObjectSatisfying(V, device, _)->
	    (widgetShow(mainObjectVBox(device)),
		retractall(panelStateMinimized(device)),
		widgetDisable(putToDropdownButton(device)))
	; true),
	(verbRequiresAtLeastOneObjectSatisfying(V, directory, _)->
	    (widgetShow(mainObjectVBox(directory)),
		retractall(panelStateMinimized(directory)),
		widgetDisable(putToDropdownButton(directory)))
	; true),
	(verbRequiresAtLeastOneObjectSatisfying(V, file, _)->
	    (widgetShow(mainObjectVBox(file)),
		retractall(panelStateMinimized(file)),
		widgetDisable(putToDropdownButton(file)))
	; true),
	(verbRequiresAtLeastOneObjectSatisfying(V, program, _)->
	    (widgetShow(mainObjectVBox(program)),
		retractall(panelStateMinimized(program)),
		widgetDisable(putToDropdownButton(program)))
	; true),
	% END
	updateItemsAfterSelectionChanged /* reads selected/2*/,
	updateDeleteIdeasToolbarButtons,
	checkListBoxDefeatBug(checkListBoxOfKind(verb)),
	addCurrentlySelectedConceptsToRecentConcepts.

updateRedArrows(VisibleVerbs):-
	updateVerbArrow(VisibleVerbs),
	updateGlobalGoArrow(VisibleVerbs),
	updateConceptArrows,
	updateObjectArrow(file, VisibleVerbs),
	updateObjectArrow(device, VisibleVerbs),
	updateObjectArrow(program, VisibleVerbs),
	updateObjectArrow(directory, VisibleVerbs),
	updateObjectArrow(time, VisibleVerbs).
updateConceptArrows:-
	updateLeftConceptArrow,
	updateRightConceptArrow,
	updateRecentConceptsArrow,
	updateRecentActionsArrow.
updateRightConceptArrow:-
	theRightConceptArrowIsVisible(big),
	imageSet(arrowImageRight(idea), 'img/redarrow.gif'),
	widgetSetTooltip(arrowEventBoxRight(idea), 'The two red arrows mean you can now choose a concept or change the letter.').
updateRightConceptArrow:-
	theRightConceptArrowIsVisible(small),
	imageSet(arrowImageRight(idea), 'img/empty.png'),
	findall(I, selected(idea-_, I), L),
	length(L, Len),
	selected(idea-_, SelectedIdea),
	concat_atom(['Don\'t forget you can select more than one concept. Currently you have selected only one: "', SelectedIdea, '". Example of multiple concept selection: send+email, configure+kde, setup+mouse+cursor, play+good+music, etc.'],
		    TextOne),
	(Len = 1 -> widgetSetTooltip(arrowEventBoxRight(idea), TextOne );
	    widgetSetTooltip(arrowEventBoxRight(idea), 'The green arrow reminds you you can select another concept if you like.')).
updateRightConceptArrow:-
	\+theRightConceptArrowIsVisible(_),
	imageSet(arrowImageRight(idea), 'img/empty.png'),
	widgetSetTooltip(arrowEventBoxRight(idea), '').
updateRecentConceptsArrow:-
	\+theRecentConceptsArrowIsVisible,
	widgetHide(recentConceptsRedArrow).
updateRecentConceptsArrow:-
	theRecentConceptsArrowIsVisible,
	widgetShow(recentConceptsRedArrow).
updateRecentActionsArrow:-
	\+theRecentActionsArrowIsVisible,
	widgetHide(recentConceptsRedArrowRight).
updateRecentActionsArrow:-
	theRecentActionsArrowIsVisible,
	widgetShow(recentConceptsRedArrowRight).
updateLeftConceptArrow:-
	theLeftConceptArrowIsVisible(big),
	imageSet(arrowImage(idea), 'img/redarrow.gif'),
	(theRightConceptArrowIsVisible(big)->
	    widgetSetTooltip(arrowEventBox(idea), 'The two red arrows mean you can now choose a concept or change the letter.');
	    widgetSetTooltip(arrowEventBox(idea), 'You must select a letter now.')).
updateLeftConceptArrow:-
	theLeftConceptArrowIsVisible(small),
	imageSet(arrowImage(idea), 'img/empty.png'),
	findall(I, selected(idea-_, I), L),
	length(L, Len),
	selected(idea-_, SelectedIdea),
	concat_atom(['Don\'t forget you can select more than one concept. Currently you have selected only one: "', SelectedIdea, '". Example of multiple concept selection: send+email, configure+kde, setup+mouse+cursor, play+good+music, etc.'],
		    TextOne),
	(Len = 1 -> widgetSetTooltip(arrowEventBox(idea), TextOne );
	    widgetSetTooltip(arrowEventBox(idea), 'The green arrow reminds you you can select another concept if you like.')).

updateLeftConceptArrow:-
	\+theLeftConceptArrowIsVisible(_),
	imageSet(arrowImage(idea), 'img/empty.png'),
	widgetSetTooltip(arrowEventBox(idea), '').

theLeftConceptArrowIsVisible(big):-
	forall(selected(K, _), K=time).
theRightConceptArrowIsVisible(big):-
	forall(selected(K, _), K=time),
	selectedLetter(_).
theRecentConceptsArrowIsVisible:-
	forall(selected(K, _), K=time),
	recentConcept(_).
theRecentActionsArrowIsVisible:-
	forall(selected(K, _), K=time),
	recentAction(_, _).
	

updateVerbArrow(VisibleVerbs):-
	theVerbArrowIsVisible(VisibleVerbs)->imageSet(arrowImage(verb), 'img/redarrow.gif'); imageSet(arrowImage(verb), 'img/empty.png').
updateObjectArrow(Kind, VisibleVerbs):-
	Kind\=verb,
	theObjectArrowIsVisible(Kind, VisibleVerbs, big),
	imageSet(arrowImage(Kind), 'img/redarrow.gif').
updateObjectArrow(Kind, VisibleVerbs):-
	Kind\=verb,
	theObjectArrowIsVisible(Kind, VisibleVerbs, small),
	imageSet(arrowImage(Kind), 'img/empty.png').
updateObjectArrow(Kind, VisibleVerbs):-
	Kind\=verb,
	\+theObjectArrowIsVisible(Kind, VisibleVerbs, _),
	imageSet(arrowImage(Kind), 'img/empty.png').

updateGlobalGoArrow(VisibleVerbs):-
	(theGlobalGoArrowIsVisible(VisibleVerbs)->
	    (widgetShow(globalGoRedArrow), setGlobalGoButtonTooltip(VisibleVerbs), widgetShow(globalGoRedArrowLeft), widgetShow(globalGoButton));
	    (widgetHide(globalGoButton),widgetHide(globalGoRedArrow), widgetHide(globalGoRedArrowLeft))).

setGlobalGoButtonTooltip(VisibleVerbs):-
	VisibleVerbs = [V],
	computeMissingText(V, Missing),
	concat_atom(['You can now press this button to execute the action "', V, '", or you can select some more ', Missing,'times from the panels below.'], Text),
	widgetSetTooltip(globalGoButton, Text).
computeMissingText(V, Text):-
	findall(K, verbRequiresAtLeastOneObjectSatisfying(V, K, _), Kinds),
	sort(Kinds, KindsSorted),
	fromListOfAtomsToReadableText(KindsSorted, Text).
fromListOfAtomsToReadableText(L, Text):-
	fromListOfAtomsToReadableTextAux('', L, Text).
fromListOfAtomsToReadableTextAux(CurrentText, List, NewText):-
	List=[],
	NewText = CurrentText.
fromListOfAtomsToReadableTextAux(CurrentText, List, NewText2):-
	List\=[],
	List = [H|T],
	readableNameOf(H, R),
	concat_atom([CurrentText,  R, '/'] , NewText),
	fromListOfAtomsToReadableTextAux(NewText, T, NewText2).
readableNameOf(A, T):-
	concat_atom([A, 's'],T). % @@ only good for english

theVerbArrowIsVisible(VisibleVerbs):-
	length(VisibleVerbs, L),
	L>1,
	selected(Something,_),
	Something \=time.



theGlobalGoArrowIsVisible(VisibleVerbs):-
	theGlobalGoButtonIsVisible(VisibleVerbs).

theGlobalGoButtonIsVisible(VisibleVerbs):-
	VisibleVerbs = [V],
	allRequiredItemsAreSelectedOrObvious(V),
	\+thereIsASelectedElementThatMakesNoSenseWithTheVerb(V).

% allRequiredItemsAreSelected means that all the required arguments are selected. It still may be that
% some not required argument is selected too.
allRequiredItemsAreSelected(V):-
	forall(verbRequiresAtLeastOneObjectSatisfying(V, K, P),
	       (selected(K, O), call(P, O))),
	selected(time, _). % all verbs require a time, so this is hard coded.
allRequiredItemsAreSelectedOrObvious(V):-
	forall(verbRequiresAtLeastOneObjectSatisfying(V, K, P),
	       ( thereIsExactlyOneVisibleItemOfKind(K) ; (selected(K, O), call(P, O)))),
	selected(time, _). % all verbs require a time, so this is hard coded.
% END

theObjectArrowIsVisible(time, VisibleVerbs, big):-
	\+selected(time, _),
	VisibleVerbs=[_].
theObjectArrowIsVisible(time, VisibleVerbs, big):-
	\+selected(time, _),
	VisibleVerbs\=[_],
	theVerbArrowIsVisible(VisibleVerbs).
theObjectArrowIsVisible(K, VisibleVerbs, big):-
	K\=time,
	VisibleVerbs=[SingleVerb],
	\+thereIsExactlyOneVisibleItemOfKind(K),
	verbRequiresAtLeastOneObjectSatisfying(SingleVerb, K, P),
	forall(selected(K, O), \+call(P, O)).
theObjectArrowIsVisible(K, VisibleVerbs, small):-
	K\=time,
	VisibleVerbs=[SingleVerb],
	\+thereIsExactlyOneVisibleItemOfKind(K),
	verbRequiresAtLeastOneObjectSatisfying(SingleVerb, K, P),
	selected(K, O), call(P, O),
	visibleObject(K, VisibleUnselectedObject),
	\+selected(K, VisibleUnselectedObject).
theObjectArrowIsVisible(K, VisibleVerbs, big):-
	K\=time,
	length(VisibleVerbs, L),
	L>1,
	\+selected(K, _),
	findall(O, visibleObject(K, O), VisibleObjectsOfMyKind),
	length(VisibleObjectsOfMyKind, NumberOfVisibleObjectsOfMyKind),
	NumberOfVisibleObjectsOfMyKind>1,
	selected(Something,_),
	Something \=time.

thereIsExactlyOneVisibleItemOfKind(K):-
	findall(O, visibleObject(K, O), VisibleObjectsOfKind),
	sort(VisibleObjectsOfKind, VisibleObjectsOfKindWithDuplicatesRemoved),
	VisibleObjectsOfKindWithDuplicatesRemoved = [_SingleObject].


	
	
decideObjectVisibility(O, Kind, VisibleVerbs):-
	theObjectIsVisible(O, Kind, VisibleVerbs),
	assert(visibleObject(Kind, O)),
	(selected(Kind, O)->State = selected; State=deselected),
	checkListBoxSetItemState(checkListBoxOfKind(Kind), O, State).
decideObjectVisibility(O, Kind, VisibleVerbs):-
	\+theObjectIsVisible(O, Kind, VisibleVerbs),
	checkListBoxSetItemState(checkListBoxOfKind(Kind), O, hidden).

theObjectIsVisible(_O, K, _VisibleVerbs):-
	(K=time ; K=verb),
	writeln(theObjectIsVisible-err),
	throw(err).
theObjectIsVisible(/*in*/O,K, _):-
	selected(K, O).
theObjectIsVisible(/*in*/O, K, VisibleVerbs):-
	theNameMatchesFilter(K, O),
	forall( selected(idea-K, I), 
		related(I, K, O)),
	member(V, VisibleVerbs), % it is required by some visible verb
	verbRequiresAtLeastOneObjectSatisfying(V, K, P),
	call(P, O).


theNameMatchesFilter(K, _):-
	entryGetText(containingTextEntry(K), T),
	T=''.
theNameMatchesFilter(K, O):-
	K\=file,
	entryGetText(containingTextEntry(K), T),
	T\='',
	downcase_atom(T, T2),
	downcase_atom(O, O2),
	sub_atom(O2, _, _, _, T2).
theNameMatchesFilter(file, O):-
	entryGetText(containingTextEntry(file), T),
	T\='',
	downcase_atom(T, T2),
	file_base_name(O, BaseName),
	downcase_atom(BaseName, O2),
	sub_atom(O2, _, _, _, T2).
	
onVerbDeselected(VV/*in*/):-
	retract(selected(verb, VV)),
	ifItIsNowFilteredAwayRemoveItFromList(verb, VV),
	% REGION see if I must enable the hide panel button
	% @@ this code would not work if more that one verb can be selected at once.
	widgetEnable(putToDropdownButton(device)),
	widgetEnable(putToDropdownButton(directory)),	
	widgetEnable(putToDropdownButton(file)),
	widgetEnable(putToDropdownButton(program)),
	% END

	((showPanelsWhenNoConceptIsSelected ; (selected(Q, _), Q\=time))->
	    updateItemsAfterSelectionChanged
	;  updateRedArrows([])),/*optimization*/


	checkListBoxDefeatBug(checkListBoxOfKind(verb)),
	updateDeleteIdeasToolbarButtons,
	hidePanelsIfNothingIsSelected.



understandWhatTheUserWantsToDoAnalyzingSelectedAndVisibleItems(_):-
	visibleVerb(V1),
	visibleVerb(V2),
	V1\=V2,
	writeln(errorMoreThanOneVisibleVerb),
	throw(errorMoreThanOneVisibleVerb).
understandWhatTheUserWantsToDoAnalyzingSelectedAndVisibleItems(/*out*/A):-
	findall(X, visibleVerb(X), [V]),
	(verbRequiresAtLeastOneObjectSatisfying(V, file, _)->
	    (selected(file, _) ->
		findall(X, selected(file, X), F) ;
		findall(X, visibleObject(file, X), F))	;
	    F = []),
	(verbRequiresAtLeastOneObjectSatisfying(V, program, _)->
	    (selected(program, _) ->
		findall(X, selected(program, X), P) ;
		findall(X, visibleObject(program, X), P));
	    P = []),
	(verbRequiresAtLeastOneObjectSatisfying(V, device, _)->
	    (selected(device, _) ->
		findall(X, selected(device, X), D) ;
		findall(X, visibleObject(device, X), D));
	    D = []),
	(verbRequiresAtLeastOneObjectSatisfying(V, directory, _)->
	    (selected(directory, _) ->
		findall(X, selected(directory, X), Dir) ;
		findall(X, visibleObject(directory, X), Dir));
	    Dir = []),
	findall(X, selected(time, X), T),
	Action = [V, F, P, D, Dir, T],
	sortActionSoThatItsRepresentationBecomesUnique(Action, A).

onTimer(Tasks, NewTasks):-
	% REGION freeze current time for all the length of the computation of onTimer
	get_time(CurrentTime),
	% END
	onTimerAux(CurrentTime, [], Tasks, NewTasks).
onTimerAux(_, Before, [], Before).
onTimerAux(CurrentTime, Before, [H|T], T2):-
	H=(Time, _),
	Time>CurrentTime, % it is still to early...
	!, onTimerAux(CurrentTime, [H|Before]/*keep H*/, T, T2).
onTimerAux(CurrentTime, Before, [H|T], T2):-
	H=(Time, A),
	Time =< CurrentTime, % it is time to execute the action...
	executeActionNow(A),
	!, onTimerAux(CurrentTime, Before/*drop H*/, T, T2).
onTimerAux(_, _, _, _):-
	throw(this-should-never-happen),
	gtrace.

removeTimeFromAction([V, P, F, D, Di, _T], [V, P, F, D, Di]).

onFilesDeletionConfirmed(Action, Tasks, NewTasks):-
	Action = [_, _, _, _, _, Times],
	% REGION choose one time at random @@ we should process all selected times!
	member(T, Times),
	convertTimeToNumber(T, N),
	% END
	removeTimeFromAction(Action, A2),
	NewTasks2 = [(N, A2) | Tasks],
	onTimer(NewTasks2, NewTasks),
	writeln(closingDialog(deleteChoiceBox)),
	dialogClose(deleteChoiceBox).



% REGION this is completely provisional @@@
convertTimeToNumber('Now', T):-
	get_time(T).
convertTimeToNumber('15 seconds from now', T):-
	get_time(T2),
	T is T2+15.
convertTimeToNumber('1 minute from now', T):-
	get_time(T2),
	T is T2+1*60.
convertTimeToNumber('2 minutes from now', T):-
	get_time(T2),
	T is T2+2*60.
convertTimeToNumber('4 minutes from now', T):-
	get_time(T2),
	T is T2+4*60.
convertTimeToNumber('5 minutes from now', T):-
	get_time(T2),
	T is T2+5*60.
convertTimeToNumber('10 minutes from now', T):-
	get_time(T2),
	T is T2+10*60.
convertTimeToNumber('15 minutes from now', T):-
	get_time(T2),
	T is T2+15*60.
convertTimeToNumber('20 minutes from now', T):-
	get_time(T2),
	T is T2+20*60.
convertTimeToNumber('30 minutes from now', T):-
	get_time(T2),
	T is T2+30*60.
convertTimeToNumber('45 minutes from now', T):-
	get_time(T2),
	T is T2+45*60.
convertTimeToNumber('45 minutes from now', T):-
	get_time(T2),
	T is T2+45*60.
convertTimeToNumber('1 hour from now', T):-
	get_time(T2),
	T is T2+1*60*60+0*60.
convertTimeToNumber('1 hour 30 minutes from now', T):-
	get_time(T2),
	T is T2+1*60*60+30*60.
convertTimeToNumber('1 hour 45 minutes from now', T):-
	get_time(T2),
	T is T2+1*60*60+45*60.
convertTimeToNumber('8 hours from now', T):-
	get_time(T2),
	T is T2+8*60*60.
convertTimeToNumber('8 hours 45 minutes from now', T):-
	get_time(T2),
	T is T2+8*60*60+45*60.
% END

deleteConfirmationLoop(Action, Tasks, NewTasks):-
	sleepUntilEventIsRaised(E),
	( E = dialogDestroyed(deleteChoiceBox)->
	       (writeln(deletionCanceled), NewTasks = Tasks);
	  E=buttonClicked(yesButton(deleteChoiceBox))->
	       onFilesDeletionConfirmed(Action, Tasks, NewTasks);
	  E=buttonClicked(noButton(deleteChoiceBox))->
        	(dialogClose(deleteChoiceBox), NewTasks = Tasks);
	  deleteConfirmationLoop(Action, Tasks, NewTasks)).



renameLoop(F):-
	sleepUntilEventIsRaised(E),
	( E = buttonClicked(renameButton)->
	    onRenameButtonClicked(F);
	  E = dialogDestroyed(renameDialog)->
	    true;
	  renameLoop(F)).
onRenameButtonClicked(F):-
	entryGetText(newNameEntry, NewName),
	NewName = F,
	dialogClose(renameDialog).
onRenameButtonClicked(F):-
	entryGetText(newNameEntry, NewName),
	NewName \= F,
	file_directory_name(F, Dir),
	addSlashToDirIfNeeded(Dir, DirSlash),
	concat_atom([DirSlash, NewName], NewNameWithPath),
	concat_atom(['mv \'', F, '\'  \'', NewNameWithPath, '\''], Command),
	shell(Command, Result),
	concat_atom(['Error in the command\n ', Command], ErrorText),
	(Result\=0->
	    messageBox(ErrorText, mainWindow);
	    (% REGION also sets ext/2, maybe improperly
	     removeOldFileFromTablesAndListBoxAndAddNewFileToSameTablesAndListBox(F, NewNameWithPath),
	     % END
	        % REGION if the file has changed extension... @@@ remove it from old table ext/2 and put it into new!
		file_name_extension(_, NewExt, NewName),
		file_name_extension(_, OldExt , F),
		(NewExt=OldExt -> true; 
		    (retractall(ext(_,NewNameWithPath)), % bug fixed: the old file may not
                                                         % have an extension! Therefore the new file may
                                                         % not have an extension either. Therefore retractall.
			assertExtensionIfAny(NewNameWithPath))),
		% END
		forall(selected(file, F),
		       checkListBoxSetButtonCheckState(checkListBoxOfKind(file), F, unchecked)),
		retractall(selected(file, _)),
		updateItemsAfterSelectionChanged,
		% REGION log
		get_time(Time),
		convert_time(Time, TimeString),
		concat_atom([TimeString, ': Renamed file: ', F, ' to  ', NewNameWithPath , '.'], Text),
		textViewAddLine(logTextView, Text))),
		% END
	dialogClose(renameDialog).
/*
  % @@ commented out until I fix the bug with filenames containing accents.
executeActionNow(['Rename file', Files, _, _]):-
	Files = [F],
	dialogNew(renameDialog, mainWindow, 'Rename file', 'img/icon.png'),
	widgetShow(renameDialog),

	vBoxNew(renameBox),
	widgetShow(renameBox),
	dialogAddToVBox(renameDialog, renameBox),

	labelNew(renameLabel, 'Please type a new name or modify the existing one:', noLineWrap),
	widgetShow(renameLabel),
	boxPackStart(renameBox, renameLabel, expand, fill),

	entryNew(newNameEntry),
	widgetShow(newNameEntry),
	boxPackStart(renameBox, newNameEntry, expand, fill),
	file_base_name(F, Base),
	entrySetText(newNameEntry, Base),

	buttonNew(renameButton, 'Rename'),
	widgetShow(renameButton),
	boxPackStart(renameBox, renameButton, doNotExpand, doNotFill),

	renameLoop(F).

	*/    
executeActionNow(['Delete files', Files, _Programs, _Devices, _]):-
        % NOTE: it is not guaranteed that the files
        % exist and are in the t/3, ext/2 and hiddenFile/1 tables! @@@ change this comment
	forall(member(F, Files), exists_file(F)),
	fromListOfFilesToQuotedString(Files, String),
	concat_atom(['rm -f ', String], Command),
	shell(Command, _),
	% REGION remove the file from all tables where it could be
	forall(member(F, Files),
	       retractall(file(F))), % it might already have been removed!
	forall(member(F, Files),
	       retractall(related(_, file, F))), % it might already have been removed!
	forall(member(F, Files),
	       retractall(ext(_, F))),
	% END
	% REGION remove the file from the listbox.
	% remove it for real, do not just hide. Otherwise at THE NEXT SORT IT WILL CRASH!
	forall(member(F, Files),
	       (checkListBoxItemExists(checkListBoxOfKind(file), F) -> % @@ bug here
	           checkListBoxDeleteItem(checkListBoxOfKind(file), F) ;
	           true)),
	% END
	forall(member(F, Files),
	       retractall(selected(file, F))),
	% REGION redraw list 
	updateItemsAfterSelectionChanged,
	% END
	% REGION log
	concat_atom(['Deleted files: ', String], Text),
	addStringToLog(Text).
        % END
executeActionNow(['Delete files', Files, _Programs, _Devices, _]):-
	member(F, Files),
	\+exists_file(F),
	concat_atom(['WARNING: Scheduled deletion discarded because the file ',
		     F, ' does not exist anymore. I suggest reparsing the database.'], Text),
	writeln(Text),
	addStringToLog(Text).
executeActionNow(['Move to...', Files, _, _, _]):-
	Files = [],
	messageBox('Error. No file selected!', mainWindow). 
executeActionNow(['Move to...', Files, _, _, _]):-
	Files\=[],
	fileOrDirSelection('Please select the destination directory from the above list, then click OK.', Dir),
	ifTheDirIsValidDoTheMovement(Dir, Files).
	
executeActionNow(['Mail files as attachments', Files, Programs, _, _]):-
	Programs=['Kmail'],
	sort(Files, SortedFiles),
	fromListOfFilesToQuotedString(SortedFiles, String),
	concat_atom(['kmail --attach ', String, '  &'], Command), % @@ fix! kmail wants --attach file1.pdf --attach file2.pdf
	writeln(Command),
	shell(Command, _),
	addStringToLog('Sent file by mail').
executeActionNow(['Mail files as attachments', Files, Programs, _, _]):-
	Programs=['Sylpheed'],
	sort(Files, SortedFiles),
	fromListOfFilesToQuotedString(SortedFiles, String),
	concat_atom(['sylpheed --attach ', String, '  &'], Command),
	writeln(Command),
	shell(Command, _),
	addStringToLog('Sent file by mail').
executeActionNow(['Rename multiple files', Files, Programs, _, _]):-
	(Programs\=['KRename'] -> writeln(error-program-should-be-krename); true),
	sort(Files, SortedFiles),
	fromListOfFilesToQuotedString(SortedFiles, String),
	concat_atom(['krename  ', String, '  &'], Command),
	writeln(Command),
	shell(Command, _),
	addStringToLog('Renamed multiple files').
executeActionNow(['Play audio', Files, Programs, _, _]):-
	sort(Files, SortedFiles),
	forall(member(P, Programs),
	       (shellName(P, P2),
		   fromListOfFilesToQuotedString(SortedFiles, String),
		   concat_atom([P2, ' ', String, '  &'], Command),
		   writeln(Command),
		   shell(Command, _))),
	addStringToLog('Played audio').
executeActionNow(['Play video', Files, Programs, _, _]):-
	sort(Files, SortedFiles),
	forall(member(P, Programs),
	       (shellName(P, P2),
		   fromListOfFilesToQuotedString(SortedFiles, String),
		   concat_atom([P2, ' ', String, '  &'], Command),
		   writeln(Command),
		   shell(Command, _))),
	addStringToLog('Played video').

ifTheDirIsValidDoTheMovement(Dir, _):-
	\+exists_directory(Dir),
	messageBox('This is not a valid directory', mainWindow).
ifTheDirIsValidDoTheMovement(Dir, _):-
	exists_directory(Dir),
	\+iAmWatchingTheDir(Dir),
	concat_atom(['The move command cannot be executed because segusoLand is not watching the destination directory: ', Dir,
		     '.\nPlease go to the options dialog box and watch the directory, then retry.'],Text),
	messageBox(Text, mainWindow),
	writeln(ifTheDirIsValidDoTheMovement-succeeded).
ifTheDirIsValidDoTheMovement(Dir, Files):-
	exists_directory(Dir),
	iAmWatchingTheDir(Dir),
	fromListOfFilesToQuotedString(Files, String),
	concat_atom(['mv ', String, ' \'', Dir, '\''], Command),
	shell(Command, Result), % 0=ok
	concat_atom(['Error in the command\n ', Command], ErrorText),
	(Result\=0->
	    messageBox(ErrorText, mainWindow);
	    (removeFromOldTablesAndCheckListBoxAndAddToNewTablesAndCheckListBox(Files, Dir),
		forall(selected(file, F),
		       checkListBoxSetButtonCheckState(checkListBoxOfKind(file), F, unchecked)),
		retractall(selected(file, _)),
		updateItemsAfterSelectionChanged,
		% REGION log
		get_time(Time),
		convert_time(Time, TimeString),
		concat_atom([TimeString, ': Moved files: ', String, ' to dir ', Dir , '.'], Text),
		textViewAddLine(logTextView, Text))).
		% END



removeFromOldTablesAndCheckListBoxAndAddToNewTablesAndCheckListBox(Files, Dir):-
	forall(member(F, Files),
	       (createNewFileNameFromOldNameAndNewDir(F, Dir, NewF),
		   removeOldFileFromTablesAndListBoxAndAddNewFileToSameTablesAndListBox(F, NewF))).
removeOldFileFromTablesAndListBoxAndAddNewFileToSameTablesAndListBox(Old, New):-
	% REGION t/3
	forall(	related(T, file, Old),
		(retract(related(T, file, Old)),
		    assert(related(T, file, New)))),
	% END
	% REGION ext/2
	forall(ext(E, Old),
	       (retract(ext(E, Old)),
		   assert(ext(E, New)))),
        % END

	% REGION CheckListBox
	(checkListBoxItemExists(checkListBoxOfKind(file), Old)->
	    (checkListBoxDeleteItem(checkListBoxOfKind(file), Old),
		addWithProperIcon(file, checkListBoxOfKind(file), New))),
	% END
	retractall(selected(file, Old)).
createNewFileNameFromOldNameAndNewDir(F, Dir, NewName):-
	file_base_name(F, Base),
	addSlashToDirIfNeeded(Dir, DirSlash),
	concat_atom([DirSlash,  Base], NewName).


addOneActionForEachSelectedTime(A, Tasks, NewTasks):-
	selectedTimes(TimeList),
	addOneActionForEachSelectedTime_Aux(A, TimeList, Tasks, NewTasks).
addOneActionForEachSelectedTime_Aux(_A, [], Tasks, Tasks).
addOneActionForEachSelectedTime_Aux(A, [Time|OtherTimes], Tasks, NewTasks):-
	convertTimeToNumber(Time, N),
	NewTasks2 = [ (N, A)|Tasks],
	addOneActionForEachSelectedTime_Aux(A, OtherTimes, NewTasks2, NewTasks).


deselectTheFoldersAndTheVerb(_K, V, _Folders):-
	% REGION deselect the folders and the verb automatically
	(selected(verb, V)->
	     (checkListBoxSetItemState(checkListBoxOfKind(verb),
				      V,
				      deselected),
		 onVerbDeselected(V))
	     ; true),
	% redraw the list because of Gtk BUG :-((
	checkListBoxDefeatBug(checkListBoxOfKind(verb)). 
        % END

retractSelectedVerbAndUpdate:-
	selected(verb, V),
	checkListBoxSetItemState(checkListBoxOfKind(verb), V, deselected),
	retract(selected(verb, V)),
	updateItemsAfterSelectionChanged.
retractSelectedVerbAndUpdate:-
	\+selected(verb, _).





% REGION pluggable piece of code that shows a dialog and asserts a folder
askForNameAndCreateNewIdea(K):-
	dialogNew(enterNewFolderNameDialog, mainWindow, 'Enter a name for the new concept', 'img/bulb.png'),
	widgetShow(enterNewFolderNameDialog),

	vBoxNew(enterNewFolderNameDialogBox),
	widgetShow(enterNewFolderNameDialogBox),
	dialogAddToVBox(enterNewFolderNameDialog, enterNewFolderNameDialogBox),

	labelNew(enterNewFolderNameLabel, 'Please type the name for the new concept', noLineWrap),
	widgetShow(enterNewFolderNameLabel),
	boxPackStart(enterNewFolderNameDialogBox, enterNewFolderNameLabel, doNotFill, fill),

	entryNew(newNameEntry),
	widgetShow(newNameEntry),
	boxPackStart(enterNewFolderNameDialogBox, newNameEntry, expand, fill, 2),

	buttonNew(createThisFolderButton, 'Create this concept'),
	widgetShow(createThisFolderButton),
	boxPackStart(enterNewFolderNameDialogBox, createThisFolderButton, doNotExpand, doNotFill, 5),

	createNewFolderLoop(K).
createNewFolderLoop(K):-
	sleepUntilEventIsRaised(E),
	(E = buttonClicked(createThisFolderButton)->
	    onCreateThisFolderButtonClicked(K);
	 E = dialogDestroyed(enterNewFolderNameDialog)->
	    writeln(exiting-createNewFolderLoop);
         (!, createNewFolderLoop(K))).
onCreateThisFolderButtonClicked(K):-
	entryGetText(newNameEntry, NewTable),
	downcase_atom(NewTable, NewTable2),
	idea(K, NewTable2),
	dialogClose(enterNewFolderNameDialog),
	concat_atom(['A ', K, ' concept with this name already exists.'], Text),
	messageBox(Text, mainWindow).
onCreateThisFolderButtonClicked(K):-
	entryGetText(newNameEntry, NewTable),
	downcase_atom(NewTable, NewTable2),
	\+idea( K, NewTable2),
	dialogClose(enterNewFolderNameDialog),
	assert(idea(K, NewTable2)).
% END pluggable piece of code

compressThisFile(F, _):-
	\+exists_file(F),
	concat_atom(['Error: compressThisFile: the file ', F, ' does not exist'], T),
	writeln(T).
compressThisFile(F, 'Bzip2' ):-
	exists_file(F),
	concat_atom(['bzip2 --best \'', F, '\''], Command),
	shell(Command, _).
compressThisFile(F, 'Gzip' ):-
	exists_file(F),
	concat_atom(['gzip --best \'', F, '\''], Command),
	shell(Command, _).

/*
  popADialogAskingForConcepts/1

  args:

  Result: output. Will be unified with either "rejected" or "accepted(Concepts)" */
popADialogAskingForConcepts(Result):-
	dialogNew(askForConceptDialog, mainWindow, 'Select concepts', 'img/bulb.png'),
	hBoxNew(askForConceptHBox),
	widgetShow(askForConceptHBox),
	dialogAddToVBox(askForConceptDialog, askForConceptHBox),

	vBoxNew(askForConceptLeftVBox),
	widgetShow(askForConceptLeftVBox),
	boxPackStart(askForConceptHBox, askForConceptLeftVBox, expand, fill, 10),
	
	vBoxNew(askForConceptRightVBox),
	widgetShow(askForConceptRightVBox),
	boxPackStart(askForConceptHBox, askForConceptRightVBox, expand, fill, 10),

	labelNew(askForConceptLabel, 'Select the concepts that you want to associate to the files.', lineWrap),
	widgetShow(askForConceptLabel),
	boxPackStart(askForConceptLeftVBox, askForConceptLabel, doNotExpand, fill, 10),


	buttonNew(associateToTheseConceptsButton, 'Associate'),
	widgetShow(associateToTheseConceptsButton),
	boxPackStart(askForConceptLeftVBox, associateToTheseConceptsButton, doNotExpand, doNotFill, 10),
		  
	checkListBoxNew(conceptsList, relief,  'concepts', selectedListHidden),
	widgetShow(conceptsList),
	boxPackStart(askForConceptRightVBox, conceptsList, expand, fill, 10),

	% REGION fill the list
	findall(I,idea(file, I), Ideas),
	sort(Ideas, Ideas2),
	forall(member(I, Ideas2),
	       checkListBoxAddItem(conceptsList,
				       I,  I,
				       ''/*tooltip*/, [], ''/*dir*/,
				       '', '', 'img/bulb.png')),
	% END
	% REGION select the concepts items that are already used by at least one file

        % END				       
        windowMaximize(askForConceptDialog),
	widgetShow(askForConceptDialog),
	askForConceptsLoop(Result, []).

askForConceptsLoop(Result, SelectedConcepts):-
	sleepUntilEventIsRaised(E),
	(askForConceptsManageEvent(E, Result, SelectedConcepts, SelectedConcepts2)->true
	;SelectedConcepts2 = SelectedConcepts),
	
	(E = dialogDestroyed(askForConceptDialog)->
	    true /*result is now instantiated*/
	; (!, askForConceptsLoop(Result, SelectedConcepts2))).
askForConceptsManageEvent(checkListBoxItemSelected(conceptsList, I), _, S, S2):-
	S2=[I|S].
askForConceptsManageEvent(checkListBoxItemDeselected(conceptsList, I), _, S, S2):-
	remove(I, S, S2).
askForConceptsManageEvent(dialogDestroyed(askForConceptDialog), rejected, S, S).
askForConceptsManageEvent(buttonClicked(associateToTheseConceptsButton), Result, S, S):-
	Result = accepted(S),
	dialogClose(askForConceptDialog).

associateFilesToConceptsIfResultIsAffirmative(_, rejected).
associateFilesToConceptsIfResultIsAffirmative(Files, accepted(Concepts)):-
	forall(member(O, Files), 
	       forall(member(C, Concepts),
		      assertIfNotAlreadyThere(related(C, file, O)))),
        % REGION update the visible ideas below the files, in the file panel
	forall(member( O, Files),
	       recomputeFolders(file, O)).
        % END


executeAction(['Configure mouse clicking behaviour', _, P, _, _, _], T, T):-
	member( 'Kde control center', P),
	shell('kcmshell mouse &', _).
executeAction(['Set mouse cursor theme', _, P, _, _, _], T, T):-
	member('Kde control center', P),
	shell('kcmshell mouse &', _).
executeAction(['Change keyboard layout', _F, P, _, _, _], T, T):-
	member('Kde control center', P),
	shell('kcmshell keyboard_layout &', _).
executeAction(['Change keyboard repeat speed', _F, P, _, _, _], T, T):-
	member('Kde control center', P),
	shell('kcmshell keyboard &', _).
executeAction(['Set mouse speed', _F, P, _, _, _], T, T):-
	member('Kde control center', P),
	shell('kcmshell mouse &', _).
executeAction(['Configure mouse clicking behaviour', _F, P, _, _, _], T, T):-
	member('Kde control center', P),
	shell('kcmshell mouse &', _).
executeAction(['Associate concepts to files', F, _, _, _, _], Tasks, Tasks):-
	popADialogAskingForConcepts(Result),
	associateFilesToConceptsIfResultIsAffirmative(F, Result).
executeAction(['Delete a concept', _, _, _, _, _], T, T):-
	messageBox('To delete a concept:\n\n1. Select the concept \n\n2. On the toolbar, click the crossed-out light bulb', mainWindow).
executeAction(['Compress files separately', F, P, _, _, _], T, T):-
	member(P, Pr),
	(Pr='Gzip' ; Pr='Bzip2'),
	forall(member(Fi, F),
	       compressThisFile(Fi, Pr)),
	% REGION transfer metadata to the new files. and remove old metadata, because the file does not exist anymore! @@@

	% END
	addStringToLog('Compressed some files').
executeAction(['Compress files separately', _, P, _, _, _], T, T):-
	member(Pr, P),
	Pr\='Gzip' ,
	Pr\='Bzip2',
	concat_atom(['Compress files separately: The selected program is not yet supported: ', Pr], Text),
	messageBox(Text, mainWindow).
executeAction(['Stop indexing this directory', _, _, _, Dirs, _], T, T):-
	stopWatchingDirs(Dirs).
executeAction(['Import directory and all its files' | _], T, T):-
	fileOrDirSelection('Please select a directory from the above list, then click OK', NewDir),
	importDirectory(NewDir, _/*I don't need the returned value */),
	redrawDirPanelClearingAndReaddingEverything,
	redrawFolderPanelClearingAndReaddingEverything,
	checkListBoxDefeatBug(checkListBoxOfKind(file)).
executeAction(['Import directory and all its files (recursively)' | _], T, T):-
	fileOrDirSelection('Please select a directory from the above list, then click OK', NewDir),
	importDirectoryRecursively(NewDir),
	redrawDirPanelClearingAndReaddingEverything,
	redrawFolderPanelClearingAndReaddingEverything,
	checkListBoxDefeatBug(checkListBoxOfKind(file)).

executeAction([ 'Show files/programs/actions/devices when no concept is selected'| _], T, T):-
	\+showPanelsWhenNoConceptIsSelected,
	assert(showPanelsWhenNoConceptIsSelected).
executeAction([ 'Show files/programs/actions/devices when no concept is selected'| _], T, T):-
	showPanelsWhenNoConceptIsSelected,
	messageBox('Panels are already visible when non concepts is selected.', mainWindow).
executeAction([ 'Do not show files/programs/actions/devices when no concept is selected'| _], T, T):-
	showPanelsWhenNoConceptIsSelected,
	retract(showPanelsWhenNoConceptIsSelected).
executeAction([ 'Do not show files/programs/actions/devices when no concept is selected'| _], T, T):-
	\+showPanelsWhenNoConceptIsSelected,
	messageBox('Panels are already being hidden when no concept is selected.', mainWindow).

	
executeAction(['Automatically deselect concepts when a nonconcept is selected'| _], T, T):-
	autoDeselectConcepts,
	retract(autoDeselectConcepts).
executeAction(['Automatically deselect concepts when a nonconcept is selected'| _], T, T):-
	\+autoDeselectConcepts,
	assert(autoDeselectConcepts).
executeAction(['Create a new concept for organizing files'| _], T, T):-
	askForNameAndCreateNewIdea(file),
	redrawFolderPanelClearingAndReaddingEverything.
executeAction(['Show/hide concepts below files'| _], T, T):-
	showFoldersBelowFiles,
	retract(showFoldersBelowFiles),
	checkListBoxSetFoldersVisibility(checkListBoxOfKind(file), hidden),
	retractSelectedVerbAndUpdate.
executeAction(['Show/hide concepts below files'| _], T, T):-
	\+showFoldersBelowFiles,
	assert(showFoldersBelowFiles),
	checkListBoxSetFoldersVisibility(checkListBoxOfKind(file), visible),
	retractSelectedVerbAndUpdate.
executeAction(['Show/hide concepts below programs'| _], T, T):-
	showFoldersBelowPrograms,
	retract(showFoldersBelowPrograms),
	checkListBoxSetFoldersVisibility(checkListBoxOfKind(program), hidden),
	retractSelectedVerbAndUpdate.
executeAction(['Show/hide concepts below programs'| _], T, T):-
	\+showFoldersBelowPrograms,
	assert(showFoldersBelowPrograms),
	checkListBoxSetFoldersVisibility(checkListBoxOfKind(program), visible),
	retractSelectedVerbAndUpdate.



%executeAction(['Manage the directories watched by segusoLand', T, T):-
%	onOptionsButtonPressed,
%	retractSelectedVerbAndUpdate.
executeAction(['Show scheduled tasks'| _], Tasks, NewTasks):-
	onManageTasksClicked(Tasks, NewTasks),
	retractSelectedVerbAndUpdate.
executeAction(['Show/hide tooltips'| _], Tasks, Tasks):-
	tooltipsAreEnabled,
	setTooltipsState(disabled),
	retract(tooltipsAreEnabled),
	retractSelectedVerbAndUpdate.
executeAction(['Show/hide tooltips'| _], Tasks, Tasks):-
	\+tooltipsAreEnabled,
	setTooltipsState(enabled),
	assert(tooltipsAreEnabled),
	retractSelectedVerbAndUpdate.

executeAction(['Toggle scrolling buttons always visible'| _], Tasks, Tasks):-
        scrollingButtonsAlwaysVisible,
        checkListBoxSetKeepScrollingBoxesAlwaysVisible(checkListBoxOfKind(verb),disabled),
	checkListBoxSetKeepScrollingBoxesAlwaysVisible(checkListBoxOfKind(directory),disabled),
	checkListBoxSetKeepScrollingBoxesAlwaysVisible(checkListBoxOfKind(idea),disabled),
        checkListBoxSetKeepScrollingBoxesAlwaysVisible(checkListBoxOfKind(file),disabled),
	checkListBoxSetKeepScrollingBoxesAlwaysVisible(checkListBoxOfKind(device),disabled),
	checkListBoxSetKeepScrollingBoxesAlwaysVisible(checkListBoxOfKind(time),disabled),
        checkListBoxSetKeepScrollingBoxesAlwaysVisible(checkListBoxOfKind(program),disabled),
        retract(scrollingButtonsAlwaysVisible),
	retractSelectedVerbAndUpdate.
executeAction(['Toggle scrolling buttons always visible'| _], Tasks, Tasks):-
        \+scrollingButtonsAlwaysVisible,
        checkListBoxSetKeepScrollingBoxesAlwaysVisible(checkListBoxOfKind(verb),enabled),
        checkListBoxSetKeepScrollingBoxesAlwaysVisible(checkListBoxOfKind(file),enabled),
        checkListBoxSetKeepScrollingBoxesAlwaysVisible(checkListBoxOfKind(directory),enabled),
	checkListBoxSetKeepScrollingBoxesAlwaysVisible(checkListBoxOfKind(idea),enabled),
	checkListBoxSetKeepScrollingBoxesAlwaysVisible(checkListBoxOfKind(device),enabled),
	checkListBoxSetKeepScrollingBoxesAlwaysVisible(checkListBoxOfKind(time),enabled),
        checkListBoxSetKeepScrollingBoxesAlwaysVisible(checkListBoxOfKind(program),enabled),
        assert(scrollingButtonsAlwaysVisible),
	retractSelectedVerbAndUpdate.


executeAction(['Show/hide log'| _], Tasks, Tasks):-
	widgetIsVisible(logTextView)-> 	widgetHide(logTextView); widgetShow(logTextView),
	retractSelectedVerbAndUpdate.


executeAction(['Show/hide file dates'| _], Tasks, Tasks):-
	\+checkListBoxTheDateIsVisible(checkListBoxOfKind(file)),
	checkListBoxSetDateVisibility(checkListBoxOfKind(file), visible),
	retractSelectedVerbAndUpdate.
executeAction(['Show/hide file dates'| _], Tasks, Tasks):-
	checkListBoxTheDateIsVisible(checkListBoxOfKind(file)),
	checkListBoxSetDateVisibility(checkListBoxOfKind(file), hidden),
	retractSelectedVerbAndUpdate.

executeAction(['Show/hide file sizes'| _], Tasks, Tasks):-
	\+checkListBoxTheSizeIsVisible(checkListBoxOfKind(file)),
	checkListBoxSetSizeVisibility(checkListBoxOfKind(file), visible),
	retractSelectedVerbAndUpdate.
executeAction(['Show/hide file sizes'| _], Tasks, Tasks):-
	checkListBoxTheSizeIsVisible(checkListBoxOfKind(file)),
	checkListBoxSetSizeVisibility(checkListBoxOfKind(file), hidden),
	retractSelectedVerbAndUpdate.
	

executeAction(['Show/hide directories below the files'| _], Tasks, Tasks):-
	\+checkListBoxTheDirIsVisible(checkListBoxOfKind(file)),
	checkListBoxSetDirVisibility(checkListBoxOfKind(file), visible),
	retractSelectedVerbAndUpdate.
executeAction(['Show/hide directories below the files'| _], Tasks, Tasks):-
	checkListBoxTheDirIsVisible(checkListBoxOfKind(file)),
	checkListBoxSetDirVisibility(checkListBoxOfKind(file), hidden),
	retractSelectedVerbAndUpdate.
	
	
executeAction(['Sort files by name'| _], Tasks, Tasks):-
	checkListBoxSortByName(checkListBoxOfKind(file)),
	retractSelectedVerbAndUpdate.
executeAction(['Sort files by size'| _], Tasks, Tasks):-
	checkListBoxSortBySize(checkListBoxOfKind(file)),
	retractSelectedVerbAndUpdate.
executeAction(['Sort files by date'| _], Tasks, Tasks):-
	checkListBoxSortByDate(checkListBoxOfKind(file)),
	retractSelectedVerbAndUpdate.
executeAction(['Refresh file list'| _], Tasks, Tasks):-
	% REGION 
	forall(file(F),
	       (exists_file(F)->
		   true;
		   (retract( file(F)),
		       retractall(ext(_, F)),
		       retractall(related(_, file,  F)),
		       (checkListBoxItemExists(checkListBoxOfKind(file), F)->
			   checkListBoxDeleteItem(checkListBoxOfKind(file), F)
		       ; true)))),
	% END
	forall(segusoDir(D),
	       seeIfThereAreNewFilesInDir(D)),
	% REGION fixes crash. The new ideas have not been added to the list box
	redrawFolderPanelClearingAndReaddingEverything,
	% END
	addStringToLog('Reparsed directories.'),
	retractSelectedVerbAndUpdate.


executeAction(['Show/hide quick-search boxes'| _], Tasks, Tasks):-
	\+theQuickSearchBoxesAreVisible,
	widgetShow(filterHBox(verb)),
	widgetShow(filterHBox(file)),
	widgetShow(filterHBox(idea)),
	widgetShow(filterHBox(directory)),
	widgetShow(filterHBox(program)),
	widgetShow(filterHBox(device)),
	assert(theQuickSearchBoxesAreVisible), % remember
	retractSelectedVerbAndUpdate.
executeAction(['Show/hide quick-search boxes'| _], Tasks, Tasks):-
	theQuickSearchBoxesAreVisible,
	widgetHide(filterHBox(verb)),
	widgetHide(filterHBox(file)),
	widgetHide(filterHBox(idea)),
	widgetHide(filterHBox(directory)),
	widgetHide(filterHBox(program)),
	widgetHide(filterHBox(device)),
	retract(theQuickSearchBoxesAreVisible),
	retractSelectedVerbAndUpdate.

executeAction(A, Tasks, NewTasks):-
	A = ['Delete files', Files, _, _, _, _],
	fromListOfFilesToQuotedString(Files, String),
	concat_atom(['Are you sure you want to delete the following files and all their attributes? \n', String], Text),
	choiceBox(deleteChoiceBox, 'Confirm deletion', Text, 'Yes I am',  'Forget that'), 
	deleteConfirmationLoop(A, Tasks, NewTasks).
executeAction(A, Tasks, NewTasks):-
	A = ['Mail files as attachments', _F, _P, _D, _Di, T],
	% REGION choose one time at random. @@ ideally, we should process all selected times!
	member(Ti, T),
	convertTimeToNumber(Ti, N),
	% END
	removeTimeFromAction(A, A2),
	NewTasks2 = [(N, A2) | Tasks],
	onTimer(NewTasks2, NewTasks).
executeAction(A, Tasks, NewTasks):-
	A = ['Rename multiple files', _F, P, _D, _Di, T],
	member('KRename', P),
	% REGION choose one time at random. @@ ideally, we should process all selected times!
	member(Ti, T),
	convertTimeToNumber(Ti, N),
	% END
	removeTimeFromAction(A, A2),
	NewTasks2 = [(N, A2) | Tasks],
	onTimer(NewTasks2, NewTasks).
executeAction(['Rename file', _, _, _, _, Times], T, T):-
	Times\=['Now'],
	messageBox('This action cannot be executed in the future. Please select only the time \'Now\' and retry.', mainWindow).
executeAction(A, N, N):-
	A = ['Rename file', _, _, _, _, Times],
	Times=['Now'],
	removeTimeFromAction(A, A2),
	executeActionNow(A2),
	addStringToLog('Renamed file').

executeAction(A, Tasks, NewTasks):-
	A = [ 'Play audio' , _, _, _, _, T],
	% REGION choose one time at random. @@ ideally, we should process all selected times!
	member(Ti, T),
	convertTimeToNumber(Ti, N),
	% END
	removeTimeFromAction(A, A2),
	NewTasks2 = [(N, A2) | Tasks],
	onTimer(NewTasks2, NewTasks),
	addStringToLog('Enqueued action: Play audio').

executeAction(A, Tasks, NewTasks):-
	A = [ 'Play video' , _, _, _, _, T],
	% REGION choose one time at random. @@ ideally, we should process all selected times!
	member(Ti, T),
	convertTimeToNumber(Ti, N),
	% END
	removeTimeFromAction(A, A2),
	NewTasks2 = [(N, A2) | Tasks],
	onTimer(NewTasks2, NewTasks),
	addStringToLog('Enqueued action: Play video').

executeAction(['View free space on disk', _, P|_], N, N):-
	member('Kdiskfree', P),
	Command = 'kdf &',
	writeln(Command),
	shell(Command, _).
executeAction(['Partition disk', _, P|_],  N, N):-
	member(Pr, P),
	shellName(Pr, Pr2),
	concat_atom([Pr2, '&'], Command),
	writeln(Command),
	shell(Command, _).
executeAction(['Rip CD', _, P|_],  N, N):-
	member(Pr, P),
	shellName(Pr, Pr2),
	concat_atom([Pr2, '&'], Command),
	writeln(Command),
	shell(Command, _).
executeAction(['Print picture', F, P|_], N, N):-
	member(Pr, P),
	shellName(Pr, Pr2),
	sort(F, Sorted),
	fromListOfFilesToQuotedString(Sorted, String),
	concat_atom([Pr2, ' ', String, '  &'], Command),
	writeln(Command),
	shell(Command, _).
executeAction(['Enqueue audio for playing', F, P|_], N, N):-
	member(Pr, P),
	shellName(Pr, Pr2),
	sort(F, F2),
	fromListOfFilesToQuotedString(F2, String),
	concat_atom([Pr2, ' --enqueue ', String, '  &'], Command),
	writeln(Command),
	shell(Command, _).
executeAction(['Enqueue video for playing', F, P|_], N, N):-
	member(Pr, P),
	shellName(Pr, Pr2),
	sort(F, F2),
	fromListOfFilesToQuotedString(F2, String),
	concat_atom([Pr2, ' --enqueue ', String, '  &'], Command),
	writeln(Command),
	shell(Command, _).
executeAction(['Edit SXW file', F, P, _, _, _], N, N):-
	member('OpenOfficeWriter', P),
	fromListOfFilesToQuotedString(F, String),
	concat_atom(['soffice  ', String, '  &'], Command),
	writeln(Command),
	shell(Command, _).

executeAction(['Edit DOC file',F, P|_], N, N):-
	member('OpenOfficeWriter', P),
	fromListOfFilesToQuotedString(F, String),
	concat_atom(['soffice  ', String, '  &'], Command),
	writeln(Command),
	shell(Command, _).
executeAction(['Edit picture', F, P|_], N, N):-
	member('The gimp', P),
	fromListOfFilesToQuotedString(F, String),
	concat_atom(['gimp-remote -n ', String, '  &'], Command),
	writeln(Command),
	shell(Command, _).
executeAction(['View print queue', _, P|_], N, N):-
	mmeber('KJobViewer', P),
	shell('kjobviewer &', _).
executeAction(['Burn data CD',F, P|_],  N, N):-
	member('K3b', P),
	fromListOfFilesToQuotedString(F, String),
	concat_atom(['k3b --datacd ', String, '  &'], Command),
	writeln(Command),
	shell(Command, Status),
	displayProgramNotInstalledIfNecessary(k3b, Status).
executeAction(['Print PDF', F, P|_],  N, N):-
	member('Kprinter', P),
	fromListOfFilesToQuotedString(F, String),
	concat_atom(['kprinter ', String, '  &'], Command),
	writeln(Command),
	shell(Command, _).
executeAction(['Take a snapshot of your screen', _, P|_],  N, N):-
	member('Ksnapshot', P),
	shell('ksnapshot &', _).
executeAction(['Burn audio CD', F, P|_], N, N):-
	member('K3b', P),
	fromListOfFilesToQuotedString(F, String),
	concat_atom(['k3b --audiocd ', String, '  &'], Command),
	writeln(Command),
	shell(Command, _).
executeAction(['Burn from ISO', F, P |_], N, N):-
	member('K3b', P),
	fromListOfFilesToQuotedString(F, String),
	concat_atom(['k3b --cdimage ', String, '  &'], Command),
	writeln(Command),
	shell(Command, _).
executeAction(['Burn video CD', F, P|_], N, N):-
	member( 'K3b', P),
	fromListOfFilesToQuotedString(F, String),
	concat_atom(['k3b --videocd ', String, '  &'], Command),
	writeln(Command),
	shell(Command, _).
executeAction(['Burn CD from BIN and CUE files', F, P|_], N, N):-
	member( 'K3b', P),
	fromListOfFilesToQuotedString(F, String),
	concat_atom(['k3b --cdimage ', String, '  &'], Command),
	writeln(Command),
	shell(Command, _).
executeAction(['View picture', F, P|_],  N, N):-
	member(Pr, P),
	shellName(Pr, Pr2),
	fromListOfFilesToQuotedString(F, String),
	concat_atom([Pr2, ' ', String, '  &'], Command),
	writeln(Command),
	shell(Command, Status),
	displayProgramNotInstalledIfNecessary(Pr2, Status).
executeAction(['View HTML file', F, P|_], N, N):-
	member(Pr, P),
	shellName(Pr, Pr2),
	fromListOfFilesToQuotedString(F, String),
	concat_atom([Pr2, ' ', String, '  &'], Command),
	writeln(Command),
	shell(Command, Status),
	displayProgramNotInstalledIfNecessary(Pr2, Status).
executeAction(['View DVI file',F, P|_], N, N):-
	member(Pr, P),
	shellName(Pr, Pr2),
	fromListOfFilesToQuotedString(F, String),
	concat_atom([Pr2, ' ', String, '  &'], Command),
	writeln(Command),
	shell(Command, Status),
	displayProgramNotInstalledIfNecessary(Pr2, Status).
executeAction(['Erase CD-RW'|_], N, N):-
	shell('k3b --erasecd &', _).
executeAction(['View calendar and schedule', _, P|_], N, N):-
	member(Pr, P),
	shellName(Pr, Pr2),
	concat_atom([Pr2, ' &'], Command),
	writeln(Command),
	shell(Command, _).
executeAction(['Set sound volume', _, P|_], N, N):-	
	member(Pr, P),
	shellName(Pr, Pr2),
	concat_atom([Pr2, ' &'], Command),
	writeln(Command),
	shell(Command, _).
executeAction(['Edit plain text file', F, P|_], N, N):-
	member(Pr, P),
	shellName(Pr, Pr2),
	fromListOfFilesToQuotedString(F, String),
	concat_atom([Pr2, ' ', String, '  &'], Command),
	writeln(Command),
	shell(Command, Status),
	displayProgramNotInstalledIfNecessary(Pr2, Status).
executeAction(['Extract archive', F, P|_], N, N):-
	member('Ark', P),
	member(Fi, F),
	concat_atom(['ark --extract \"', Fi, '\" &'], Command),
	writeln(Command),
	shell(Command, _).
executeAction(['Compress files', F, P|_],N, N):-
	member('FileRoller', P),
	fromListOfFilesToQuotedString(F, String),
	concat_atom(['file-roller --add ', String, '  '], Command), % NOT in background!
	writeln(Command),
	shell(Command, _).
executeAction(['Compress files', F, P|_], N, N):-
	member('Ark', P),
	fromListOfFilesToQuotedString(F, String),
	concat_atom(['ark --add ', String, '  '], Command), % NOT in background!
	writeln(Command),
	shell(Command, _).
executeAction(['Extract archive', F, P|_],N, N):-
	member('FileRoller', P),
	member(Fi, F),
	concat_atom(['file-roller --extract \"', Fi, '\" &'], Command),
	writeln(Command),
	shell(Command, _).
executeAction(['View contents of archive', F, P|_], N, N):-
	member(Pr, P),
	shellName(Pr, Pr2),
	member(Fi, F),
	concat_atom([Pr2, ' \"', Fi, '\" &'], Command),
	writeln(Command),
	shell(Command, _).
executeAction(['Check mail', _, P|_], N, N):-
	member(Pr, P),
	Pr='Sylpheed',
	concat_atom(['sylpheed --receive-all &'], Command),
	writeln(Command),
	shell(Command, _).
executeAction(['Check mail', _, P|_], N, N):-
	member(Pr, P),
	Pr\='Sylpheed',
	shellName(Pr, Pr2),
	concat_atom([Pr2, ' &'], Command),
	writeln(Command),
	shell(Command, _).
executeAction(['Compose a new email', _, P|_], N, N):-
	member('Kmail', P),
	concat_atom(['kmail --composer &'], Command),
	writeln(Command),
	shell(Command, _).
executeAction(['Compose a new email', _, P|_], N, N):-
	member('Sylpheed', P),
	concat_atom(['sylpheed --compose &'], Command),
	writeln(Command),
	shell(Command, _).
executeAction(['Compose a new email', _, P|_], N, N):-
	member(Pr, P),
	Pr\='Kmail',
	Pr\='Sylpheed',
	shellName(Pr, Pr2),
	concat_atom([Pr2, ' &'], Command),
	writeln(Command),
	shell(Command, _).
executeAction(['Browse the internet', _, P|_], N, N):-
	member(Pr, P),
	shellName(Pr, Pr2),
	concat_atom([Pr2, ' &'], Command),
	shell(Command , _).
executeAction(['Browse the directory of this file', F |_], N, N):-
	F\=[_],
	messageBox('You cannot select more than one file with this action.', mainWindow).
executeAction(['Browse the directory of this file', F, P|_], N, N):-
	F=[Fi],
	member(Pr, P),
	shellName(Pr, Pr2),
	file_directory_name(Fi, D),
	concat_atom([Pr2, ' \'', D , '\' &'], Command),
	shell(Command, _).

executeAction(['Open terminal in the directory of this file', F|_], N, N):-
	F\=[_],
	messageBox('You cannot select more than one file with this action.', mainWindow).
executeAction(['Open terminal in the directory of this file', F, P|_], N, N):-
	F = [Fi],
	member(Pr, P),
	shellName(Pr, Pr2),
	file_directory_name(Fi, D),
	concat_atom([Pr2, ' --working-directory=', D , ' &'], Command),
	shell(Command, _).
executeAction(['View PDF', F, P|_], N, N):-
	member(Fi, F),
	member(Pr, P),
	shellName(Pr, Pr2),
	concat_atom([Pr2, ' \'', Fi , '\' &'], Command),
	shell(Command, _).
executeAction(['Edit TM file', F, P|_], N, N):-
	member('TeXmacs', P),
	sort(F, Sorted),
	fromListOfFilesToQuotedString(Sorted, String),
	concat_atom(['texmacs ', String, '  &'], Command),
	writeln(Command),
	shell(Command, _).
executeAction(['View CPU usage',_, P|_], N, N):-
	member(Pr, P),
	shellName(Pr, Pr2),
	concat_atom([Pr2, ' &'], Command),
	writeln(Command),
	shell(Command, _).
executeAction(['View or terminate active processes', _, P|_], N, N):-
	member(Pr, P),
	shellName(Pr, Pr2),
	concat_atom([Pr2, ' &'], Command),
	writeln(Command),
	shell(Command, _).
executeAction(['View memory usage', _, P|_], N, N):-
	member(Pr, P),
	shellName(Pr, Pr2),
	concat_atom([Pr2, ' &'], Command),
	writeln(Command),
	shell(Command, _).
executeAction(['View SWAP use', _, P|_], N, N):-
	member(Pr, P),
	shellName(Pr, Pr2),
	concat_atom([Pr2, ' &'], Command),
	writeln(Command),
	shell(Command, _).
executeAction(['Take a snapshot of your screen', _, P|_], N, N):-
	member(Pr, P),
	shellName(Pr, Pr2),
	concat_atom([Pr2, ' &'], Command),
	writeln(Command),
	shell(Command, _).
executeAction(['Transfer files to and from an FTP site', _, P|_], N, N):-
	member(Pr, P),
	shellName(Pr, Pr2),
	concat_atom([Pr2, ' &'], Command),
	writeln(Command),
	shell(Command, _).

executeAction(['Edit 3D picture', F, P |_], N, N):-
	member(Fi, F),
	member(Pr, P),
	shellName(Pr, Pr2),
	concat_atom([Pr2, ' \'', Fi , '\' &'], Command),
	writeln(Command),
	shell(Command, _).

executeAction([V/*in*/|_], T, T):-
	concat_atom(['Action \"', V, '\" not implemented.\nWe need help!'], Text),
	messageBox(Text, mainWindow).





fromListOfFilesToQuotedString(List, String):-
	fromListOfFilesToQuotedStringAux(List, '', String).

fromListOfFilesToQuotedStringAux([], CurString, CurString).
fromListOfFilesToQuotedStringAux([H|T], CurString, NewString):-
	H\=[],
	concat_atom([CurString, ' \"', H, '\"'], S2),
	fromListOfFilesToQuotedStringAux(T, S2, NewString).


displayProgramNotInstalledIfNecessary(_, 0).
displayProgramNotInstalledIfNecessary(Program, Status):-
	Status \= 0,
	concat_atom([Program, ' is not installed.'], M),
	messageBox(M, mainWindow).


deselectAllConcepts:-
	forall(selected(idea-K, C),
	       (checkListBoxSetItemState(checkListBoxOfKind(idea), pair(K, C), deselected),
		   retract(selected(idea-K,C)))).


onObjectSelected(K/*in*/, O/*in*/):-
	selected(K, O). % if it's already selected, don't do anything.
onObjectSelected(K, O):-
	\+selected(K, O),
	assert(selected(K, O)),
	(autoDeselectConcepts->deselectAllConcepts; true),
	updateItemsAfterSelectionChanged,
	updateDeleteIdeasToolbarButtons,
	addCurrentlySelectedConceptsToRecentConcepts.

addCurrentlySelectedConceptsToRecentConcepts:-
	\+selected(idea-_, _).
addCurrentlySelectedConceptsToRecentConcepts:-
	selected(idea-_, _),
	findall((I, K), selected(idea-K, I), IdeasPairs),
	sort(IdeasPairs, IdeasPairs2),
	retractall(recentConcept(IdeasPairs2)),
	asserta(recentConcept(IdeasPairs2)).

% REGION a simple way to have a runtime assertion mechanism
iDeclareThisIsTrue(F):-
	iDeclareThisIsTrue(F, 'Not-specified').
iDeclareThisIsTrue(F, Context):-
	(F-> true
	;   (	concat_atom(['ERROR: in context ', Context, ': runtime assertion is false : '], T),
		write(T),
		writeln(F),
		throw(T))).
% END

sortActionSoThatItsRepresentationBecomesUnique([V, F, P, D, Di, T], [V, F2, P2, D2, Di2, T2]):-
	sort(F, F2),
	sort(P, P2),
	sort(D, D2),
	sort(Di, Di2),
	sort(T, T2).

addActionToRecentActionList(A):-
	recentAction(repeated, A),
	retract(recentAction(repeated, A)),
	asserta(recentAction(repeated, A)).
addActionToRecentActionList(A):-
	\+recentAction(repeated, A),
	recentAction(notRepeated, A),
	retract(recentAction(notRepeated, A)),
	asserta(recentAction(repeated, A)).
addActionToRecentActionList(A):-
	\+recentAction(repeated, A),
	\+recentAction(notRepeated, A),
	asserta(recentAction(notRepeated, A)).


onIdeaSelected(K, I):-
	\+selected(idea-K, I),
	assert(selected(idea-K, I)),
	showPanelsAgain,
	updateItemsAfterSelectionChanged2,
	% REGION only necessary for folder narrowing
	redrawFolderPanelHidingAndReshowing,
	% END
	updateDeleteIdeasToolbarButtons.
	


showPanelsAgain:-
	(panelStateMinimized(device)->true;widgetShow(mainObjectVBox(device))),
	(panelStateMinimized(directory)->true;widgetShow(mainObjectVBox(directory))),
	(panelStateMinimized(file)->true;widgetShow(mainObjectVBox(file))),
	(panelStateMinimized(program)->true;widgetShow(mainObjectVBox(program))),
	(panelStateMinimized(time)->true;widgetShow(timeBox)),
	widgetShow(verbBox),
	widgetHide(scrolledWindowTutorial).



updateItemsAfterSelectionChanged2:-
	clearAllPanels,
		
	% REGION recompute the visible verbs
	findall(V, verb(V), Verbs),
	sort(Verbs, VerbsSorted),


	sublist(theVerbIsVisible, VerbsSorted, VisibleVerbsSorted),

	retractall(visibleVerb(_)),
	forall(member(M, VisibleVerbsSorted),
	       assert(visibleVerb(M))),
	% END

	narrowOrRedrawTheVerbList(VisibleVerbsSorted),

	retractall(visibleObject(_, _)),
	
	forall(program(M), decideObjectVisibility(M,program, VisibleVerbsSorted)),

	forall(file(M), decideObjectVisibility(M,file, VisibleVerbsSorted)),

	forall(device(M), decideObjectVisibility(M,device, VisibleVerbsSorted)),

	forall(directory(M), decideObjectVisibility(M,directory, VisibleVerbsSorted)),

	updateRedArrows(VisibleVerbsSorted).
	%redrawFolderPanelHidingAndReshowing.

updateDeleteIdeasToolbarButtons:-
	(selected(idea-_, _)->(widgetShow(deleteIdeaButton))
	;( widgetHide(deleteIdeaButton))).

clearAllPanels:-
	checkListBoxHideAllItems(checkListBoxOfKind(file)),
	checkListBoxHideAllItems(checkListBoxOfKind(device)),
	checkListBoxHideAllItems(checkListBoxOfKind(program)).

thereIsASelectedObject:-
	(selected(program, _) ; selected(file, _) ; selected(device, _); selected(directory, _)).

narrowOrRedrawTheVerbList(VisibleVerbsSorted):-
	(selected(verb, _) ;\+thereIsASelectedObject),
	% we must not group verbs in any way
	checkListBoxClear(checkListBoxOfKind(verb)),
	forall(member(M, VisibleVerbsSorted), 
	       addWithProperIcon(verb, checkListBoxOfKind(verb), M)),
	% REGION set the selected/deselected state of the visible ones	
	forall(member(V, VisibleVerbsSorted),
	       ( selected(verb, V)-> 
		   checkListBoxSetItemState(checkListBoxOfKind(verb), V, selected) ; true)).
	% END
/*
narrowOrRedrawTheVerbList(VisibleVerbsSorted):-
	\+selected(verb, _),
	\+thereIsASelectedObject,
	% we must group the zeroary verbs first
	checkListBoxClear(checkListBoxOfKind(verb)),
	groupVerbsByArity(Zero, OneOrMore, VisibleVerbsSorted),
	forall(member(M, Zero),
	       addWithProperIcon(verb, checkListBoxOfKind(verb), M)),
	length(Zero, L),
	length(OneOrMore, L2),
	( (L>0,L2>0)  -> checkListBoxAddSeparator(checkListBoxOfKind(verb)); true),
	forall(member(M, OneOrMore),
	       addWithProperIcon(verb, checkListBoxOfKind(verb), M)),
	% REGION set the selected/deselected state of the visible ones	
	forall(member(V, VisibleVerbsSorted),
	       ( selected(verb, V)-> 
		   checkListBoxSetItemState(checkListBoxOfKind(verb), V, selected) ; true)).
	% END
*/ 
narrowOrRedrawTheVerbList(VisibleVerbsSorted):-
	\+selected(verb, _),
	thereIsASelectedObject,
	% we must group the verbs by genericity
	checkListBoxClear(checkListBoxOfKind(verb)),
	groupVerbsByGenericity(GenericVerbs, ContextVerb, VisibleVerbsSorted),
	forall(member(M, ContextVerb),
	       addWithProperIcon(verb, checkListBoxOfKind(verb), M)),
	length(ContextVerb, L),
	length(GenericVerbs, L2),
	( (L>0,L2>0)  -> checkListBoxAddSeparator(checkListBoxOfKind(verb)); true),
	forall(member(M, GenericVerbs),
	       addWithProperIcon(verb, checkListBoxOfKind(verb), M)).

groupVerbsByArity(Zero, OneOrMore, VisibleVerbsSorted):-
	groupVerbsByArityAux([], [], ZeroUnsorted, OneOrMoreUnsorted, VisibleVerbsSorted),
	sortByNameInsensitive(ZeroUnsorted, Zero),
	sortByNameInsensitive(OneOrMoreUnsorted, OneOrMore).

groupVerbsByArityAux(Z, O, Z, O, []).
groupVerbsByArityAux(Zero1, One1, Zero3, One2, [H|T]):-
	\+verbRequiresAtLeastOneObjectSatisfying(H, _, _),
	Zero2 = [H | Zero1],
	!,
	groupVerbsByArityAux(Zero2, One1, Zero3, One2, T).
groupVerbsByArityAux(Zero1, One1, Zero2, One3, [H|T]):-
	verbRequiresAtLeastOneObjectSatisfying(H, _, _),
	One2 = [H | One1],
	!,
	groupVerbsByArityAux(Zero1, One2, Zero2, One3, T).


groupVerbsByGenericity(/*out*/GenericVerbsSorted, /*out*/ContextVerbsSorted, /*in*/VisibleVerbsSorted):-
	groupVerbsByGenericityAux([], [], GenericVerbs, ContextVerbs, VisibleVerbsSorted),
	sortByNameInsensitive(GenericVerbs, GenericVerbsSorted),
	sortByNameInsensitive(ContextVerbs, ContextVerbsSorted).
	
groupVerbsByGenericityAux(GenericVerbs, ContextVerbs, GenericVerbs, ContextVerbs, []).
groupVerbsByGenericityAux(CurGenericVerbs, CurContextVerbs, NextGenericVerbs, NextContextVerbs, [H|T]):-
	% REGION if the verb is generic, add it to the generic verbs list
	\+theVerbIsImportant(H),
	NextGenericVerbs2 = [H | CurGenericVerbs],
	% END
	!,
	groupVerbsByGenericityAux(NextGenericVerbs2, CurContextVerbs, NextGenericVerbs, NextContextVerbs, T).
groupVerbsByGenericityAux(CurGenericVerbs, CurContextVerbs, NextGenericVerbs, NextContextVerbs, [H|T]):-
	% REGION if the verb is contextual, add it to the contextual verbs list
	theVerbIsImportant(H),
	NextContextVerbs2 = [H | CurContextVerbs],
	% END
	!,
	groupVerbsByGenericityAux(CurGenericVerbs, NextContextVerbs2, NextGenericVerbs, NextContextVerbs, T).

theVerbIsImportant(_):-
	\+selected(file, _),
	\+selected(program, _).
theVerbIsImportant(V):-	
	selected(program, _),
	\+verbRequiresAtLeastOneObjectSatisfying(V, program, anyProgram).
theVerbIsImportant(V):-
	selected(file, _),
	\+verbRequiresAtLeastOneObjectSatisfying(V, file, anyFile).


updateItemsAfterSelectionChanged:-

	% REGION recompute the visible verbs
	findall(V, verb(V), Verbs),

	sort(Verbs, VerbsSorted),
	sublist(theVerbIsVisible, VerbsSorted, VisibleVerbsSorted),
	retractall(visibleVerb(_)),
	forall(member(M, VisibleVerbsSorted),
	       assert(visibleVerb(M))),
	% END
	
	narrowOrRedrawTheVerbList(VisibleVerbsSorted),
	retractall(visibleObject(_,_)),
	forall(program(P), decideObjectVisibility(P,program, VisibleVerbsSorted)),
	forall(file(F), decideObjectVisibility(F,file, VisibleVerbsSorted)),
	forall(device(D), decideObjectVisibility(D,device, VisibleVerbsSorted)),
	forall(directory(M), decideObjectVisibility(M,directory, VisibleVerbsSorted)),
	updateRedArrows(VisibleVerbsSorted).



onObjectDeselected(K, O):-
	retract(selected(K, O)),
	(K=time -> true ; ifItIsNowFilteredAwayRemoveItFromList(K, O)),
	((showPanelsWhenNoConceptIsSelected ; (selected(Q, _), Q\=time))->
	    updateItemsAfterSelectionChanged
	;  updateRedArrows([]))/*optimization*/,
	updateDeleteIdeasToolbarButtons,
	hidePanelsIfNothingIsSelected.

onIdeaDeselected(K, I):-
	writeln(onIdeaDeselected),
	retract(selected(idea-K, I)),
	((showPanelsWhenNoConceptIsSelected ; (selected(Q, _), Q\=time))->
	    updateItemsAfterSelectionChanged2
	;  updateRedArrows([]))/*optimization*/,
	
	updateDeleteIdeasToolbarButtons,
		writeln(onIdeaDeselected-3),
	% REGION only necessary for folder narrowing
	redrawFolderPanelHidingAndReshowing,
	% END
	writeln(onIdeaDeselected-4),
	hidePanelsIfNothingIsSelected,
	writeln(onIdeaDeselected-quit).
	

hidePanelsIfNothingIsSelected:-
	selected(K, _),
	K\=time.
hidePanelsIfNothingIsSelected:-
	showPanelsWhenNoConceptIsSelected.
hidePanelsIfNothingIsSelected:-
	forall(selected(K, _),K=time),
	\+showPanelsWhenNoConceptIsSelected,
	widgetHide(mainObjectVBox(device)),
	widgetHide(mainObjectVBox(directory)),
	widgetHide(mainObjectVBox(file)),
	widgetHide(verbBox),
	widgetHide(timeBox),
	widgetHide(mainObjectVBox(program)),
	widgetShow(scrolledWindowTutorial).

ifItIsNowFilteredAwayRemoveItFromList(K, O):-
	selected(idea-K, I),
	\+theObjectIsCompatibleWithTheIdea(I,K, O),
	checkListBoxSetItemState(checkListBoxOfKind(K), O, hidden).
ifItIsNowFilteredAwayRemoveItFromList(K, O):-
	forall(selected(idea-K, I),
	       theObjectIsCompatibleWithTheIdea(I, K, O)).

theObjectIsCompatibleWithTheIdea(I, K, O):-
	related(I, K, O).
theObjectIsCompatibleWithTheIdea(I, K, O):-
	visibleVerb(V),
	related(I,verb, V), % @@ better yet: theVerbIsCompatibleWithTheIdea(V, I)
	verbRequiresAtLeastOneObjectSatisfying(V, K, P),
	call(P, O).



updateChangeViewSentence(K, SelectedTables):-
	fromListToReadableSentence(SelectedTables, Sentence),
	concat_atom(['I want to view only those ', K, 's that have been dispatched as ', Sentence], Text),
	labelSetText(dynamicFilterLabel, Text).


% REGION fromListToReadableSentence/2. Translates [a,b,c] into a more readable 'a and b and c.'
fromListToReadableSentence([], '...').
fromListToReadableSentence(L, S):-
	L\=[],
	fromListToReadableSentenceAux('', L, S).
fromListToReadableSentenceAux(CurrentSentence, [H], New):-
	concat_atom([CurrentSentence, H, '.'], New).
fromListToReadableSentenceAux(CurrentSentence, L, New):-
	L\=[_],
	L=[H|T],
	concat_atom([CurrentSentence, H, ' and '], NewSentence),
	!,
	fromListToReadableSentenceAux(NewSentence, T, New).
% END

computeItemsToBeShown(K, T, Items):-
	K\=file,
	findall(I, (selected(K, I) ; related(T, K, I)),  Items).
computeItemsToBeShown(file, T, Items):-
	findall(O, (selected(file, O) ; related(T, file, O)) , Items).
	

checkListBoxSetItemVisibility(checkListBoxOfKind(K), I, visible):-
	selected(K, I),
	checkListBoxSetItemState(checkListBoxOfKind(K), I, selected).
checkListBoxSetItemVisibility(checkListBoxOfKind(K), I, visible):-
	\+selected(K, I),
	checkListBoxSetItemState(checkListBoxOfKind(K), I, deselected).
checkListBoxSetItemVisibility(checkListBoxOfKind(K), I, hidden):-
	checkListBoxSetItemState(checkListBoxOfKind(K), I, hidden).

addWithProperIcon(file, _CheckListBox, Name):-
	file_base_name(Name, BaseName),
	\+theFileHasAValidUtf8Name(BaseName),
	concat_atom(['Skipped file ', BaseName, ' because the name is not a valid UTF one'] , Text),
	writeln(Text).
addWithProperIcon(file, CheckListBox, Name):-
	file_base_name(Name, BaseName),
	theFileHasAValidUtf8Name(BaseName),
	file_directory_name(Name, Dir),
	file_name_extension(_, Ext, BaseName),
	downcase_atom(Ext, Ext_lowercase),
	typeToIcon(Ext_lowercase, Icon),
	time_file(Name, Date),
	convert_time(Date, ReadableDate),
	size_file(Name, Size),
	SizeInMB is Size /(1024*1024),
	concat_atom([SizeInMB, ' MB'], SizeInMBText),
	% REGION @@ slow... be sure that addWithProperIcon/3 is not called in critical loops
	findall(F, (idea(file, F), related(F, file, Name)), Folders),
	% END
	checkListBoxAddItem(CheckListBox, Name, BaseName, ''/*tooltip*/, Folders, Dir,SizeInMBText, ReadableDate, Icon).
addWithProperIcon(verb, CheckListBox, Name):-
	verbToIcon(Name, Icon),
	verbTooltip(Name, Tooltip),
	% REGION @@ slow... be sure that addWithProperIcon/3 is not called in critical loops
	findall(F, (idea(verb, F), related(F, verb, Name)), Folders),
	% END
	checkListBoxAddItem( CheckListBox, Name, Name, Tooltip, Folders, '', '', '',  Icon).
addWithProperIcon(directory, CheckListBox, Name):-
	checkListBoxAddItem( CheckListBox, Name, Name, '', [], '', '', '',  'img/folder.png').
addWithProperIcon(program, CheckListBox, Name):-
	programToIcon(Name, Icon),
	findall(F, (idea(program, F), related(F, program, Name)), Folders),
	checkListBoxAddItem(CheckListBox, Name, Name, ''/*tooltip*/,Folders, '', '', '',  Icon).
addWithProperIcon(device, CheckListBox, Name):-
	deviceToIcon(Name, Icon),
	findall(F, (idea(device, F), related(F, device, Name)), Folders),
	checkListBoxAddItem(CheckListBox, Name, Name, ''/*tooltip*/,Folders, '', '', '',Icon).




	
checkListBoxSetButtonCheckState(C, I, checked):-
	checkListBoxSetItemState(C, I, selected).
checkListBoxSetButtonCheckState(C, I, unchecked):-
	checkListBoxSetItemState(C, I, deselected).



	
	       


	
fromKindToLabel(file, labelCurrentFileFilter).
fromKindToLabel(device, labelCurrentDeviceFilter).
fromKindToLabel(program, labelCurrentProgramFilter).
fromKindToLabel(verb, labelCurrentVerbFilter).

messageBox(Text, Parent):-
	dialogNew(messageDialog, Parent, 'Message', 'img/icon.png'),
	vBoxNew(messageBoxBox),
	widgetShow(messageBoxBox),
	dialogAddToVBox(messageDialog, messageBoxBox),
	
	labelNew(messageLabel, Text, lineWrap),
	widgetShow(messageLabel),
	boxPackStart(messageBoxBox, messageLabel, expand, fill),


	hBoxNew(dialogHBox),
	widgetShow(dialogHBox),
	boxPackStart(messageBoxBox, dialogHBox, doNotExpand, fill, 10),
	
	buttonNew(messageOkButton, 'Ok'),
	widgetShow(messageOkButton),
	boxPackStart(dialogHBox, messageOkButton, expand, doNotFill),
	widgetShow(messageDialog),
	writeln(entering-messagebox-loop),
	messageBoxLoop.

messageBoxLoop:-
	sleepUntilEventIsRaised(E),
	( 
	  E = buttonClicked(messageOkButton)->
	    writeln(i-am-calling-widget-destroy), dialogClose(messageDialog);
	  E = dialogDestroyed(messageDialog)->
	    writeln(found-dialog-destroyed);
	  writeln(messageBoxLoop-ignoredEvent(E))),

	(E = dialogDestroyed(messageDialog)->
	    writeln(exitingMessageBoxLoop);
	 E = buttonClicked(messageOkButton)->
	    writeln(exitingMessageBoxLoop);
	 (!, messageBoxLoop)).


choiceBox(Id, Title, Text, OkText, CancelText):-
	dialogNew(Id, mainWindow, Title, 'img/icon.png'), 
	vBoxNew(choiceBoxBox(Id)),
	widgetShow(choiceBoxBox(Id)),
	dialogAddToVBox(Id, choiceBoxBox(Id)),
	
	labelNew(choiceLabel(Id), Text, lineWrap),
	widgetShow(choiceLabel(Id)),
	boxPackStart(choiceBoxBox(Id), choiceLabel(Id), expand, fill),

	imageButtonNew(yesButton(Id), 'img/hand-yes.png', OkText),
	widgetShow(yesButton(Id)),
	boxPackStart(choiceBoxBox(Id), yesButton(Id), doNotExpand, fill),

	imageButtonNew(noButton(Id), 'img/hand-no.png', CancelText),
	widgetShow(noButton(Id)),
	boxPackStart(choiceBoxBox(Id), noButton(Id), doNotExpand, fill),
	widgetShow(Id).
	

onDestroyMessageBox.
onMessageButtonPressed:-
	dialogClose(messageDialog).
/*
onOptionsButtonPressed:-
	dialogNew(optionsDialog, mainWindow, 'Options', 'img/options.png'),

	hBoxNew(dialogBox),
	widgetShow(dialogBox),
	dialogAddToVBox(optionsDialog, dialogBox),

	vBoxNew(leftVBox),
	widgetShow(leftVBox),
	boxPackStart(dialogBox, leftVBox, true, true, 30),

	labelNew(currentDirsLabel, 'On the right, you see the list of directories currently watched by segusoLand. This list determines the files that appear in segusoLand. You can watch and unwatch directories.\n\nYou can also associate a directory to one or more concepts. This way, if you have already catalogued your data into directories, you won\'t need to catalog them again into concepts.', lineWrap),
	widgetShow(currentDirsLabel),
	boxPackStart(leftVBox, currentDirsLabel, doNotExpand, fill),

	checkListBoxNew(currentDirsCheckListBox, relief, 'directories', selectedListHidden),
	widgetShow(currentDirsCheckListBox),
	boxPackStart(dialogBox, currentDirsCheckListBox, expand, fill),

	% REGION fill the currentDirsCheckListBox with the current dirs
	findall(D, segusoDir(D), Dirs),
	sortByNameInsensitive(Dirs, DirsSorted),
	forall(member(M, DirsSorted), checkListBoxAddItem(currentDirsCheckListBox,  M, M, '',
							  [],'', '', '', 'img/folder.png')),
	% END

	
	imageButtonNew(addNewDirButton, 'img/folder.png', 'Watch a new directory'),
	widgetShow(addNewDirButton),
	widgetSetTooltip(addNewDirButton, 'If you watch a new directory, all its files will appear in segusoLand'),
	boxPackStart(leftVBox, addNewDirButton, doNotExpand, doNotFill, 10),

	imageButtonNew(removeDirButton,'img/delete-folder.png', 'Stop watching this directory'),
	widgetShow(removeDirButton),
	widgetSetTooltip(removeDirButton, 'If you click here, the selected directories will be unwatched, which means that the files contained in these directories will not appear into segusoLand anymore.\n\nBEWARE: The files themselves will not be deleted (of course), but any catalogation of the files into "concepts" will be lost. If you have catalogued these files into concepts inside segusoLand, you will have to do it again.'),
	boxPackStart(leftVBox, removeDirButton, doNotExpand, doNotFill, 10),
	widgetDisable(removeDirButton),

	imageButtonNew(catalogDirButton,'img/edit-folders.png', 'Associate this directory to one or more concepts'),
	widgetShow(catalogDirButton),
	boxPackStart(leftVBox, catalogDirButton, doNotExpand, doNotFill, 10),
	widgetSetTooltip(catalogDirButton, 'With this button you can associate all the files contained in the directory to one or more "concepts".\n\nThis button is useful, for example, if you already have a list of songs, catalogued by album, and do not want to catalog them again into concepts.'),
	widgetDisable(catalogDirButton),

	windowMaximize(optionsDialog),
	widgetShow(optionsDialog),
	optionDialogLoop([]).

optionDialogLoop(SelectedDirs):-
	var(SelectedDirs),
	throw(error-var-SelectedDirs).
optionDialogLoop(SelectedDirs):-
	nonvar(SelectedDirs),
	sleepUntilEventIsRaised(E),
	(E=checkListBoxItemSelected(currentDirsCheckListBox, I)->
	    onDirSelected(I, SelectedDirs, NewSelectedDirs)
	; E=checkListBoxItemDeselected(currentDirsCheckListBox, I)->
	    onDirDeselected(I, SelectedDirs, NewSelectedDirs)
	; E=buttonClicked(addNewDirButton)->
	    (	fileOrDirSelection('Please select a directory from the above list, then click OK', NewDir),
		onAddNewDirButtonPressed(NewDir), SelectedDirs = NewSelectedDirs);
	E=buttonClicked(removeDirButton)->
	    onRemoveDirButtonPressed(SelectedDirs, NewSelectedDirs);
	E = buttonClicked(catalogDirButton)->
	     (onCatalogDirsButtonPressed(SelectedDirs), NewSelectedDirs = SelectedDirs);
	E=dialogDestroyed(optionsDialog)->
	    (NewSelectedDirs = SelectedDirs);
	(writeln(discardedEvent(E)), NewSelectedDirs = SelectedDirs)),
	
	(E=dialogDestroyed(optionsDialog)->
	     writeln(exitingDialogLoop);
	 (!, optionDialogLoop(NewSelectedDirs))).
*/

% do not delete! it is part of the dir dialog
editFeaturesForNewDirLoop2(SelectedTables, _Dirs):-
	var(SelectedTables),
	throw(error).
editFeaturesForNewDirLoop2(SelectedTables, Dirs):-
	nonvar(SelectedTables),

	sleepUntilEventIsRaised(E),
	(editFeaturesForNewDirs_ManageEvent(E, SelectedTables, NewSelectedTables, Dirs)->true; NewSelectedTables = SelectedTables),
 
	(E = dialogDestroyed(editFeaturesDialog3)->   writeln(exiting-loop-3)
	; (!, editFeaturesForNewDirLoop2(NewSelectedTables, Dirs))).

editFeaturesForNewDirs_ManageEvent(checkListBoxItemSelected(allFeaturesCheckListBox, T), S, NewS, _Dirs):-
	NewS = [T|S].
editFeaturesForNewDirs_ManageEvent(checkListBoxItemDeselected(allFeaturesCheckListBox, T), S, NewS, _Dirs):-
	remove(T, S, NewS).

editFeaturesForNewDirs_ManageEvent(dialogDestroyed(editFeaturesDialog3), [], [], _).
editFeaturesForNewDirs_ManageEvent(dialogDestroyed(editFeaturesDialog3), SelectedTables, SelectedTables, Dirs):-
	SelectedTables\=[],
	forall(member(M, Dirs),
	       addAllFilesInDirToGivenTablesIfTheyAreNotAlreadyInThere(SelectedTables, M)).
addAllFilesInDirToGivenTablesIfTheyAreNotAlreadyInThere(Tables, NewDir):-
	% NewDir is already watched, but it might contain some new files, so the files
	% may or may not be in the All Files  table.
	addSlashToDirIfNeeded(NewDir, NewDirSlashed),
	concat_atom([NewDirSlashed, '*'], VisibleFilesQuery),
	concat_atom([NewDirSlashed, '.*'], HiddenFilesQuery),
	expand_file_name(VisibleFilesQuery, VisibleFiles),
	expand_file_name(HiddenFilesQuery, HiddenFiles),
	append(VisibleFiles, HiddenFiles, AllFiles),
	forall( (member(M, AllFiles), exists_file(M)/*skip .  .. and dirs */),
		(assertIfNotAlreadyThere(file(M)),
		    forall(member(T, Tables),
			   assertIfNotAlreadyThere(related(T, file, M))))),
	% REGION add to ext tables if needed
	forall(member(M, AllFiles),
	       assertExtensionIfAnyAndNotAlreadyAsserted(M)),
	
	% END

	% fill checkListBox with only NEW items. If you don't put them , it will crash at the next sort
	forall(member(M, AllFiles),
	       (checkListBoxItemExists(checkListBoxOfKind(file), M)-> true;
		   addWithProperIcon(file, checkListBoxOfKind(file), M))).

assertExtensionIfAnyAndNotAlreadyAsserted(FileName):-
	file_name_extension(_, '', FileName) .
assertExtensionIfAnyAndNotAlreadyAsserted(N):-
	file_name_extension(_, Ext, N),
	Ext\='',
	nonvar(Ext),
	atom(Ext),
	%verbRequiresAtLeastOneObjectSatisfying(_, ext(Ext)),  % if the extension is useful.... @@ doesn't work because there
	% areindirect extensions, such as videoFile... We could transform videoFile/1 into ext(videoFile, _), and it would work.
	downcase_atom(Ext, Ext_lowercase),
	assertIfNotAlreadyThere(ext(Ext_lowercase, N)).

/*

onAddNewDirButtonPressed(NewDir):-
	iAmWatchingTheDir(NewDir),
	concat_atom(['segusoLand is ALREADY watching the directory ', NewDir, '!'], T),
	messageBox(T, mainWindow).
onAddNewDirButtonPressed(NewDir):- 
	\+iAmWatchingTheDir(NewDir),

	% REGION ask whether you want to give all files one or more attributes
	dialogNew(addAttributeToAllFilesInDirDialog, mainWindow, 'Catalog the new files?', 'img/icon.png'),

	vBoxNew(addAttributeToAllFilesInDirDialogBox),
	widgetShow(addAttributeToAllFilesInDirDialogBox),
	dialogAddToVBox(addAttributeToAllFilesInDirDialog, addAttributeToAllFilesInDirDialogBox),

	concat_atom(['You have chosen to watch the directory ', NewDir, '.\n\nSegusoLand is now going to import the files contained in ', NewDir, '. You can now choose to associate the files with one or more "concepts", in order to reach them more easily. For example, you could associate them to the concepts \"Music\", \"Classical\", \"by Richard Wagner\". If you decide not to do this now, the files will still be accessible, and you will be able to associate them later. \n\nWhat do you want to do?'], Text),

	labelNew(testLabel, Text, lineWrap),
	widgetShow(testLabel),
	boxPackStart(addAttributeToAllFilesInDirDialogBox, testLabel, expand, fill),

	buttonNew(markTheFilesButton,
		  'Associate the files to one or more concepts I will select shortly.'),
	widgetShow(markTheFilesButton),
	boxPackStart(addAttributeToAllFilesInDirDialogBox, markTheFilesButton, doNotExpand, fill),
	
	buttonNew(doNotMarkTheFilesButton,
		  'Do not catalog the files.'),
	widgetShow(doNotMarkTheFilesButton),
	boxPackStart(addAttributeToAllFilesInDirDialogBox, doNotMarkTheFilesButton, doNotExpand, fill),
	widgetShow(addAttributeToAllFilesInDirDialog),
	% END
	markFileLoop(NewDir).
*/
markFileLoop(NewDir):-
	sleepUntilEventIsRaised(E),
	( E = buttonClicked(markTheFilesButton)->
	    onMarkTheFilesNowPressed(NewDir);
	  E= buttonClicked(doNotMarkTheFilesButton)->
	    onDoNotMarkTheFilesNowPressed(NewDir);
	  E = dialogDestroyed(addAttributeToAllFilesInDirDialog)->
	    writeln(exitingLoop);
	  (!, markFileLoop(NewDir))).



editFeaturesForNewDirLoop(SelectedTables, _):-
	var(SelectedTables),
	throw(error-var-SelectedTables).
editFeaturesForNewDirLoop(SelectedTables, NewDir):-
	nonvar(SelectedTables),
	sleepUntilEventIsRaised(E),
	( E = checkListBoxItemDeselected(allFeaturesCheckListBox, I)->
	      onFeatureDeselectedWhenAddingFeaturesToAllFiles(I, SelectedTables, NewSelectedTables);
	  E = checkListBoxItemSelected(allFeaturesCheckListBox, I)->
	      onFeatureSelectedWhenAddingFeaturesToAllFiles(I, SelectedTables, NewSelectedTables);
	  E = buttonClicked(addSelectedFeatureButton)->
	      (onAddSelectedFeatureButtonPressed2(SelectedTables), NewSelectedTables = SelectedTables);
	  E = entryTextChanged(newFeatureTextBox)->
	     (onNewFeatureTextBoxTextChanged,
              SelectedTables=NewSelectedTables);
	  E = dialogDestroyed(editFeaturesDialog2)->
    	     (onSelectFeatureDialogWhenAddingNewFilesDestroyed(SelectedTables, NewDir),
	      SelectedTables = NewSelectedTables);
	 (SelectedTables = NewSelectedTables, writeln(ignoredEvent(E)))),
 
	(E=dialogDestroyed(editFeaturesDialog2)->
	    writeln(exitingLoop);
	 (!, editFeaturesForNewDirLoop(NewSelectedTables, NewDir))).






onFeatureSelectedWhenAddingFeaturesToAllFiles(Feature, SelectedTables, NewSelectedTables):- % features are really tables, hence the name
	NewSelectedTables = [Feature | SelectedTables].
onFeatureDeselectedWhenAddingFeaturesToAllFiles(Feature, SelectedTables, NewSelectedTables):- % features are really tables, hence the name
	remove(Feature, SelectedTables, NewSelectedTables).

onSelectFeatureDialogWhenAddingNewFilesDestroyed([], _).
onSelectFeatureDialogWhenAddingNewFilesDestroyed(SelectedTables, NewDir):-
	SelectedTables\=[],
	% I must now add the files to the given tables.
	%entryGetText(newDirTextBox, NewDir),
	assert(segusoDir(NewDir)),
	addFilesOfDirToAllFilesTable_HiddenFilesTable_ExtTable(NewDir),
	% REGION fill the selected tables
	addSlashToDirIfNeeded(NewDir, NewDirSlashed),
	concat_atom([NewDirSlashed, '*'], VisibleFilesQuery),
	concat_atom([NewDirSlashed, '.*'], HiddenFilesQuery),
	expand_file_name(VisibleFilesQuery, VisibleFiles),
	expand_file_name(HiddenFilesQuery, HiddenFiles),
	writeln('HiddenFilesQuery'(HiddenFilesQuery)),
	forall((member(V, VisibleFiles), exists_file(V)/*this is to skip . and ..  and directories! */),
	       forall(member(T, SelectedTables),
		      assert(related(T, file, V)))),
	forall((member(H, HiddenFiles),exists_file(H)),
	       forall(member(T2, SelectedTables),
		      assert(related(T2, file, H)))),
	% END

	% REGION insert the new files into the listbox
	forall((member(V2, VisibleFiles), exists_file(V2)),
	       addWithProperIcon(file, checkListBoxOfKind(file), V2)),
	forall((member(H2, HiddenFiles), exists_file(H2)),
	       addWithProperIcon(file, checkListBoxOfKind(file), H2)),
	% END
	% REGION add the new dir to the list of dirs
	checkListBoxAddItem(currentDirsCheckListBox, NewDir, NewDir, ''/*tooltip*/,
			    []/*folders make no sense*/,
			    '', '', '', 'img/folder.png').

importDirectory(NewDir, filesNotCalculated /*out*/):-
	iAmWatchingTheDir(NewDir),
	messageBox('segusoLand is already watching this directory!', mainWindow).
importDirectory(/*in*/NewDir, Files/*out*/):-
	\+iAmWatchingTheDir(NewDir),
	writeln(importDirectory(NewDir)),
	assert(segusoDir(NewDir)),
	addFilesOfDirToAllFilesTable_HiddenFilesTable_ExtTable(NewDir),
	% REGION fill the Files-to-dispatch table
	addSlashToDirIfNeeded(NewDir, NewDirSlashed),
	concat_atom([NewDirSlashed, '*'], VisibleFilesQuery),
	concat_atom([NewDirSlashed, '.*'], HiddenFilesQuery),
	expand_file_name(VisibleFilesQuery, VisibleFiles),
	expand_file_name(HiddenFilesQuery, HiddenFiles),
	% END
	writeln(importDirectory-addingVisibleItemsToList),
	% REGION insert the new items into the listbox
	forall((member(V2, VisibleFiles), exists_file(V2), theFileHasAValidUtf8Name(V2)),
	       addWithProperIcon(file, checkListBoxOfKind(file), V2)),
	%writeln(importDirectory-addingHiddenItemsToList-HiddenFiles),
	forall((member(H2, HiddenFiles), exists_file(H2), theFileHasAValidUtf8Name(H2)),
	       addWithProperIcon(file, checkListBoxOfKind(file), H2)),
	% END
	writeln(importDirectory-appendingLists),
	append(VisibleFiles, HiddenFiles,  Files).
findAllFilesInDir(D, Files):-
	addSlashToDirIfNeeded(D, DirSlashed),
	concat_atom([DirSlashed, '*'], VisibleFilesQuery),
	concat_atom([DirSlashed, '.*'], HiddenFilesQuery),
	expand_file_name(VisibleFilesQuery, VisibleFiles),
	expand_file_name(HiddenFilesQuery, HiddenFiles),
	append(HiddenFiles, VisibleFiles, Files).
	
importDirectoryRecursively(D):-
	writeln(importDirectoryRecursively(D)),
	(iAmWatchingTheDir(D)->findAllFilesInDir(D, Files);
	    importDirectory(D, Files /* out*/)),
	% import children
	forall((member(Child, Files), exists_directory(Child), file_base_name(Child, B), B\='.', B\='..'),
	       importDirectoryRecursively(Child)).
	
redrawDirPanelClearingAndReaddingEverything:-
	checkListBoxClear(checkListBoxOfKind(directory)),
	findall(D, segusoDir(D), Dirs),
	sort(Dirs, Dirs2),
	forall(member(M, Dirs2),
	       checkListBoxAddItem(checkListBoxOfKind(directory), M, M,
				   ''/*tooltip*/,
				   []/*make no sense*/,
				   '', '', '', 'img/folder.png')),
        forall(selected(directory, SD),
		checkListBoxSetItemState(checkListBoxOfKind(directory), SD, selected)),
	checkListBoxDefeatBug(checkListBoxOfKind(directory)).

stopWatchingDirs(SelectedDirs):-
	% REGION remove all files in those dirs dir from any table!
	findall(F, (member(D2, SelectedDirs), fileInDir(F, D2)), FilesToRemove),
	sort(FilesToRemove, FilesToRemoveWithoutRepetitions),
	removeFilesFromCheckListBoxAndDatabase(FilesToRemoveWithoutRepetitions), % also redraws the list
	% END
		writeln(debug-d-4),
	forall(member(D3, SelectedDirs), (writeln(retracting(D3)),retract(segusoDir(D3)))),
	writeln(debug-d-5),
	forall(member(D3, SelectedDirs),
	       checkListBoxDeleteItem(checkListBoxOfKind(directory), D3)),
	writeln(debug-d-6).

fileInDir(F, D):- 
	file(F),
	file_directory_name(F, D1),
	concat_atom([D1, '/'], D2),
	expand_file_name(D, [DExp]),
	(D=D2; D=D1; DExp = D2; DExp = D1).
removeFilesFromCheckListBoxAndDatabase(L):- % L is a list of files with full path name
	% REGION remove the file from all tables where it could be. BUT NOT DERIVED TABLES.
	forall(member(F1, L),
	       retract(file(F1))),
	forall(member(F1, L),
	       retractall(related(_, file, F1))),
	forall(member(F2, L),
	       retractall(ext(_, F2))),
	% END
	% REGION remove the file from the listbox.
	%        Remove it for real, do not just hide. Otherwise at THE NEXT SORT IT WILL CRASH!
	forall(member(F5, L),
	       checkListBoxDeleteItem(checkListBoxOfKind(file), F5)).
	% END



	
% main procedure, called by file.cc.
run(DirWhereTheSubdirectory_Img_isLocated):-
	% REGION set the work dir  because it is boring to concat this dir each time I need to read an image. This way
	%  I simply say img/file.png
	working_directory(_, DirWhereTheSubdirectory_Img_isLocated),
	% END

	assertDefaultShit,

	PaddingForQuickSearchBoxes = 7,

	% REGION read database
	writeln(reading-database),
	createDbIfItDoesNotExist,
	expand_file_name('~', [HomeDir]),
	concat_atom([HomeDir, '/.segusoLand-database'], DatabaseFileNamewithFullPath),
	open(DatabaseFileNamewithFullPath, read, Dat),
	readLoop(Dat), % writes related(Table, Kind, File) and segusoDir(D)
	close(Dat),
	writeln(finished-reading-database),
	% END

	windowNew(mainWindow, 'segusoLand - created by Maurizio Colucci', 'img/icon.png'),

	
	vBoxNew(mainBoxVertical),
	containerAdd(mainWindow, mainBoxVertical),
	widgetShow(mainBoxVertical),


	hBoxNew(menuAndToolbarAndImageHBox),
	widgetShow(menuAndToolbarAndImageHBox),
	boxPackStart(mainBoxVertical, menuAndToolbarAndImageHBox, doNotExpand, fill),


	frameNew(dropdownListFrame),
	widgetShow(dropdownListFrame),
	boxPackStart(menuAndToolbarAndImageHBox, dropdownListFrame, doNotExpand, fill),
	hBoxNew(dropDownHBox),
	widgetShow(dropDownHBox),


	
	optionMenuNew(timeOptionMenu),
	buttonSetReliefNone(timeOptionMenu),
	forall( time( X),
	       optionMenuAddItem(timeOptionMenu, X, X)),
	(panelStateMinimized(time)->widgetShow(timeOptionMenu); true),
	
	(panelStateMinimized(time)->optionMenuSetSelectedItem(timeOptionMenu, 'Now'); true),
	boxPackStart(dropDownHBox, timeOptionMenu, doNotExpand, fill),

	imageButtonNew(putTimeToPanelButton, 'img/cross.png', ''),
	widgetSetSizeRequest(putTimeToPanelButton, 33, 33),
	widgetSetTooltip(putTimeToPanelButton, 'Click here to restore the time panel.'),
	(panelStateMinimized(time)->widgetShow(putTimeToPanelButton); true),
	boxPackStart(dropDownHBox, putTimeToPanelButton, doNotExpand, fill),


	


	frameNew(toolbarFrame),
	widgetShow(toolbarFrame),
	boxPackStart(menuAndToolbarAndImageHBox, toolbarFrame, expand, fill),

	containerAdd(dropdownListFrame, dropDownHBox),

	hBoxNew(fileSortBox),
	widgetShow(fileSortBox),
	containerAdd(toolbarFrame, fileSortBox),

	imageButtonNew(deselectAllButton, 'img/deselect-all.png', 'Deselect all'),
	widgetSetSizeRequest(deselectAllButton, -1, 42),
	buttonSetReliefNone(deselectAllButton),
	widgetSetTooltip(deselectAllButton,
			 'Deselect everything except the selected time. You usually click here to begin a new action.'),
	widgetShow(deselectAllButton),
	boxPackStart(fileSortBox, deselectAllButton, doNotExpand, doNotFill),

	vSeparatorNew(recentButtonsSeparatorLeft),
	widgetShow(recentButtonsSeparatorLeft),
	boxPackStart(fileSortBox, recentButtonsSeparatorLeft, doNotExpand, fill, 5),
	
	labelNew(recentLabel, 'Recent:', noLineWrap),
	widgetShow(recentLabel),
	boxPackStart(fileSortBox, recentLabel, doNotExpand, doNotFill),

	imageNew(recentConceptsRedArrow, 'img/redarrow-right.gif'),
	boxPackStart(fileSortBox, recentConceptsRedArrow, doNotExpand,doNotFill),

	imageButtonNew(recentButton-concept, 'img/bulb.png', 'concepts'),
	widgetShow(recentButton-concept),
	widgetSetTooltip(recentButton-concept,  'Allows you to quickly select recently used concepts. It is faster than the "index of concepts".'),
	buttonSetReliefNone(recentButton-concept),
	boxPackStart(fileSortBox, recentButton-concept, doNotExpand, doNotFill),



	imageButtonNew(recentButton-action,'img/action.png',  'actions'),
	widgetSetTooltip(recentButton-action, 'Allows you to quickly repeat an action. Most useful for sorting files, refreshing the file list, etc.'),
	widgetShow(recentButton-action),
	buttonSetReliefNone(recentButton-action),
	boxPackStart(fileSortBox, recentButton-action, doNotExpand, doNotFill),

	imageNew(recentConceptsRedArrowRight, 'img/redarrow-left.gif'),
	boxPackStart(fileSortBox, recentConceptsRedArrowRight, doNotExpand,doNotFill),

/*
	imageButtonNew(recentButton-file, 'img/text-file.png', 'files'),
	widgetShow(recentButton-file),
	buttonSetReliefNone(recentButton-file),
	boxPackStart(fileSortBox, recentButton-file, doNotExpand, doNotFill),
*/
	vSeparatorNew(recentButtonsSeparatorRight),
	widgetShow(recentButtonsSeparatorRight),
	boxPackStart(fileSortBox, recentButtonsSeparatorRight, doNotExpand, fill, 5),

	hBoxNew(spacerBeforeGlobalGoButton),
	widgetSetSizeRequest(spacerBeforeGlobalGoButton, 15, -1),
	widgetShow(spacerBeforeGlobalGoButton),
	boxPackStart(fileSortBox, spacerBeforeGlobalGoButton, doNotExpand, doNotFill),

	
	imageNew(globalGoRedArrow, 'img/redarrow-right.gif'),
	boxPackStart(fileSortBox, globalGoRedArrow, doNotExpand,doNotFill),

	buttonNew(globalGoButton, 'Execute the action'),
	boxPackStart(fileSortBox, globalGoButton, doNotExpand, doNotFill),
	widgetSetTooltip(globalGoButton, 'Press this button to execute the action you composed, or select something else from the panels below.'),

	imageNew(globalGoRedArrowLeft, 'img/redarrow-left.gif'),
	boxPackStart(fileSortBox, globalGoRedArrowLeft, doNotExpand,doNotFill),

	hBoxNew(spacerAfterGlobalGoButton),
	widgetSetSizeRequest(spacerAfterGlobalGoButton, 15, -1),
	widgetShow(spacerAfterGlobalGoButton),
	boxPackStart(fileSortBox, spacerAfterGlobalGoButton, doNotExpand, doNotFill),



	imageButtonNew(deleteIdeaButton, 'img/bulb-delete.png', ''),
	buttonSetReliefNone(deleteIdeaButton),
	widgetSetTooltip(deleteIdeaButton, 'Click here to delete the selected concepts. You will loose any association between those concepts and files/verbs/devices/programs.'),
	boxPackStart(fileSortBox, deleteIdeaButton, doNotExpand, doNotFill, 5),


	

	
        % END


	
	% END

	hBoxNew(mainBox),
	widgetShow(mainBox),
	containerAdd(mainBoxVertical, mainBox),

	vBoxNew(mainObjectVBox(idea)),
	widgetShow(mainObjectVBox(idea)),
	boxPackStart(mainBox, mainObjectVBox(idea), doNotExpand, fill),


		

	vBoxNew(verbBox),
	(showPanelsWhenNoConceptIsSelected->widgetShow(verbBox); true),
	boxPackStart(mainBox, verbBox, expand, fill),


	
	vBoxNew(mainObjectVBox(file)),
	((showPanelsWhenNoConceptIsSelected, \+panelStateMinimized(file))->widgetShow(mainObjectVBox(file)); true),
	boxPackStart(mainBox, mainObjectVBox(file),  expand, fill),



	vBoxNew(mainObjectVBox(program)),
	((showPanelsWhenNoConceptIsSelected, \+panelStateMinimized(program))->widgetShow(mainObjectVBox(program)); true),
	boxPackStart(mainBox, mainObjectVBox(program), expand, fill),



	vBoxNew(mainObjectVBox(device)),
	((showPanelsWhenNoConceptIsSelected, \+panelStateMinimized(device))->widgetShow(mainObjectVBox(device)); true),
	boxPackStart(mainBox, mainObjectVBox(device),  expand, fill),

	vBoxNew(mainObjectVBox(directory)),
	((showPanelsWhenNoConceptIsSelected, \+panelStateMinimized(directory))->widgetShow(mainObjectVBox(directory)); true),
	boxPackStart(mainBox, mainObjectVBox(directory), expand, fill),


	vBoxNew(timeBox),
	((showPanelsWhenNoConceptIsSelected, \+panelStateMinimized(time))->widgetShow(mainObjectVBox(time)); true),
	boxPackStart(mainBox, timeBox,  expand, fill),

	labelNew(generalMessageLabel, 'Welcome to segusoLand, a program for lazy people who know WHAT to do with their computer, but do not know HOW to do it and don\'t want to spend time learning it.\n\nThe main purpose of segusoLand is to make the transition to Linux immediate. The idea is that segusoLand must not be learnt: you tell segusoLand what you want to do with your own words.\n\nAnother purpose of segusoLand is to allow you to do any task the same way; you play a music the same way you shut down your computer. You rename a file the same way you repartition the hard disk. So you don\'t need to relearn everything from scratch, and keep in mind many different techniques to do things. Once you know how to do something, you know how to do everything.\n\nThis is all you need to know to use segusoLand: \n\n* Decide what you want to do. For example "Listen to some good music by Tracy Chapman".\n\n* Mentally pick some words from the sentence (verbs or nouns). For example, "listen", "good" and "Chapman".\n\n* Select ALL these words from the "index of concepts" on the left. Don\'t browse the other words. Just go straight to the word you thought. Like in a dictionary. (If you don\'t find the words, please email the authors and the words will be added.) \n\n* Some lists will pop up, hiding this message. Select things from the lists (a red arrow will help you). When you have selected enough things, a button called "execute" will appear. Click it: the music will be played.\n\nJust remember that, by default, segusoLand only knows the files which are in your home directory (~). So you will not find the words "Chapman" and "good" if there is no file in your home directory whose name or path contains those words. However, it is easy to import other directories into segusoLand. So easy we are not going to tell you how!', lineWrap),

	widgetShow(generalMessageLabel),
	scrolledWindowNew(scrolledWindowTutorial),
	widgetShow(scrolledWindowTutorial),
	boxPackStart(mainBox, scrolledWindowTutorial, expand, fill),



	
	(showPanelsWhenNoConceptIsSelected->true;widgetShow(scrolledWindowTutorial)),
	containerAdd(scrolledWindowTutorial, generalMessageLabel),


	% REGION fill verbs
	writeln(verbs),

	% REGION substring box
	hBoxNew(filterHBox(verb)),
	boxPackStart(verbBox, filterHBox(verb), doNotExpand, fill, PaddingForQuickSearchBoxes),

	
	
	entryNew(containingTextEntry(verb)),
	widgetShow(containingTextEntry(verb)),
	boxPackStart(filterHBox(verb), containingTextEntry(verb), expand, fill, PaddingForQuickSearchBoxes),
	widgetSetTooltip(containingTextEntry(verb), 'SUBSTRING SEARCH: If you enter some text here, only verbs whose name contains that text will be shown. Beware: the visible verbs determine the visible objects, devices, and programs.'),



	imageButtonNew(deleteSubstringFilterButton(verb), 'img/delete-mbuto.png', ''),
	widgetShow(deleteSubstringFilterButton(verb)),
	boxPackStart(filterHBox(verb), deleteSubstringFilterButton(verb), doNotExpand, fill),
	widgetSetTooltip(deleteSubstringFilterButton(verb), 'Remove the current substring filter'),

	% END substring box

	frameNew(verbFrame),
	boxPackStart(verbBox, verbFrame, doNotExpand, fill),
	
	hBoxNew(verbFilterBox),
	widgetShow(verbFilterBox),
	containerAdd(verbFrame, verbFilterBox),




	hBoxNew(labelAndArrowBox(verb)),
	widgetShow(labelAndArrowBox(verb)),
	boxPackStart(verbBox, labelAndArrowBox(verb), doNotExpand, fill),

	eventBoxNew(arrowEventBox(verb)),
	widgetShow(arrowEventBox(verb)),
	boxPackStart(labelAndArrowBox(verb), arrowEventBox(verb), doNotExpand, fill),
	imageNew(arrowImage(verb), 'img/empty.png'),
	containerAdd(arrowEventBox(verb), arrowImage(verb)),
	widgetShow(arrowImage(verb)),
	miscSetPadding(arrowImage(verb), 10, 0),	
	miscSetAlignment(arrowImage(verb), 1.0, 0.5),
	

	labelNew(currentViewLabel(verb), 'Actions', noLineWrap),
	widgetShow(currentViewLabel(verb)),
	boxPackStart(labelAndArrowBox(verb), currentViewLabel(verb), expand, fill),


	imageNew(arrowImageGoButton, 'img/empty.png'),
	widgetShow(arrowImageGoButton),
	boxPackStart(labelAndArrowBox(verb), arrowImageGoButton, doNotExpand, fill),

	% REGION Add a fake box because the image is too rightwise
	vBoxNew(youCanClickGo),
	widgetShow(youCanClickGo),
	widgetSetSizeRequest(youCanClickGo, 20, 1),
	boxPackStart(labelAndArrowBox(verb), youCanClickGo, doNotExpand, fill),
	% END
	
	checkListBoxNew(checkListBoxOfKind(verb), relief, 'verbs', selectedListHidden),
	widgetShow(checkListBoxOfKind(verb)),
	boxPackStart(verbBox, checkListBoxOfKind(verb), expand, fill),

	findall(V, verb(V), Verbs),
	sortByNameInsensitive(Verbs, Verbs2),
	forall(member(X, Verbs2),
	       addWithProperIcon(verb, checkListBoxOfKind(verb), X)),

	% REGION verb sort box
	hBoxNew(verbSortBox),
	boxPackStart(verbBox, verbSortBox, doNotExpand, fill),
	
	labelNew(verbSortLabel, 'Sort by ', noLineWrap),
	widgetShow(verbSortLabel),
	boxPackStart(verbSortBox, verbSortLabel, doNotExpand, fill),
	
	imageButtonNew(sortByVerbNameButton, 'img/name.png', ''),
	widgetShow(sortByVerbNameButton),
	boxPackStart(verbSortBox, sortByVerbNameButton, expand, fill),
	widgetDisable(sortByVerbNameButton),

	imageButtonNew(sortByVerbUsageButton, 'img/1.png', ''),
	widgetShow(sortByVerbUsageButton),
	boxPackStart(verbSortBox, sortByVerbUsageButton, expand, fill),
	widgetDisable(sortByVerbUsageButton),

	imageButtonNew(sortByVerbRelevanceButton, 'img/size.png', ''),
	widgetShow(sortByVerbRelevanceButton),
	boxPackStart(verbSortBox, sortByVerbRelevanceButton, expand, fill),
	widgetDisable(sortByVerbRelevanceButton),
	% END

	
        % END
	% REGION file panel
	writeln(file-panel),
	% REGION substring box
	hBoxNew(filterHBox(file)),
	%widgetShow(filterHBox(file)),
	boxPackStart(mainObjectVBox(file), filterHBox(file), doNotExpand, fill, PaddingForQuickSearchBoxes),




	entryNew(containingTextEntry(file)),
	widgetShow(containingTextEntry(file)),
	boxPackStart(filterHBox(file), containingTextEntry(file), expand, fill, PaddingForQuickSearchBoxes),
	widgetSetTooltip(containingTextEntry(file), 'SUBSTRING SEARCH: If you enter "dsd123" here, only the concepts whose name contains "dsd123" will be shown.'),

	imageButtonNew(deleteSubstringFilterButton(file), 'img/delete-mbuto.png', ''),
	widgetShow(deleteSubstringFilterButton(file)),
	boxPackStart(filterHBox(file), deleteSubstringFilterButton(file), doNotExpand, fill),
	widgetSetTooltip(deleteSubstringFilterButton(file), 'Remove the current substring filter'),

	% END substring box

	% REGION expert file filter box

	frameNew(fileFrame),
	boxPackStart(mainObjectVBox(file), fileFrame, doNotExpand, fill),

	hBoxNew(filterBox),
	widgetShow(filterBox),
	containerAdd(fileFrame, filterBox),


	
	hBoxNew(labelAndArrowBox(file)),
	widgetShow(labelAndArrowBox(file)),
	boxPackStart(mainObjectVBox(file), labelAndArrowBox(file), doNotExpand, fill),

	eventBoxNew(arrowEventBox(file)),
	widgetShow(arrowEventBox(file)),
	boxPackStart(labelAndArrowBox(file), arrowEventBox(file), doNotExpand, fill),
	imageNew(arrowImage(file), 'img/empty.png'),
	containerAdd(arrowEventBox(file), arrowImage(file)),
	widgetShow(arrowImage(file)),
	miscSetPadding(arrowImage(file), 10, 0),	
	miscSetAlignment(arrowImage(file), 1.0, 0.5),


	labelNew(currentViewLabel(file), 'Files', noLineWrap),
	widgetShow(currentViewLabel(file)),
	boxPackStart(labelAndArrowBox(file), currentViewLabel(file), expand, fill),


	imageButtonNew(putToDropdownButton(file), 'img/cross-transparent.png', ''),
	widgetShow(putToDropdownButton(file)),
	buttonSetReliefNone(putToDropdownButton(file)),
	widgetSetTooltip(putToDropdownButton(file),
			 'Hide the file panel. It will be shown automatically when needed, i.e. when you select a verb which requires a file.'),
	boxPackStart(labelAndArrowBox(file), putToDropdownButton(file), doNotExpand, fill),


	checkListBoxNew(checkListBoxOfKind(file), relief, 'files', selectedListVisible),
	widgetShow(checkListBoxOfKind(file)),
	boxPackStart(mainObjectVBox(file), checkListBoxOfKind(file), expand, fill),

	writeln(filling-predefined-tables),
	% REGION load files from disk and put them into the predefined tables
	forall(segusoDir(D), addFilesOfDirToAllFilesTable_HiddenFilesTable_ExtTable(D)),
	% END
	writeln(filling-predefined-tables-end),
	writeln(filling-file-checklistbox),
	% REGION fill the CheckListBox with all the files.
	forall(file(File),
	       addWithProperIcon(file, checkListBoxOfKind(file), File)),
	% END
	writeln(filling-file-checklistbox-end),
	

	
	% REGION device panel

	% REGION substring box
	hBoxNew(filterHBox(device)),
	%widgetShow(filterHBox(device)),
	boxPackStart(mainObjectVBox(device), filterHBox(device), doNotExpand, fill, PaddingForQuickSearchBoxes),




		
	entryNew(containingTextEntry(device)),
	widgetShow(containingTextEntry(device)),
	boxPackStart(filterHBox(device), containingTextEntry(device), expand, fill, PaddingForQuickSearchBoxes),
	widgetSetTooltip(containingTextEntry(device),
			 'SUBSTRING SEARCH: If you enter some text here, only devices whose name contains that text will be shown.'),

	imageButtonNew(deleteSubstringFilterButton(device), 'img/delete-mbuto.png', ''),
	widgetShow(deleteSubstringFilterButton(device)),
	boxPackStart(filterHBox(device), deleteSubstringFilterButton(device), doNotExpand, fill),
	widgetSetTooltip(deleteSubstringFilterButton(device), 'Remove the current substring filter'),


	% END substring box

	frameNew(deviceFrame),
	boxPackStart(mainObjectVBox(device), deviceFrame, doNotExpand, fill),
	
	hBoxNew(deviceFilterBox),
	widgetShow(deviceFilterBox),
	containerAdd(deviceFrame, deviceFilterBox),
	
	



	hBoxNew(labelAndArrowBox(device)),
	widgetShow(labelAndArrowBox(device)),
	boxPackStart(mainObjectVBox(device), labelAndArrowBox(device), doNotExpand, fill),

	eventBoxNew(arrowEventBox(device)),
	widgetShow(arrowEventBox(device)),
	boxPackStart(labelAndArrowBox(device), arrowEventBox(device), doNotExpand, fill),
	imageNew(arrowImage(device), 'img/empty.png'),
	containerAdd(arrowEventBox(device), arrowImage(device)),
	widgetShow(arrowImage(device)),
	miscSetPadding(arrowImage(device), 10, 0),	
	miscSetAlignment(arrowImage(device), 1.0, 0.5),


	labelNew(currentViewLabel(device), 'Devices', noLineWrap),
	widgetShow(currentViewLabel(device)),
	boxPackStart(labelAndArrowBox(device), currentViewLabel(device), expand, fill),

	imageButtonNew(putToDropdownButton(device), 'img/cross-transparent.png', ''),
	widgetShow(putToDropdownButton(device)),
	buttonSetReliefNone(putToDropdownButton(device)),
	widgetSetTooltip(putToDropdownButton(device),
			 'Hide the device panel. It will be shown automatically when needed, i.e. when you select a verb which requires a device.'),
	boxPackStart(labelAndArrowBox(device), putToDropdownButton(device), doNotExpand, fill),

	checkListBoxNew(checkListBoxOfKind(device), relief, 'devices', selectedListHidden),
	widgetShow(checkListBoxOfKind(device)),
	boxPackStart(mainObjectVBox(device), checkListBoxOfKind(device), expand, fill),

	findall(D, device(D), Devices),
	sortByNameInsensitive(Devices, Devices2),
	forall(member(D, Devices2),
	       addWithProperIcon(device, checkListBoxOfKind(device), D)),

	
	% END
	% REGION program panel
	writeln(program-panel),
	% REGION substring box
	hBoxNew(filterHBox(program)),
	boxPackStart(mainObjectVBox(program), filterHBox(program), doNotExpand, fill, PaddingForQuickSearchBoxes),



	entryNew(containingTextEntry(program)),
	widgetShow(containingTextEntry(program)),
	boxPackStart(filterHBox(program), containingTextEntry(program), expand, fill, PaddingForQuickSearchBoxes),
	widgetSetTooltip(containingTextEntry(program),
			 'SUBSTRING SEARCH: If you enter some text here, only programs whose name contains that text will be shown.'),


	imageButtonNew(deleteSubstringFilterButton(program), 'img/delete-mbuto.png', ''),
	widgetShow(deleteSubstringFilterButton(program)),
	boxPackStart(filterHBox(program), deleteSubstringFilterButton(program), doNotExpand, fill),
	widgetSetTooltip(deleteSubstringFilterButton(program), 'Remove the current substring filter'),

	% END substring box

	frameNew(programFrame),
	boxPackStart(mainObjectVBox(program), programFrame, doNotExpand, fill),
	
	hBoxNew(programFilterBox),
	widgetShow(programFilterBox),
	containerAdd(programFrame, programFilterBox),



	hBoxNew(labelAndArrowBox(program)),
	widgetShow(labelAndArrowBox(program)),
	boxPackStart(mainObjectVBox(program), labelAndArrowBox(program), doNotExpand, fill),

	eventBoxNew(arrowEventBox(program)),
	widgetShow(arrowEventBox(program)),
	boxPackStart(labelAndArrowBox(program), arrowEventBox(program), doNotExpand, fill),
	imageNew(arrowImage(program), 'img/empty.png'),
	containerAdd(arrowEventBox(program), arrowImage(program)),
	widgetShow(arrowImage(program)),
	miscSetPadding(arrowImage(program), 10, 0),	
	miscSetAlignment(arrowImage(program), 1.0, 0.5),

	labelNew(currentViewLabel(program), 'Programs', noLineWrap),
	widgetShow(currentViewLabel(program)),
	boxPackStart(labelAndArrowBox(program), currentViewLabel(program), expand, fill),


	imageButtonNew(putToDropdownButton(program), 'img/cross-transparent.png', ''),
	widgetShow(putToDropdownButton(program)),
	buttonSetReliefNone(putToDropdownButton(program)),
	widgetSetTooltip(putToDropdownButton(program),
			 'Hide the program panel. It will be shown automatically when needed, i.e. when you select a verb which requires a program.'),
	boxPackStart(labelAndArrowBox(program), putToDropdownButton(program), doNotExpand, fill),


	checkListBoxNew(checkListBoxOfKind(program), relief, 'programs', selectedListHidden),
	boxPackStart(mainObjectVBox(program), checkListBoxOfKind(program), expand, fill),
	
	findall(P, program( P), Programs),
	sortByNameInsensitive(Programs, Programs2),
	forall(member(M, Programs2),
	       addWithProperIcon(program, checkListBoxOfKind(program), M)),
	widgetShow(checkListBoxOfKind(program)),

	
	% END
	% REGION directory panel
	writeln(directory-panel),
	% REGION substring box
	hBoxNew(filterHBox(directory)),
	boxPackStart(mainObjectVBox(directory), filterHBox(directory), doNotExpand, fill, PaddingForQuickSearchBoxes),



	entryNew(containingTextEntry(directory)),
	widgetShow(containingTextEntry(directory)),
	boxPackStart(filterHBox(directory), containingTextEntry(directory), expand, fill, PaddingForQuickSearchBoxes),
	widgetSetTooltip(containingTextEntry(directory),
			 'SUBSTRING SEARCH: If you enter some text here, only directories whose name contains that text will be shown.'),

		imageButtonNew(deleteSubstringFilterButton(directory), 'img/delete-mbuto.png', ''),
	widgetShow(deleteSubstringFilterButton(directory)),
	boxPackStart(filterHBox(directory), deleteSubstringFilterButton(directory), doNotExpand, fill),
	widgetSetTooltip(deleteSubstringFilterButton(directory), 'Remove the current substring filter'),
	% END substring box

	frameNew(directoryFrame),
	boxPackStart(mainObjectVBox(directory), directoryFrame, doNotExpand, fill),
	
	hBoxNew(directoryFilterBox),
	widgetShow(directoryFilterBox),
	containerAdd(directoryFrame, directoryFilterBox),

	hBoxNew(labelAndArrowBox(directory)),
	widgetShow(labelAndArrowBox(directory)),
	boxPackStart(mainObjectVBox(directory), labelAndArrowBox(directory), doNotExpand, fill),

	eventBoxNew(arrowEventBox(directory)),
	widgetShow(arrowEventBox(directory)),
	boxPackStart(labelAndArrowBox(directory), arrowEventBox(directory), doNotExpand, fill),
	imageNew(arrowImage(directory), 'img/empty.png'),
	containerAdd(arrowEventBox(directory), arrowImage(directory)),
	widgetShow(arrowImage(directory)),
	miscSetPadding(arrowImage(directory), 10, 0),	
	miscSetAlignment(arrowImage(directory), 1.0, 0.5),

	labelNew(currentViewLabel(directory), 'Directories', noLineWrap),
	widgetShow(currentViewLabel(directory)),
	boxPackStart(labelAndArrowBox(directory), currentViewLabel(directory), expand, fill),


	imageButtonNew(putToDropdownButton(directory), 'img/cross-transparent.png', ''),
	widgetShow(putToDropdownButton(directory)),
	buttonSetReliefNone(putToDropdownButton(directory)),
	widgetSetTooltip(putToDropdownButton(directory),
			 'Hide the directory panel. It will be shown automatically when needed, i.e. when you select a verb which requires a directory.'),
	boxPackStart(labelAndArrowBox(directory), putToDropdownButton(directory), doNotExpand, fill),


	checkListBoxNew(checkListBoxOfKind(directory), relief, 'directories', selectedListHidden),
	boxPackStart(mainObjectVBox(directory), checkListBoxOfKind(directory), expand, fill),
	
	findall(P, directory( P), Directories),
	sortByNameInsensitive(Directories, Directories2),
	forall(member(M, Directories2),
	       addWithProperIcon(directory, checkListBoxOfKind(directory), M)),
	widgetShow(checkListBoxOfKind(directory)),

	
	% END

	% REGION idea panel
	writeln(concept-panel),
	hBoxNew(filterHBox(idea)),
	boxPackStart(mainObjectVBox(idea), filterHBox(idea), doNotExpand, fill, PaddingForQuickSearchBoxes),



	entryNew(containingTextEntry(idea)),
	widgetShow(containingTextEntry(idea)),
	boxPackStart(filterHBox(idea), containingTextEntry(idea), expand, fill, PaddingForQuickSearchBoxes),
	widgetSetTooltip(containingTextEntry(idea),
			 'SUBSTRING SEARCH: If you enter some text here, only the concepts whose name contains that text will be shown.'),

		imageButtonNew(deleteSubstringFilterButton(idea), 'img/delete-mbuto.png', ''),
	widgetShow(deleteSubstringFilterButton(idea)),
	boxPackStart(filterHBox(idea), deleteSubstringFilterButton(idea), doNotExpand, fill),
	widgetSetTooltip(deleteSubstringFilterButton(idea), 'Remove the current substring filter'),
	
	hBoxNew(labelAndArrowBox(idea)),
	widgetShow(labelAndArrowBox(idea)),
	boxPackStart(mainObjectVBox(idea), labelAndArrowBox(idea), doNotExpand, fill),

	eventBoxNew(arrowEventBox(idea)),
	widgetShow(arrowEventBox(idea)),
	boxPackStart(labelAndArrowBox(idea), arrowEventBox(idea), doNotExpand, fill),
	imageNew(arrowImage(idea), 'img/empty.png'),
	containerAdd(arrowEventBox(idea), arrowImage(idea)),
	widgetShow(arrowImage(idea)),
	miscSetPadding(arrowImage(idea), 10, 0),	
	miscSetAlignment(arrowImage(idea), 1.0, 0.5),



	labelNew(currentViewLabel(idea), 'Index of concepts', noLineWrap),
	widgetShow(currentViewLabel(idea)),
	boxPackStart(labelAndArrowBox(idea), currentViewLabel(idea), expand, fill),

	eventBoxNew(arrowEventBoxRight(idea)),
	widgetShow(arrowEventBoxRight(idea)),
	boxPackStart(labelAndArrowBox(idea), arrowEventBoxRight(idea), doNotExpand, fill),
	imageNew(arrowImageRight(idea), 'img/empty.png'),
	containerAdd(arrowEventBoxRight(idea), arrowImageRight(idea)),
	widgetShow(arrowImageRight(idea)),
	miscSetPadding(arrowImageRight(idea), 10, 0),	
	miscSetAlignment(arrowImageRight(idea), 1.0, 0.5),

	hBoxNew(ideaHBox),
	widgetShow(ideaHBox),
	boxPackStart(mainObjectVBox(idea), ideaHBox, expand, fill),

	
	% REGION letters

	
	vBoxNew(letterVBox),
	widgetShow(letterVBox),
	boxPackStart(ideaHBox, letterVBox, doNotExpand, fill, 3),

	toggleButtonNew(letterButton-0, '0..9'),
	widgetShow(letterButton-0),
	buttonSetReliefNone(letterButton-0),
	boxPackStart(letterVBox, letterButton-0, doNotExpand, fill),
	

	hBoxNew(letterHBox),
	widgetShow(letterHBox),
	boxPackStart(letterVBox, letterHBox, expand, fill),
	
	vBoxNew(letterVBox1),
	widgetShow(letterVBox1),
	boxPackStart(letterHBox, letterVBox1, doNotExpand, fill),

	vBoxNew(letterVBox2),
	widgetShow(letterVBox2),
	boxPackStart(letterHBox, letterVBox2, doNotExpand, fill),



	toggleButtonNew(letterButton-a, a),
	widgetShow(letterButton-a),
	buttonSetReliefNone(letterButton-a),
	boxPackStart(letterVBox1, letterButton-a, expand, fill),

	toggleButtonNew(letterButton-b, b),
	widgetShow(letterButton-b),
	buttonSetReliefNone(letterButton-b),
	boxPackStart(letterVBox2, letterButton-b, expand, fill),

	toggleButtonNew(letterButton-c, c),
	widgetShow(letterButton-c),
	buttonSetReliefNone(letterButton-c),
	boxPackStart(letterVBox1, letterButton-c, expand, fill),

	toggleButtonNew(letterButton-d, d),
	widgetShow(letterButton-d),
	buttonSetReliefNone(letterButton-d),
	boxPackStart(letterVBox2, letterButton-d, expand, fill),

	toggleButtonNew(letterButton-e, e),
	widgetShow(letterButton-e),
	buttonSetReliefNone(letterButton-e),
	boxPackStart(letterVBox1, letterButton-e, expand, fill),

	toggleButtonNew(letterButton-f, f),
	widgetShow(letterButton-f),
	buttonSetReliefNone(letterButton-f),
	boxPackStart(letterVBox2, letterButton-f, expand, fill),

	toggleButtonNew(letterButton-g, g),
	widgetShow(letterButton-g),
	buttonSetReliefNone(letterButton-g),
	boxPackStart(letterVBox1, letterButton-g, expand, fill),

	toggleButtonNew(letterButton-h, h),
	widgetShow(letterButton-h),
	buttonSetReliefNone(letterButton-h),
	boxPackStart(letterVBox2, letterButton-h, expand, fill),

	toggleButtonNew(letterButton-i, i),
	widgetShow(letterButton-i),
	buttonSetReliefNone(letterButton-i),
	boxPackStart(letterVBox1, letterButton-i, expand, fill),

	toggleButtonNew(letterButton-j, j),
	widgetShow(letterButton-j),
	buttonSetReliefNone(letterButton-j),
	boxPackStart(letterVBox2, letterButton-j, expand, fill),

	toggleButtonNew(letterButton-k, k),
	widgetShow(letterButton-k),
	buttonSetReliefNone(letterButton-k),
	boxPackStart(letterVBox1, letterButton-k, expand, fill),

	toggleButtonNew(letterButton-l, l),
	widgetShow(letterButton-l),
	buttonSetReliefNone(letterButton-l),
	boxPackStart(letterVBox2, letterButton-l, expand, fill),

	toggleButtonNew(letterButton-m, m),
	widgetShow(letterButton-m),
	buttonSetReliefNone(letterButton-m),
	boxPackStart(letterVBox1, letterButton-m, expand, fill),

	toggleButtonNew(letterButton-n, n),
	widgetShow(letterButton-n),
	buttonSetReliefNone(letterButton-n),
	boxPackStart(letterVBox2, letterButton-n, expand, fill),

	toggleButtonNew(letterButton-o, o),
	widgetShow(letterButton-o),
	buttonSetReliefNone(letterButton-o),
	boxPackStart(letterVBox1, letterButton-o, expand, fill),

	toggleButtonNew(letterButton-p, p),
	widgetShow(letterButton-p),
	buttonSetReliefNone(letterButton-p),
	boxPackStart(letterVBox2, letterButton-p, expand, fill),

	toggleButtonNew(letterButton-q, q),
	widgetShow(letterButton-q),
	buttonSetReliefNone(letterButton-q),
	boxPackStart(letterVBox1, letterButton-q, expand, fill),

	toggleButtonNew(letterButton-r, r),
	widgetShow(letterButton-r),
	buttonSetReliefNone(letterButton-r),
	boxPackStart(letterVBox2, letterButton-r, expand, fill),

	toggleButtonNew(letterButton-s, s),
	widgetShow(letterButton-s),
	buttonSetReliefNone(letterButton-s),
	boxPackStart(letterVBox1, letterButton-s, expand, fill),

	toggleButtonNew(letterButton-t, t),
	widgetShow(letterButton-t),
	buttonSetReliefNone(letterButton-t),
	boxPackStart(letterVBox2, letterButton-t, expand, fill),

	toggleButtonNew(letterButton-u, u),
	widgetShow(letterButton-u),
	buttonSetReliefNone(letterButton-u),
	boxPackStart(letterVBox1, letterButton-u, expand, fill),

	toggleButtonNew(letterButton-v, v),
	widgetShow(letterButton-v),
	buttonSetReliefNone(letterButton-v),
	boxPackStart(letterVBox2, letterButton-v, expand, fill),

	toggleButtonNew(letterButton-w, w),
	widgetShow(letterButton-w),
	buttonSetReliefNone(letterButton-w),
	boxPackStart(letterVBox1, letterButton-w, expand, fill),

	toggleButtonNew(letterButton-x, x),
	widgetShow(letterButton-x),
	buttonSetReliefNone(letterButton-x),
	boxPackStart(letterVBox2, letterButton-x, expand, fill),

	toggleButtonNew(letterButton-y, y),
	widgetShow(letterButton-y),
	buttonSetReliefNone(letterButton-y),
	boxPackStart(letterVBox1, letterButton-y, expand, fill),

	toggleButtonNew(letterButton-z, z),
	widgetShow(letterButton-z),
	buttonSetReliefNone(letterButton-z),
	boxPackStart(letterVBox2, letterButton-z, expand, fill),

	% END letters

	
	checkListBoxNew(checkListBoxOfKind(idea), noRelief, 'concepts', selectedListHidden),
	widgetShow(checkListBoxOfKind(idea)),
	boxPackStart(ideaHBox, checkListBoxOfKind(idea), expand, fill),
	widgetSetSizeRequest(checkListBoxOfKind(idea), 136, -1),
	
	redrawFolderPanelClearingAndReaddingEverything,
	% END

	

	% REGION time panel
	writeln(building-time-panel),

	hBoxNew(labelAndArrowBox(time)),
	widgetShow(labelAndArrowBox(time)),
	boxPackStart(timeBox, labelAndArrowBox(time), doNotExpand, fill),

	imageNew(arrowImage(time), 'img/empty.png'),
	widgetShow(arrowImage(time)),
	miscSetPadding(arrowImage(time), 10, 0),	
	miscSetAlignment(arrowImage(time), 1.0, 0.5),
	boxPackStart(labelAndArrowBox(time), arrowImage(time), doNotExpand, fill),


	labelNew(currentViewLabel(time), 'Times', noLineWrap),
	widgetShow(currentViewLabel(time)),
	boxPackStart(labelAndArrowBox(time), currentViewLabel(time), expand, fill),


	imageButtonNew(putTimeToDropdownButton, 'img/cross-transparent.png', ''),
	widgetShow(putTimeToDropdownButton),
	buttonSetReliefNone(putTimeToDropdownButton),
	widgetSetTooltip(putTimeToDropdownButton, 'Transform the time panel into a dropdown list which takes less space.'),
	boxPackStart(labelAndArrowBox(time), putTimeToDropdownButton, doNotExpand, fill),
	
	checkListBoxNew(checkListBoxOfKind(time), relief, 'times', selectedListHidden),
	widgetShow(checkListBoxOfKind(time)),
	boxPackStart(timeBox, checkListBoxOfKind(time), expand, fill),
	forall( time( Time),
	       
	       checkListBoxAddItem(checkListBoxOfKind(time), Time, Time, ''/*tooltip*/,
				   []/*folders make no sense*/,'', '', '', 'img/1.png')),


	% END
	% REGION log
	writeln(creating-log),
	textViewNew(logTextView),
	%widgetShow(logTextView),
	boxPackStart(mainBoxVertical, logTextView, doNotExpand, fill),
	% END
	
	writeln(creating-log-end),
        
	windowMaximize(mainWindow),
	widgetShow(mainWindow),

	% REGION refresh the verb list because it wasn't grouped
	updateItemsAfterSelectionChanged,
	% END
	writeln(debug-111),

	checkListBoxSetFoldersVisibility(checkListBoxOfKind(idea), hidden),
	checkListBoxSetFoldersVisibility(checkListBoxOfKind(verb), hidden),

	% REGION set the scrollingButtonsAlwaysVisible
	(scrollingButtonsAlwaysVisible->(
					 checkListBoxSetKeepScrollingBoxesAlwaysVisible(checkListBoxOfKind(verb),enabled),
					 checkListBoxSetKeepScrollingBoxesAlwaysVisible(checkListBoxOfKind(file),enabled),
					 checkListBoxSetKeepScrollingBoxesAlwaysVisible(checkListBoxOfKind(directory),enabled),
					 checkListBoxSetKeepScrollingBoxesAlwaysVisible(checkListBoxOfKind(idea),enabled),
					 checkListBoxSetKeepScrollingBoxesAlwaysVisible(checkListBoxOfKind(device),enabled),
					 checkListBoxSetKeepScrollingBoxesAlwaysVisible(checkListBoxOfKind(time),enabled),
					 checkListBoxSetKeepScrollingBoxesAlwaysVisible(checkListBoxOfKind(program),enabled));true),
					 
	writeln(debug-2),
	% END
	% REGION save the quick search button for Henrik :-)
	(theQuickSearchBoxesAreVisible->
	    	(widgetShow(filterHBox(verb)),
		    widgetShow(filterHBox(file)),
		    widgetShow(filterHBox(directory)),
		    widgetShow(filterHBox(idea)),
		    widgetShow(filterHBox(program)),
		    widgetShow(filterHBox(device)));
	    true /*don't do anything in this case*/),
	% END
	writeln(mainLoopCalled),
	mainLoop([]).


mainLoop(Tasks):-
	var(Tasks),
	writeln(error-var-tasks),
	throw(var-tasks).
mainLoop(Tasks):-
	
	nonvar(Tasks),
	sleepUntilEventIsRaised(E),
	(mainLoopManageEvent(E, Tasks, NewTasks)->true ; (writeln(ignoredEvent(E)), NewTasks=Tasks)),



	(E=windowDestroyed(mainWindow)->
	    writeln('exiting program');
	( !,mainLoop(NewTasks))).

readableRepresOfList(L, R):-
	L=[R].
readableRepresOfList([], '  ').
readableRepresOfList(L, L):-
	L\=[_],
	L\=[].

removePathsFromFiles([V, Files, P, D, Di, T], [V, Files2, P, D, Di, T]):-
	maplist(file_base_name, Files, Files2).




recomputeFolders(K, File):-
	findall(F, (idea(K, F), related(F, K, File)), Folders),
	checkListBoxSetItemFolders(checkListBoxOfKind(K), File, Folders).

depressOtherLetterButtons:-
	selectedLetter(L),
	toggleButtonSetPressedState(letterButton-L, released),
	retract(selectedLetter(L)).
depressOtherLetterButtons:-
	\+selectedLetter(_).


fromIdeaKindPairToReadableIdea((Idea, Kind), ReadablePair):-
	Kind \= verb, 
	idea(K2, Idea),
	K2 \= Kind,
	concat_atom([Idea, ' (', Kind, ')'], ReadablePair).
fromIdeaKindPairToReadableIdea((Idea, verb), Idea).
fromIdeaKindPairToReadableIdea((Idea, Kind), ReadablePair):-
	forall(idea(K, Idea), K=Kind),
	ReadablePair = Idea.

recentConceptsLoop:-
	sleepUntilEventIsRaised(E),
	ignore(recentConceptsManageEvent(E)),
	
	(E = dialogDestroyed(recentDialog-concept)->
	    true
	; (!, recentConceptsLoop)).

recentActionsLoop(Tasks, NewTasks):-
	sleepUntilEventIsRaised(E),
	(recentActionsManageEvent(E, Tasks, NewTasks)->true
	; writeln(recentActionsLoop-discarded-event-E)),
	
	(E = dialogDestroyed(recentDialog-action)->
	    true
	; (!, recentActionsLoop(Tasks, NewTasks))).

recentConceptsManageEvent(checkListBoxItemSelected(recentConceptsList, L)):-
	% REGION deselect all
	forall((selected(K, O), K\=time, K\=idea-_),
	       ( checkListBoxSetItemState(checkListBoxOfKind(K), O,  hidden),
		   retract(selected(K, O)))),
	forall((selected(idea-K, O)),
	       ( checkListBoxSetItemState(checkListBoxOfKind(idea), pair(K, O),  hidden),
		   retract(selected(idea-K, O)))),
	% END
	% REGION select the ideas
	% L is a list of pairs [ (Idea1, Kind1), (Idea2, Kind2)]
	forall(member((I, K), L),
	       assert(selected(idea-K, I))),
	% END
	showPanelsAgain,
	dialogClose(recentDialog-concept),
	updateItemsAfterSelectionChanged2,
	redrawFolderPanelHidingAndReshowing,
	checkListBoxDefeatBug(checkListBoxOfKind(verb)),
	addCurrentlySelectedConceptsToRecentConcepts,
	updateDeleteIdeasToolbarButtons.
recentActionsManageEvent(dialogDestroyed(recentDialog-action), Tasks, Tasks).
recentActionsManageEvent(checkListBoxItemSelected(recentActionsList, Action), Tasks, NewTasks):-
	addActionToRecentActionList(Action),
	executeAction(Action, Tasks, NewTasks),
	% REGION this must be called AFTER executeAction! executeAction might enter a confirmation loop
	% which would receive and discard the event "dialogDestroyed(recentDialog-action)".
	dialogClose(recentDialog-action). % this raises an event dialogDestroyed(recentDialog-action)
        % END

mainLoopManageEvent(buttonClicked(recentButton-action), Tasks, NewTasks):-
	iDeclareThisIsTrue(var(NewTasks)),
	dialogNew(recentDialog-action, mainWindow, 'Recent actions', 'img/action.png'),

	hBoxNew(recentActionsHBox),
	widgetShow(recentActionsHBox),
	dialogAddToVBox(recentDialog-action, recentActionsHBox),

	labelNew(recentActionsLabel,
		 'These are the most recently executed actions.', lineWrap),
	widgetShow(recentActionsLabel),
	boxPackStart(recentActionsHBox, recentActionsLabel, doNotExpand, fill, 10),
	
	checkListBoxNew(recentActionsList, relief, 'actions', selectedListHidden),
	widgetShow(recentActionsList),
	boxPackStart(recentActionsHBox, recentActionsList, expand, fill, 10),


	findall(X, recentAction(repeated, X), RecentActions1),
	findall(X, recentAction(notRepeated, X), RecentActions2),
	forall(member(L, RecentActions1),
	       (
		 % L is an action: [V, Files, Programs, Devices, Dirs, Times]
		 removePathsFromFiles(L, L2),
		 maplist(readableRepresOfList, L2, LRead),
		 L=[V|_],
		 verbToIcon(V, Icon),
		 checkListBoxAddItem(recentActionsList, L, LRead,
				       ''/*tooltip*/, [], ''/*dir*/,
				       '', '', Icon))),
	writeln(debugffdfd),
        ((RecentActions1 = [] ; RecentActions2 = []) -> true; checkListBoxAddSeparator(recentActionsList)),
	forall(member(L, RecentActions2),
	       (  
		  removePathsFromFiles(L, L2),
		  maplist(readableRepresOfList, L2, LRead),
		  L=[V|_],
		  verbToIcon(V, Icon),
		  checkListBoxAddItem(recentActionsList, L, LRead,
				       ''/*tooltip*/, [], ''/*dir*/,
				       '', '', Icon))),
 	writeln(debugffdfd2),
        windowMaximize(recentDialog-action),
	widgetShow(recentDialog-action),
	recentActionsLoop(Tasks, NewTasks),
	iDeclareThisIsTrue(nonvar(NewTasks)).
				 
mainLoopManageEvent(buttonClicked(recentButton-concept), Tasks, Tasks):-
	dialogNew(recentDialog-concept, mainWindow, 'Recent concepts', 'img/bulb.png'),

	hBoxNew(recentConceptsHBox),
	widgetShow(recentConceptsHBox),
	dialogAddToVBox(recentDialog-concept, recentConceptsHBox),

	labelNew(recentConceptsLabel,
		 'These are the most recently used combinations of concepts. Click to select that combination in the main screen.', lineWrap),
	widgetShow(recentConceptsLabel),
	boxPackStart(recentConceptsHBox, recentConceptsLabel, doNotExpand, fill, 10),
	
	checkListBoxNew(recentConceptsList, relief, 'concepts', selectedListHidden),
	widgetShow(recentConceptsList),
	boxPackStart(recentConceptsHBox, recentConceptsList, expand, fill, 10),


	findall(X, recentConcept(X), RecentConcepts),
	% RecentConcepts is a list of lists of pairs:  [  [(play, verb), (michael, file)], [(burn, verb), (pdf, file)] ]
	forall(member(L, RecentConcepts),
	       % L is a list of pairs : [(burn, verb), (pdf, file)]
	       (   maplist(fromIdeaKindPairToReadableIdea, L, ReadableList),
		   (ReadableList=[SingleElement]->ReadableList2 = SingleElement ; ReadableList2 = ReadableList),
		   (member((Idea, verb), L)->verbIdeaToIcon(Idea, Icon); Icon = ''),
		   checkListBoxAddItem(recentConceptsList, L, ReadableList2,
				       ''/*tooltip*/, [], ''/*dir*/,
				       '', '', Icon))),
        windowMaximize(recentDialog-concept),
	widgetShow(recentDialog-concept),
	recentConceptsLoop.
	
mainLoopManageEvent(buttonClicked(deleteIdeaButton), Tasks, Tasks):-
	forall(selected(idea-K, I),
	       (retractall(related(I, K, _)),
		   retract(idea(K, I)),
		   checkListBoxDeleteItem(checkListBoxOfKind(idea), pair(K, I)))),
	retractall(selected(idea-_, _)),
	updateItemsAfterSelectionChanged,
        % REGION @@ update the file folders shown below the files! :-( very slow...
	forall(file(O),
	       recomputeFolders(file, O)),
	forall(program(O),
	       recomputeFolders(program, O)),
	%forall(verb(O),
%	       recomputeFolders(verb, O)), % @@@ crashes
	forall(device(O),
	       recomputeFolders(device, O)),
	% END
	redrawFolderPanelClearingAndReaddingEverything.

/*
mainLoopManageEvent(buttonClicked(autoAssociateIdeaAndFileButton), T, T):-
	writeln(not-implemented),
	throw(not-implemented), % @@@ show message if no file idea selected.
	forall(selected(idea, I),
	       (downcase_atom(I, I2),
		   forall(file(F),
			  (downcase_atom(F, F2),
			      (sub_atom(F2, _, _, _, I2)->
				  assertIfNotAlreadyThere(related(I, file, F))
			      ; true))))),
        % @@ update the visible ideas
	forall(selected(file, O),
	       recomputeFolders(file, O)),
	updateItemsAfterSelectionChanged.
*/
mainLoopManageEvent(buttonClicked(letterButton-L), Tasks, Tasks):-
	toggleButtonIsPressed(letterButton-L),
	writeln(buttonClicked(letterButton-L-pressed)),
	depressOtherLetterButtons,
	assert(selectedLetter(L)),
	updateConceptArrows,
	redrawFolderPanelHidingAndReshowing,
	writeln(buttonClicked(letterButton-L-pressed-exit)).
mainLoopManageEvent(buttonClicked(letterButton-L), Tasks, Tasks):-
	\+toggleButtonIsPressed(letterButton-L),
	writeln(buttonClicked(letterButton-L-not-pressed)),
	retract(selectedLetter(L)),
	updateConceptArrows,
	redrawFolderPanelHidingAndReshowing,
	writeln(buttonClicked(letterButton-L-not-pressed-exit)).
mainLoopManageEvent(checkListBoxItemExecuted(checkListBoxOfKind(verb), I), Tasks, NewTasks):-
	onVerbExecuted(I, Tasks, NewTasks).

mainLoopManageEvent(optionMenuItemSelected(timeOptionMenu, Time), Tasks, Tasks):-
	retractall(selected(time, _)),
	assert(selected(time, Time)),
	updateItemsAfterSelectionChanged.
mainLoopManageEvent(checkListBoxItemSelected(checkListBoxOfKind(verb), I), T, T):-
	onVerbSelected(I).
/*
mainLoopManageEvent(buttonClicked(deassociateWithIdeaButton), Tasks, Tasks):-
	findall((K,F), selected(idea-K, F), SelectedTables),
	forall((selected(K, O), K\=idea-_),
	       forall(member((Kind, Idea), SelectedTables),
		      retractall(related(Idea, Kind, O)))),
	forall(selected(K, O),
	       recomputeFolders(K, O)).
*/

mainLoopManageEvent(buttonClicked(globalGoButton), Tasks, NewTasks):-
	addCurrentlySelectedConceptsToRecentConcepts,
	understandWhatTheUserWantsToDoAnalyzingSelectedAndVisibleItems(A),
	addActionToRecentActionList(A),
	executeAction(A, Tasks, NewTasks),
	widgetHide(globalGoRedArrowLeft),
	widgetHide(globalGoRedArrow),
	widgetHide(globalGoButton).




mainLoopManageEvent(buttonClicked(putTimeToDropdownButton), Tasks, Tasks):-
	assert(panelStateMinimized(time)),
	widgetHide(timeBox),
	widgetShow(putTimeToPanelButton),
	widgetShow(timeOptionMenu),
	optionMenuSetSelectedItem(timeOptionMenu, 'Now'),
	retractall(selected(time, _)),
	assert(selected(time, 'Now')).
mainLoopManageEvent(buttonClicked(putToDropdownButton(K)), Tasks, Tasks):-
	forall(selected(K, F),
	       (   writeln(debug-before),
		   checkListBoxSetItemState(checkListBoxOfKind(K), F, deselected),
		   writeln(debug-before2),
		   onObjectDeselected(K, F))),
	assert(panelStateMinimized(K)),
	widgetHide(mainObjectVBox(K)).
mainLoopManageEvent(buttonClicked(putTimeToPanelButton), Tasks, Tasks):-
	widgetShow(timeBox),
	retract(panelStateMinimized(time)),
	widgetHide(putTimeToPanelButton),
	widgetHide(timeOptionMenu),

	forall(time( T),
	       checkListBoxSetItemState(checkListBoxOfKind(time), T, deselected)),
	checkListBoxSetItemState(checkListBoxOfKind(time), 'Now', selected),

	retractall(selected(time, _)),
	assert(selected(time, 'Now')).


	
mainLoopManageEvent(checkListBoxItemDeselected(checkListBoxOfKind(verb), I), T, T):-
	onVerbDeselected(I).
mainLoopManageEvent(checkListBoxItemSelected(checkListBoxOfKind(K), I), T, T):-
	K\=idea,
	onObjectSelected(K, I).
mainLoopManageEvent(checkListBoxItemSelected(checkListBoxOfKind(idea), pair(K, I)), T, T):-
	onIdeaSelected(K, I).
mainLoopManageEvent(checkListBoxItemDeselected(checkListBoxOfKind(K), I), T, T):-
	K\=idea,
	onObjectDeselected(K, I).
mainLoopManageEvent(checkListBoxItemDeselected(checkListBoxOfKind(idea), pair(K, I)), T, T):-
	onIdeaDeselected(K, I).
mainLoopManageEvent( buttonClicked(deselectAllButton), T, T):-
	forall( (selected(K, Thing), K\=time, K\=idea-_),
		(checkListBoxSetButtonCheckState(checkListBoxOfKind(K), Thing, unchecked),
		    retract(selected(K, Thing)),
		    ifItIsNowFilteredAwayRemoveItFromList(K, Thing))),
	forall( selected(idea-K, I),
		(checkListBoxSetButtonCheckState(checkListBoxOfKind(idea), pair( K, I), unchecked),
		    retract(selected(idea-K, I)))),
	((showPanelsWhenNoConceptIsSelected ; (selected(Q, _), Q\=time))->
	    updateItemsAfterSelectionChanged
	;  updateRedArrows([]))/*optimization*/,

	hidePanelsIfNothingIsSelected,
	widgetEnable(putToDropdownButton(device)),
	widgetEnable(putToDropdownButton(file)),
	widgetEnable(putToDropdownButton(program)),
	updateDeleteIdeasToolbarButtons,
	redrawFolderPanelHidingAndReshowing,
	widgetHide(globalGoButton),
	widgetHide(globalGoRedArrowLeft),
	widgetHide(globalGoRedArrow).


%mainLoopManageEvent(buttonClicked(showHiddenFilesButton), T, T):-
%	onShowHiddenFilesButtonToggled.
%mainLoopManageEvent(buttonClicked(reparseDiskButton), T, T):-
%	onReparseDiskButtonPressed.

%mainLoopManageEvent(menuItemClicked(tooltipsItem), T, T):-
%	onShowTooltipsButtonToggled.
/*
mainLoopManageEvent(buttonClicked(showFileDateButton), T, T):-
	onShowDateToggled.
mainLoopManageEvent(buttonClicked(showFileDirButton), T, T):-
	onShowDirToggled.
mainLoopManageEvent(buttonClicked(showFileSizeButton), T, T):-
	onShowSizeToggled.
*/

mainLoopManageEvent(menuItemClicked(aboutItem), T, T):-
	onAboutButtonPressed.
mainLoopManageEvent(buttonClicked(newFilterButton(K)), T, T):-
	onNewFilterButtonPressedAux(K).
mainLoopManageEvent(buttonClicked(editFeaturesButton(K)), T, T):-
	onEditFeaturesButtonPressedAux2(K).

mainLoopManageEvent(buttonClicked(deleteSubstringFilterButton(K)), T, T):-
	K\=idea,
	entrySetText(containingTextEntry(K), ''),
	updateItemsAfterSelectionChanged.
mainLoopManageEvent(buttonClicked(deleteSubstringFilterButton(K)), T, T):-
	K=idea,
	entrySetText(containingTextEntry(K), ''),
	redrawFolderPanelHidingAndReshowing.
mainLoopManageEvent(menuItemClicked(quitItem), T, T):-
	widgetDestroy(mainWindow),
	onMainWindowDestroyed.
/*
mainLoopManageEvent( buttonClicked(sortByFileNameButton), T, T):-
	onSortByFileNameButtonToggled.
mainLoopManageEvent( buttonClicked(sortByFileDateButton), T, T):-
	onSortByFileDateButtonToggled.
mainLoopManageEvent( buttonClicked(sortByFileSizeButton), T, T):-
	onSortByFileSizeButtonToggled.
*/
/*
mainLoopManageEvent(menuItemToggled(logItem), T, T):-
	menuCheckItemIsChecked(viewMenu, logItem),
	widgetShow(logTextView).
mainLoopManageEvent(menuItemToggled(logItem), T, T):-
	\+menuCheckItemIsChecked(viewMenu, logItem),
	widgetHide(logTextView).
*/
/*
mainLoopManageEvent(buttonClicked(showQuickSearchButton), T, T):-
	toggleButtonIsPressed(showQuickSearchButton),
	widgetShow(filterHBox(verb)),
	widgetShow(filterHBox(file)),
	widgetShow(filterHBox(program)),
	widgetShow(filterHBox(device)).
mainLoopManageEvent(buttonClicked(showQuickSearchButton), T, T):-
	\+toggleButtonIsPressed(showQuickSearchButton),
	widgetHide(filterHBox(verb)),
	widgetHide(filterHBox(file)),
	widgetHide(filterHBox(program)),
	widgetHide(filterHBox(device)).
*/
mainLoopManageEvent(buttonClicked(editTimeFeaturesButton), T, T):-
	dialogNew(newTimeDialog, mainWindow, 'Create new time', 'img/1.png'),

	vBoxNew(newTimeVBox),
	widgetShow(newTimeVBox),
	dialogAddToVBox(newTimeDialog, newTimeVBox),

	labelNew(startTimeLabel, 'Starting time:', noLineWrap),
	widgetShow(startTimeLabel),
	boxPackStart(newTimeVBox, startTimeLabel, expand, fill),

	hBoxNew(startTimeHBox),
	widgetShow(startTimeHBox),
	boxPackStart(newTimeVBox, startTimeHBox, expand, fill),

	% REGION seconds
	labelNew(startLabel(seconds), 'seconds ', noLineWrap),
	widgetShow(startLabel(seconds)),
	boxPackStart(startTimeHBox, startLabel(seconds), doNotExpand, fill),
	
	entryNew(startEntry(seconds)),
	widgetShow(startEntry(seconds)),
	entrySetText(startEntry(seconds), '0'),
	boxPackStart(startTimeHBox, startEntry(seconds), expand, fill),

	% END
	% REGION minutes
	labelNew(startLabel(minutes), 'minutes ', noLineWrap),
	widgetShow(startLabel(minutes)),
	boxPackStart(startTimeHBox, startLabel(minutes), doNotExpand, fill),
	
	entryNew(startEntry(minutes)),
	widgetShow(startEntry(minutes)),
	entrySetText(startEntry(minutes), '0'),
	boxPackStart(startTimeHBox, startEntry(minutes), expand, fill),

	% END
	% REGION hours
	labelNew(startLabel(hours), 'hours ', noLineWrap),
	widgetShow(startLabel(hours)),
	boxPackStart(startTimeHBox, startLabel(hours), doNotExpand, fill),
	
	entryNew(startEntry(hours)),
	widgetShow(startEntry(hours)),
	entrySetText(startEntry(hours), '0'),
	boxPackStart(startTimeHBox, startEntry(hours), expand, fill),

	% END
	% REGION days
	labelNew(startLabel(days), 'days ', noLineWrap),
	widgetShow(startLabel(days)),
	boxPackStart(startTimeHBox, startLabel(days), doNotExpand, fill),
	
	entryNew(startEntry(days)),
	widgetShow(startEntry(days)),
	entrySetText(startEntry(days), '0'),
	boxPackStart(startTimeHBox, startEntry(days), expand, fill),
	% END



	
     	labelNew(periodicTimeLabel, 'Repeat every:', noLineWrap),
	widgetShow(periodicTimeLabel),
	boxPackStart(newTimeVBox, periodicTimeLabel, expand, fill),

	hBoxNew(repeatTimeHBox),
	widgetShow(repeatTimeHBox),
	boxPackStart(newTimeVBox, repeatTimeHBox, expand, fill),

	% REGION seconds
	labelNew(repeatLabel(seconds), 'seconds ', noLineWrap),
	widgetShow(repeatLabel(seconds)),
	boxPackStart(repeatTimeHBox, repeatLabel(seconds), doNotExpand, fill),
	
	entryNew(repeatEntry(seconds)),
	widgetShow(repeatEntry(seconds)),
	entrySetText(repeatEntry(seconds), '0'),
	boxPackStart(repeatTimeHBox, repeatEntry(seconds), expand, fill),

	% END
	% REGION minutes
	labelNew(repeatLabel(minutes), 'minutes ', noLineWrap),
	widgetShow(repeatLabel(minutes)),
	boxPackStart(repeatTimeHBox, repeatLabel(minutes), doNotExpand, fill),
	
	entryNew(repeatEntry(minutes)),
	widgetShow(repeatEntry(minutes)),
	entrySetText(repeatEntry(minutes), '0'),
	boxPackStart(repeatTimeHBox, repeatEntry(minutes), expand, fill),

	% END
	% REGION hours
	labelNew(repeatLabel(hours), 'hours ', noLineWrap),
	widgetShow(repeatLabel(hours)),
	boxPackStart(repeatTimeHBox, repeatLabel(hours), doNotExpand, fill),
	
	entryNew(repeatEntry(hours)),
	widgetShow(repeatEntry(hours)),
	entrySetText(repeatEntry(hours), '0'),
	boxPackStart(repeatTimeHBox, repeatEntry(hours), expand, fill),

	% END
	% REGION days
	labelNew(repeatLabel(days), 'days ', noLineWrap),
	widgetShow(repeatLabel(days)),
	boxPackStart(repeatTimeHBox, repeatLabel(days), doNotExpand, fill),
	
	entryNew(repeatEntry(days)),
	widgetShow(repeatEntry(days)),
	entrySetText(repeatEntry(days), '0'),
	boxPackStart(repeatTimeHBox, repeatEntry(days), expand, fill),

	% END

	
	hBoxNew(createTimeHBox),
	widgetShow(createTimeHBox),
	boxPackStart(newTimeVBox, createTimeHBox, expand, fill),
	
	buttonNew(createTimeButton,'create'),
	widgetShow(createTimeButton),
	widgetDisable(createTimeButton),
	boxPackStart(createTimeHBox, createTimeButton, expand, doNotFill),
	
	widgetShow(newTimeDialog),

	newTimeLoop.
mainLoopManageEvent(entryTextChanged(containingTextEntry(K)), T, T):-
	K\=idea,
	updateItemsAfterSelectionChanged.
mainLoopManageEvent(entryTextChanged(containingTextEntry(K)), T, T):-
	K=idea,
	redrawFolderPanelHidingAndReshowing.
mainLoopManageEvent(windowDestroyed(mainWindow), T, T):-
	onMainWindowDestroyed.
mainLoopManageEvent(timer, Tasks, NewTasks):-
	onTimer(Tasks, NewTasks).





newTimeLoop:-
	sleepUntilEventIsRaised(E),

	( E = entryTextChanged(startLabel(_T))->
	    (onStartTimeChanged, newTimeLoop);
	  E = dialogDestroyed(newTimeDialog)->
	    writeln(exiting-new-time-dialog);
	  (writeln(ignoredEvent(E)), !, newTimeLoop)).

	
% @@ update view.
/*
	entryGetText(containingTextEntry(file), ''),
	retractall(nameFilter(K, _)).
onContainingTextEntryTextChanged(K):-
	entryGetText(containingTextEntry(file), T),
	T\='',
	retractall(nameFilter(K, _)),
	assert(nameFilter(K, T)).
*/
	
onAboutButtonPressed:-
	dialogNew(aboutDialog, mainWindow, 'About segusoLand', 'img/icon.png'),

	hBoxNew(aboutHBox),
	widgetShow(aboutHBox),
	dialogAddToVBox(aboutDialog, aboutHBox),

	imageNew(candleImage, 'img/zombieJesus.png'),
	widgetShow(candleImage),
	boxPackStart(aboutHBox, candleImage, doNotExpand, fill),

	labelNew(aboutLabel, 'segusoLand is created by Maurizio Colucci (also known as seguso). \nEmail:seguso.forever@tin.it. \nCoding: Maurizio Colucci, Henrik Brink, Sylvain Glaize, James Dickens. \nHome page of segusoLand: http://segusoland.sourceforge.net', lineWrap),
	widgetShow(aboutLabel),
	boxPackStart(aboutHBox, aboutLabel, expand, fill),
	
	widgetShow(aboutDialog).
	
onManageTasksClicked(Tasks, NewTasks):-
	dialogNew(tasksDialog, mainWindow, 'Manage tasks', 'img/1.png'),

	vBoxNew(tasksVBox),
	widgetShow(tasksVBox),
	dialogAddToVBox(tasksDialog, tasksVBox),

	hBoxNew(tasksHbox),
	widgetShow(tasksHbox),
	boxPackStart(tasksVBox, tasksHbox, doNotExpand, fill),

	labelNew(tasksLabel,
		 'Here you can select some tasks and click \"Delete selected tasks\". If you don\'t want to delete any task, simply close the window.', lineWrap),
	widgetShow(tasksLabel),
	boxPackStart(tasksHbox, tasksLabel, expand, fill),
	
	buttonNew(deleteTaskButton, 'Delete selected tasks'),
	widgetShow(deleteTaskButton),
	boxPackStart(tasksHbox, deleteTaskButton, doNotExpand, doNotFill),
	
	checkListBoxNew(tasksListBox, relief, 'tasks', selectedListHidden),
	widgetShow(tasksListBox),
	boxPackStart(tasksVBox, tasksListBox, expand, fill),


	forall(member(T, Tasks),
	       (readableRepresentationOfTask(T, Read),
		   checkListBoxAddItem(tasksListBox, T, Read, ''/*tooltip*/,
				       []/*folders make no sense*/,'', '', '', 'img/1.png'))),
	widgetShow(tasksDialog),
	windowMaximize(tasksDialog),
	tasksLoop([], Tasks, NewTasks),
	checkListBoxDefeatBug(checkListBoxOfKind(verb)).

tasksLoop(SelectedTasks, _, _):-
	var(SelectedTasks),
	throw(error-var-selected-tasks).
tasksLoop(_, Tasks, _):-
	var(Tasks),
	throw(error-var-tasks).
tasksLoop(/*in*/SelectedTasks, /*in*/Tasks, /*out*/NewTasks):-
	nonvar(SelectedTasks),
	nonvar(Tasks),
	sleepUntilEventIsRaised(E),
	( E = dialogDestroyed(tasksDialog)->
	    NewTasks = Tasks /*and exit*/;
	  E = buttonClicked(deleteTaskButton)->
	    (writeln(sel-SelectedTasks),
		onDeleteTaskPressed(SelectedTasks, NewSelectedTasks, Tasks, NewTasks2),
		tasksLoop(NewSelectedTasks, NewTasks2, NewTasks));
	  E = checkListBoxItemSelected(tasksListBox, I)->
	    tasksLoop([I|SelectedTasks], Tasks, NewTasks);
	  E = checkListBoxItemDeselected(tasksListBox, I)->
	    (remove( I, SelectedTasks, NewSelectedTasks),
		tasksLoop(NewSelectedTasks, Tasks, NewTasks));
	  (writeln(ignoredEvent(E)),
	      !, tasksLoop(SelectedTasks, Tasks, NewTasks))).
readableRepresentationOfTask(T, Repr):-
	T=(Time, A),
	convert_time(Time, String),
	Repr = (String, A).
onDeleteTaskPressed(SelectedTasks, []/*NewSelectedTasks*/, Tasks, NewTasks):-
	forall(member(T, SelectedTasks),
	       checkListBoxDeleteItem(tasksListBox, T)),
	removeListFromList(SelectedTasks, Tasks, NewTasks).



removeListFromList([], L2, L2).
removeListFromList([H|T], L2, NewL2):-
	remove(H, L2, NewL2Temp),
	removeListFromList(T, NewL2Temp, NewL2).
addStringToLog(S):-
	% REGION log
	get_time(Time),
	convert_time(Time, TimeString),
	concat_atom([TimeString, ': ', S], Text),
	textViewAddLine(logTextView, Text).
	% END
	
seeIfThereAreNewFilesInDir(D):-
	addSlashToDirIfNeeded(D, DirSlash),
	concat_atom([DirSlash, '*'], ReqVisible),  
	concat_atom([DirSlash, '.*'], ReqHidden), 
	expand_file_name(ReqVisible, VisibleFiles),  % if the dir does not exist, it returns [], which is good! @@ but add a check?
	expand_file_name(ReqHidden, HiddenFiles),
	append(VisibleFiles, HiddenFiles, AllFiles),
	forall((member(F, AllFiles)),
	       ifFileIsNotAlreadyInDatabaseAddToDatabase(F)).
seeIfThereAreNewFilesInDir(_):-
	writeln(seeIfThereAreNewFilesInDir-failed),
	throw(error).
ifFileIsNotAlreadyInDatabaseAddToDatabase(F):-
	file( F).
ifFileIsNotAlreadyInDatabaseAddToDatabase(F):-
	\+file( F),
	\+theFileHasAValidUtf8Name(F),
	writeln(discarded-file-utf-name-not-valid-F).
ifFileIsNotAlreadyInDatabaseAddToDatabase(F):-
	\+file( F),
	\+exists_file(F),
	writeln(discarded-file-because-not-real-file-F).
ifFileIsNotAlreadyInDatabaseAddToDatabase(F):-
	\+file(F),
	exists_file(F),
	theFileHasAValidUtf8Name(F),
	assert(file(F)),
	assertExtensionIfAny(F),
	assertConcepts(F),
	addWithProperIcon(file, checkListBoxOfKind(file), F).
        % @@@ hidden Files...

% Also works if the Dir does not exist. :-)
addFilesOfDirToAllFilesTable_HiddenFilesTable_ExtTable(Dir):-
	addSlashToDirIfNeeded(Dir, DirSlash),
	concat_atom([DirSlash, '*'], ReqVisible),  
	concat_atom([DirSlash, '.*'], ReqHidden), 
	expand_file_name(ReqVisible, VisibleFiles),  % if the dir does not exist, it returns [], which is good! @@ but add a check?
	expand_file_name(ReqHidden, HiddenFiles),
	append(VisibleFiles, HiddenFiles, AllFiles),
	% REGION all files table
	forall(member(Q, AllFiles),
	       ifItIsAnOrdinaryFileAndHasAValidUtfNameThenAssertItIntoAllTables(Q)).

ifItIsAnOrdinaryFileAndHasAValidUtfNameThenAssertItIntoAllTables(Q):-
	\+exists_file(Q).
ifItIsAnOrdinaryFileAndHasAValidUtfNameThenAssertItIntoAllTables(Q):-
	\+theFileHasAValidUtf8Name(Q).
ifItIsAnOrdinaryFileAndHasAValidUtfNameThenAssertItIntoAllTables(Q):-
	exists_file(Q),
	theFileHasAValidUtf8Name(Q),
	assert(file(Q)),
	assertExtensionIfAny(Q),
	assertConcepts(Q).
	
addSlashToDirIfNeeded(Dir, Dir):-
	dirEndsInSlash(Dir).
addSlashToDirIfNeeded(Dir, DirSlash):-
	\+dirEndsInSlash(Dir),
	concat_atom([Dir, '/'], DirSlash).
dirEndsInSlash(Dir):-
	atom_chars(Dir, List),
	listEndsWith(List, '/').
listEndsWith([C], C).
listEndsWith([Element], C):-
	Element\=C,
	fail.
listEndsWith(L, C):-
	L\=[_],
	L=[H|T],
	H\=[], T\=[],
	!,
	listEndsWith(T, C).
	
% speed it up!
assertThisIsATextFileIfItIs(F):-
	concat_atom(['file \'', F, '\''], Command),
	open(pipe(Command), read, Stream),
	get(Stream, Term),
	writeln(justRead(Term)),
	close(Stream).

	
fromKindToCurrentViewLabel(program, labelCurrentProgramFilter).
fromKindToCurrentViewLabel(verb, labelCurrentVerbFilter).
fromKindToCurrentViewLabel(device, labelCurrentDeviceFilter).
fromKindToCurrentViewLabel(file, labelCurrentFileFilter).


createDbIfItDoesNotExist:-
	expand_file_name('~', [HomeDir]),
	concat_atom([HomeDir, '/.segusoLand-database'], DatabaseFileNamewithFullPath),
	exists_file(DatabaseFileNamewithFullPath).
createDbIfItDoesNotExist:-
	expand_file_name('~', [HomeDir]),
	concat_atom([HomeDir, '/.segusoLand-database'], DatabaseFileNamewithFullPath),
	\+exists_file(DatabaseFileNamewithFullPath),
	shell('touch ~/.segusoLand-database', Res),
	(Res=0->writeln('WARNING: Database not found, created anew');
	    writeln('could not create database! Exiting')),
	Res=0,
	assert(segusoDir('~')),
	assert(panelStateMinimized(directory)).

assertDefaultShit:-
	% REGION add default ideas
	assertIdeas([
		     acceleration ,
		     action,
		     add ,
		     administration,
		     adjust,
		     always,
		     amule,
		     ape,
		     appearance ,
		     append ,
		     application,
		     appointments ,
		     associate ,
		     archive ,
		     archiving ,
		     arrow ,	
		     article ,
		     assign,
		     attach,
		     attachment ,
		     attribute ,
		     audio ,
		     background ,
		     behaviour ,
		     bin,
		     binary,
		     blank,
		     box,
		     browse ,
		     burn ,
		     button ,
		     bytes,
		     'c++',
		     calendar,
		     cancel,
		     catalogue,
		     category,
		     cd,
		     check  ,
		     change  ,
		     clear  ,
		     cli,
		     click  ,
		     client  ,
		     close  ,
		     colour  ,
		     command,
		     compose  ,
		     copy  ,
		     compress  ,
		     compression  ,
		     computer  ,
		     compiled,
		     concept  ,
		     configure  ,
		     connect,
		     console,
		     contents,
		     convert  ,
		     copy, 
		     cpu, 
		    create  ,
		     cryptography,
		    cursor  ,
		    customize  ,
		     cue,
		     cut, %
		     data  ,
		     database,
		     date  ,
		     deassociate  ,
		     deb,
		     decode,
		    decompress  ,
		    decorations  ,
		    define  ,
		    delete  ,
		     deselect,
		    desktop  ,
		    device  ,
		    different  ,
		     difficult  ,
		     directory  ,
		     disable,
		     disc  ,
		     display  ,
		     doc,
		     document  ,
		     double  ,
		     download,
		     drive  ,
		     dvd,
		     dvi,
		     easy  ,
		     edit  ,
		     editor  ,
		     edonkey,
		     end,
		     eject,
		     email  ,
		     emule,
		     empty  ,
		     enable,
		     encode,
		     encrypt,
		     enqueue  ,
		     entry,
		     erase  ,
		     extract  ,
		     exit,
		     feature,
		     feel  ,
		     file  ,
		     filter,
		     flac,
		     floppy  ,
		     folder  ,
		     fonts  ,
		     format  ,
		     free  ,
		     frequency  ,
		     ftp,
		     gnome,
		     go  ,
		     gpg,
		     graphics  ,
		     hard  ,
		     hardware  ,
		     hear,
		     hybernate,
		     height,
		     hide,
		     hidden  ,
		     history,
		     idea  ,
		     image,
		     import  ,
		     index,
		     install  ,
		     incoming  ,
		     international  ,
		     internet  ,
		     iso,
		     kazaa,
		     key,
		     kde,
		     keyboard  ,
		     keys  ,
		     kill,
		     language  ,
		     label  ,
		     layout,
		     latex,
		     legal  ,
		     line  ,
		     link  ,
		     'listen to'  ,
		     list  ,
		     login,
		     log,
		     logout,
		     lmule,
		     look  ,
		     mail  ,
		     management  ,
		     megabytes,
		     memory,
		     metadata  ,
		     mldonkey,
		     modify  ,
		     monitor  ,
		     monitoring,
		     mouse  ,
		     move  ,
		     movie  ,
		     mp3,
		     multimedia  ,
		     multiple  ,
		     music  ,
		     napster,
		     navigate  ,
		     nationality,
		     networking  ,
		     network  ,
		     new  ,
		     news,
		     newsgroups  ,
		     name  ,
		     note  ,
		     office,
		     ogm,
		     ogg,
		     open  ,
		     options  ,
		     order  ,
		     pack  ,
		     package  ,
		     packed  ,
		     panel  ,
		     partition  ,
		     paste,
		     pdf,
		     percentage,
		     peripherals,
		     picture,
		     pgp,
		     plain,
		     play,
		     pointer,
		     poor,
		     power,
		     press,
		     precompiled,
		     print,
		     printer,
		     processing,
		     process,
		     program,
		     programming,
		     protocol,
		     publishing,
		     queue,
		     quick,
		     quit,
		     ram,
		     read,
		     receive,
		     refresh,
		    remove,
		    rename,
		     repeat,
		    reset,
		     resize,
		     resolution,
		    restart,
		     resume,
		    repartition,
		     rip,
		     save,
		    saver,
		    schedule,
		     scheduled,
		    science,
		    screen,
		     screenshot,
		    scrolling,
		     search,
		    segusoland,
		    see,
		     select,
		     session,
		    set,
		    send,
		    settings,
		    setup,
		    shortcuts,
		    show,
		    shutdown,
		     site,
		    size,
		     snapshot,
		     share,
		     software,
	  song,
	  sort,
	  sound,
	  space,
	  speed,
		     statistics,
		     stop,
		     string,
		     substring,
	  subtitle,
		     sub,
		     synchronize,
		     system,
		     swap,
		     take,
		     tasks  ,
		     terminal,
		     tex,
		     texmacs,
		     tm,
		     terminate,
	  text  ,
	  theme  ,
		     time,
	  topic  ,
		     track,
		    transfer,
		    trashcan  ,
	  'turn on'  ,
	  'turn off'  ,
		     unassociate,
	  unpack  ,
	  unpacked  ,
	  unwatch  ,
	  unzip  ,
		     update,
	  url,
		     usage,
	  usb,
		     user,
	  usenet  ,
	  velocity  ,
	  very  ,
	  video  ,
	  view  ,
	  visibility  ,
		     volume,
	  watch  ,
		     wav,
	  'x window system',
		     xmule,
		     xcdroast,
	  web  ,
	  windows  ,
	  word  ,
		     width,
	  write  ,
	  www,
	  zip]  ),

	assertExtensionsAutomatically,

	% END

	% REGION add a default categorizazion of verbs
	assertRelationship(verb, 'View files contained in RPM',
			   [rpm, extract, view, show, package, compiled, precompiled, binary,
			    system, administration,
			    display, contents, file, list, archive, packed, archiving, see]),
	assertRelationship(verb, 'Save database now',
			   [segusoland, database, save, data]),
	assertRelationship(verb, 'Login as different user',
			   [logout, login, user, exit, quit, session, terminate, end, kde, gnome]),
	assertRelationship(verb, 'Hybernate',
			   [logout, login, user, exit, quit, session, terminate, end, kde, gnome, hybernate,
			    system, management, administration, hardware]),
	assertRelationship(verb, 'View files contained in DEB', 
			   [deb, extract, view, show, package,compiled, precompiled, binary,
			    system, administration,
			    display, contents, file, list, archive, packed, archiving, see]),
	assertRelationship(verb, 'Share and download files over the internet',
			   [share, download, file, internet, 
			     networking, amule, multimedia,
			     emule, xmule, napster, kazaa, lmule, edonkey, mldonkey]),
			    
	assertRelationship(verb, 'Rip CD', [rip, cd, processing, audio, video, sound, wav, mp3, ogg, disc,
					    peripherals, device, hardware, 
					    ogm, flac, movie, multimedia, decode, encode, extract, track]),
	assertRelationship(verb, 'Rip DVD', [rip, dvd, processing, audio, video, sound, wav, mp3, ogg, disc,
					    ogm, flac, movie, multimedia, peripherals, device, hardware,
					     decode, encode, extract, track]),

	assertRelationship(verb, 'Decompress APE to WAV',
			   [decompress, unpack, decode, unzip, ape, wav, multimedia, sound, audio, music, song, processing, extract]),
	assertRelationship(verb, 'Decompress MP3 to WAV',
			   [decompress, unpack, decode, unzip, mp3, wav, multimedia, sound, audio, music, song, processing, extract]),
	assertRelationship(verb, 'Decompress OGG to WAV',
			   [decompress, unpack, decode, unzip, ogg, wav, multimedia, sound, audio, music, song, processing, extract]),
	assertRelationship(verb, 'Decompress OGM to WAV',
			   [decompress, unpack, decode, unzip, ogm, wav, multimedia, sound, audio, music, song, processing, extract]),

	assertRelationship(verb, 'Compress WAV to MP3',
			   [compress, pack, packed,  encode, zip, mp3, wav, multimedia, sound, audio, music, song, processing]),
	assertRelationship(verb, 'Compress WAV to OGM',
			   [compress, pack, packed, encode, zip, ogm, wav, multimedia, sound, audio, music, song, processing]),
	assertRelationship(verb, 'Compress WAV to FLAC',
			   [compress, pack, packed, encode, zip, flac, wav, multimedia, sound, audio, music, song, processing]),

	assertRelationship(verb, 'View print queue',
			   [hardware, device, peripherals, printer, print,
			    queue, list, view, show, see, display, look]),

	assertRelationship(verb, 'Encrypt',
			   [file, cryptography, encrypt, hide, hidden, key, keys,
			    encode, pgp, gpg, file, processing]),

	assertRelationship(verb, 'Partition disk',
			   [disc, partition , empty, blank, repartition, resize,
			    size, change, modify, system, management,
			    administration, memory, free, hardware, device, peripherals]),

	assertRelationship(verb, 'Format disk',
			   [disc, format, erase, clear,  empty, blank, partition, repartition, resize,
			    size, change, modify, system, management,
			    administration, memory, free, hardware, device, peripherals]),

	
	assertRelationship(verb, 'Close program automatically X minutes from now',
			   [program, close, kill, terminate, quit]),
	assertRelationship(verb, 'Show/hide file sizes',
			   [size, bytes, megabytes, segusoland, settings, setup, set, show, file, see, display,
			    view, delete, hide, hidden, visibility, remove, options, configure,adjust, customize]),
	assertRelationship(verb, 'Show/hide log',
			   [log, history, segusoland,  show, see, display,
			    view,  hide, hidden, visibility]),
	assertRelationship(verb, 'Show/hide file dates',
			   [date, time, segusoland, settings, setup, set, show, file, see, display,
			    view, delete, hide, hidden, visibility, remove, options, configure,
			    adjust, customize]),
	assertRelationship(verb, 'Show/hide directories below the files',
			   [directory, folder, segusoland, settings, setup, set, show, file, see, display,
			    view, delete, hide, hidden, visibility, remove, options, configure,
			    adjust, customize]),
	assertRelationship(verb, 'Delete directory', [directory, delete, remove, folder, management, file]),
	assertRelationship(verb, 'Stop indexing this directory',
			   [directory, delete, options, customize, adjust, configure, segusoland,
			    contents, see, unwatch, view, display, show,
			    settings, set, setup, stop, index, remove, folder, management, file]),
	assertRelationship(verb, 'Import directory and all its files',
			   [directory, segusoland, settings,set, setup, configure, watch , customize,adjust,
			    options, import, add, index,show, view, see, display, folder, management, file]),
	assertRelationship(verb, 'Import directory and all its files (recursively)',
			   [directory, segusoland, settings,set, setup, configure, watch , customize,adjust,
			    options, import, add, index,show, view, see, display, folder, management, file]),
	assertRelationship(verb, 'Browse directory', [directory, browse, open, program, see, contents,
						      show, view, look, display, folder, management, file]),
	assertRelationship(verb, 'Copy files to directory', [directory, copy, folder, management, file]),
	assertRelationship(verb, 'Move files to directory (preserving concepts)',
			   [directory, cut, paste, copy, move,
			    folder, management, file]),
	assertRelationship(verb, 'Associate concepts to directory (and all its files)',
			   [directory, associate, assign, concept, idea, metadata, attribute,
			    feature, folder, category, contents, file]),
	assertRelationship(verb, 'Rename directory (preserving concepts)',
			   [directory, change, name, modify, 
			    set, rename, folder, management, file]),
	assertRelationship(verb, 'View PDF', [pdf, document,text, look, file, show, open, read, view, display, office, see]),

	assertRelationship(verb, 'View PDF', [pdf, document,text, look, file, show, open, read, view, display, see]),
	assertRelationship(verb, 'View DVI file', [dvi,text, document, look, file, show, open, read, view, display, see]),
	
	assertRelationship(verb, 'Create new text file', [ create,  write, new, text, plain, file, document, word, processing]),
	assertRelationship(verb, 'Transfer files to and from an FTP site',
			   [ transfer, file, ftp, site,network, go, networking,url, connect, client, internet,
			     send, receive , protocol]),
	assertRelationship(verb, 'View calendar and schedule', [ view, show, display, modify, change, calendar,
								 take, note, schedule, scheduled, appointments, see,
								 office, tasks, management]),
	assertRelationship(verb, 'Rename file', [ change, rename , file, name, modify]),
	assertRelationship(verb, 'Rename multiple files', [ change, rename, file, name, modify]),
	assertRelationship(verb, 'Change keyboard layout', [ kde, modify, change, 'x window system', settings,
							     nationality, layout, key, 
							     keyboard, options, button,
							     customize, configure, setup, set,
							     language, adjust,
							     keys, peripherals, international, device, hardware]),
	assertRelationship(verb, 'Change keyboard repeat speed',
			   [ kde, key, keys, modify, click, press, change, 'x window system', settings,
			     speed, repeat, velocity, button,
			     keyboard, options,
			     customize, configure, setup, set,
			     language, adjust,
			      peripherals, device, hardware]),
	assertRelationship(verb, 'Create new PDF file', [ office, create, write,  new, text, pdf, file, document, word, processing]),
	assertRelationship(verb, 'Print PDF', [ office, printer, send, print, text, pdf, file, document]),
	assertRelationship(verb, 'Print text file', [ printer, send, print, text, plain, file, document]),
	assertRelationship(verb, 'Edit plain text file', [edit, editor, programming, view, display, open,
							  show, change, modify, text, look, read, office,
							  file, see, document, plain, processing]),
	assertIfNotAlreadyThere(related('create' , verb,'Create new SXW file' )),
	assertIfNotAlreadyThere(related('office' , verb,'Create new SXW file' )),
	assertIfNotAlreadyThere(related('new' , verb,'Create new SXW file' )),
	assertIfNotAlreadyThere(related(sxw , verb,'Create new SXW file' )),
	assertIfNotAlreadyThere(related('file' , verb,'Create new SXW file' )),
	assertIfNotAlreadyThere(related('document' , verb,'Create new SXW file' )),
	assertIfNotAlreadyThere(related('word' , verb,'Create new SXW file' )),
	assertIfNotAlreadyThere(related('text' , verb,'Create new SXW file' )),
	assertIfNotAlreadyThere(related('processing' , verb,'Create new SXW file' )),

	assertRelationship(verb, 'Edit SXW file',
			   [office, edit, change, modify, view, open,
			    display, show, file, document, word, processing,
			    text, look, read, see, editor, sxw]),

	assertRelationship(verb, 'Edit TM file',
			   [office, edit, publishing, change, modify, view, open,
			    display, show, file, document, word, processing,
			    text, look, read, see, editor, texmacs, tm]),

	assertRelationship(verb, 'Edit DOC file',
			   [office, edit, change, modify, view, open,
			    display, show, file, document, word, processing,
			    text, look, read, see, editor, doc]),


	assertIfNotAlreadyThere(related(cd , verb,'Burn data CD' )),
	assertIfNotAlreadyThere(related(disc , verb,'Burn data CD' )),
	assertIfNotAlreadyThere(related(peripherals , verb,'Burn data CD' )),
		assertIfNotAlreadyThere(related(hardware , verb,'Burn data CD' )),
			assertIfNotAlreadyThere(related(device , verb,'Burn data CD' )),
	assertIfNotAlreadyThere(related('burn' , verb,'Burn data CD' )),
	assertIfNotAlreadyThere(related('data' , verb,'Burn data CD' )),
	assertIfNotAlreadyThere(related('create' , verb,'Burn data CD' )),
	assertIfNotAlreadyThere(related('write' , verb,'Burn data CD' )),
	assertIfNotAlreadyThere(related('new' , verb,'Burn data CD' )),
	assertIfNotAlreadyThere(related('archiving' , verb,'Burn data CD' )),
	assertIfNotAlreadyThere(related('archive' , verb,'Burn data CD' )),

	assertRelationship(verb, 'Burn CD from BIN and CUE files',
			   [cd, disc, burn, bin, cue, create, write,
			    new, archiving, archive, multimedia, peripherals, device , hardware,
			    music, song, audio, sound]),
	assertRelationship(verb, 'Burn video CD',
			   [cd, disc, peripherals, device, hardware, burn,  create, write, video, movie,
			    new, archiving, archive, multimedia]),
	assertRelationship(verb, 'Burn CD from audio and CUE files',
			   [cd, disc, peripherals , device, hardware, burn, ogg, ogm, mp3, flac,
			    ape, cue, create, write,
			    new, archiving, archive, multimedia,
			    music, song, audio, sound]),
	assertRelationship(verb, 'Burn from ISO',
			   [cd, disc, peripherals, device, hardware, burn, iso, dvd, image, create, write,
			    new, archiving, archive]),
	
	assertIfNotAlreadyThere(related(cd , verb,'Burn audio CD' )),
	assertIfNotAlreadyThere(related(peripherals , verb,'Burn audio CD' )),
		assertIfNotAlreadyThere(related(device , verb,'Burn audio CD' )),
			assertIfNotAlreadyThere(related(hardware , verb,'Burn audio CD' )),
	assertIfNotAlreadyThere(related(disc , verb,'Burn audio CD' )),
	assertIfNotAlreadyThere(related('burn' , verb,'Burn audio CD' )),
	assertIfNotAlreadyThere(related('audio' , verb,'Burn audio CD' )),
	assertIfNotAlreadyThere(related('create' , verb,'Burn audio CD' )),
	assertIfNotAlreadyThere(related('write' , verb,'Burn audio CD' )),
	assertIfNotAlreadyThere(related('new' , verb,'Burn audio CD' )),
	assertIfNotAlreadyThere(related('archiving' , verb,'Burn audio CD' )),
	assertIfNotAlreadyThere(related('archive' , verb,'Burn audio CD' )),
	assertIfNotAlreadyThere(related('multimedia' , verb,'Burn audio CD' )),
	assertIfNotAlreadyThere(related('music' , verb,'Burn audio CD' )),
	assertIfNotAlreadyThere(related('song' , verb,'Burn audio CD' )),
	assertIfNotAlreadyThere(related('sound' , verb,'Burn audio CD' )),


	assertRelationship(verb, 'Erase CD-RW',
			   [erase, delete, clear, peripherals, device, hardware, cd, dvd, empty, archiving, blank, archive, remove, contents, disc, free]),
	assertRelationship(verb, 'Eject disc',
			   [eject, cd, dvd, disc, floppy, peripherals, device, hardware]),

	assertRelationship(verb, 'Create a new concept for organizing files',
			   [concept, file, idea, create, new, add, segusoland, metadata, attribute,
			    feature, category]),
	assertRelationship(verb, 'Delete a concept',
			   [concept, idea, delete, remove, hide, hidden,
			    segusoland, metadata, attribute,
			    feature, category]),

	assertRelationship(verb, 'Associate concepts to files',
			   [concept, assign, catalogue, idea, append, delete, change, modify, define, set, management, add, remove, segusoland,
			    metadata, attribute, file, associate, deassociate, unassociate, 
			    attach,attachment, label, feature, category]),


	assertRelationship(verb, 'Download a file from the internet with resume',
			   [download, file, internet, resume,
			    networking, transfer, internet]),
	assertRelationship(verb, 'Check mail', [check, incoming, email, mail, open, look, see, display, show
							, new, networking, internet, view]),
	assertRelationship(verb, 'Read newsgroups', [newsgroups, usenet, read, look, see, display, show,
						    news, networking, internet, view]),
	assertRelationship(verb, 'Compose a new email', [write, check, compose, send, email, mail,
							 new, networking, internet]),
	assertRelationship(verb, 'Mail files as attachments',
			   [send, email, mail,  networking, internet, attach, append, attachment]),

	assertIfNotAlreadyThere(related('networking' , verb,'Browse the internet' )),
	assertIfNotAlreadyThere(related('internet' , verb,'Browse the internet' )),
	assertIfNotAlreadyThere(related('look' , verb,'Browse the internet' )),
	assertIfNotAlreadyThere(related('go' , verb,'Browse the internet' )),
	assertIfNotAlreadyThere(related(www , verb,'Browse the internet' )),
	assertIfNotAlreadyThere(related(url , verb,'Browse the internet' )),
	assertIfNotAlreadyThere(related('link' , verb,'Browse the internet' )),
	assertIfNotAlreadyThere(related('web' , verb,'Browse the internet' )),
	assertIfNotAlreadyThere(related('navigate' , verb,'Browse the internet' )),
	assertIfNotAlreadyThere(related('browse' , verb,'Browse the internet' )),
	assertIfNotAlreadyThere(related('see' , verb,'Browse the internet' )),
	assertIfNotAlreadyThere(related('show' , verb,'Browse the internet' )),
	assertIfNotAlreadyThere(related('view' , verb,'Browse the internet' )),
	assertIfNotAlreadyThere(related('open' , verb,'Browse the internet' )),
	assertIfNotAlreadyThere(related('display' , verb,'Browse the internet' )),
	assertIfNotAlreadyThere(related('link' , verb,'Browse the internet' )),
	assertIfNotAlreadyThere(related('address' , verb,'Browse the internet' )),
	assertIfNotAlreadyThere(related('site' , verb,'Browse the internet' )),


	assertIfNotAlreadyThere(related('add' , verb,'Compress files' )),
	assertIfNotAlreadyThere(related('single' , verb,'Compress files' )),
	assertIfNotAlreadyThere(related('archive' , verb,'Compress files' )),
	assertIfNotAlreadyThere(related('archiving' , verb,'Compress files' )),
	assertIfNotAlreadyThere(related('pack' , verb,'Compress files' )),
	assertIfNotAlreadyThere(related('packed' , verb,'Compress files' )),
	assertIfNotAlreadyThere(related('file' , verb,'Compress files' )),
	assertIfNotAlreadyThere(related('compress' , verb,'Compress files' )),
	assertIfNotAlreadyThere(related('compression' , verb,'Compress files' )),
	assertIfNotAlreadyThere(related('zip' , verb,'Compress files' )),
	assertIfNotAlreadyThere(related('create' , verb,'Compress files' )),
	assertIfNotAlreadyThere(related('new' , verb,'Compress files' )),

	assertIfNotAlreadyThere(related('add' , verb,'Compress files separately' )),
	assertIfNotAlreadyThere(related('different' , verb,'Compress files separately' )),
	assertIfNotAlreadyThere(related('multiple' , verb,'Compress files separately' )),
	assertIfNotAlreadyThere(related('archive' , verb,'Compress files separately' )),
	assertIfNotAlreadyThere(related('archiving' , verb,'Compress files separately' )),
	assertIfNotAlreadyThere(related('pack' , verb,'Compress files separately' )),
	assertIfNotAlreadyThere(related('file' , verb,'Compress files separately' )),
	assertIfNotAlreadyThere(related('packed' , verb,'Compress files separately' )),
	assertIfNotAlreadyThere(related('compress' , verb,'Compress files separately' )),
	assertIfNotAlreadyThere(related('compression' , verb,'Compress files separately' )),
	assertIfNotAlreadyThere(related('zip' , verb,'Compress files separately' )),
	assertIfNotAlreadyThere(related('create' , verb,'Compress files separately' )),
	assertIfNotAlreadyThere(related('new' , verb,'Compress files separately' )),


	assertIfNotAlreadyThere(related('view' , verb,'View contents of archive' )),	
	assertIfNotAlreadyThere(related('open' , verb,'View contents of archive' )),
	assertIfNotAlreadyThere(related('see' , verb,'View contents of archive' )),
	assertIfNotAlreadyThere(related('display' , verb,'View contents of archive' )),
	assertIfNotAlreadyThere(related('show' , verb,'View contents of archive' )),
	assertIfNotAlreadyThere(related('look' , verb,'View contents of archive' )),
	assertIfNotAlreadyThere(related('archive' , verb,'View contents of archive' )),
	assertIfNotAlreadyThere(related('archiving' , verb,'View contents of archive' )),
	assertIfNotAlreadyThere(related('packed' , verb,'View contents of archive' )),
	assertIfNotAlreadyThere(related('compression' , verb,'View contents of archive' )),


	
	assertIfNotAlreadyThere(related('file' , verb,'Delete files' )),
	assertIfNotAlreadyThere(related('delete' , verb,'Delete files' )),
	assertIfNotAlreadyThere(related('remove' , verb,'Delete files' )),
	assertIfNotAlreadyThere(related('management' , verb,'Delete files' )),
	assertIfNotAlreadyThere(related('erase' , verb,'Delete files' )),
	assertIfNotAlreadyThere(related('trashcan' , verb,'Delete files' )),


	assertRelationship(verb, 'Play audio',
			   [multimedia, hear, play, audio, open, music, song,
			    'listen to', sound]),

	assertIfNotAlreadyThere(related('multimedia' , verb,'Enqueue audio for playing' )),
	assertIfNotAlreadyThere(related('play' , verb,'Enqueue audio for playing' )),
	assertIfNotAlreadyThere(related('queue' , verb,'Enqueue audio for playing' )),
	assertIfNotAlreadyThere(related('enqueue' , verb,'Enqueue audio for playing' )),
	assertIfNotAlreadyThere(related('list' , verb,'Enqueue audio for playing' )),
	assertIfNotAlreadyThere(related('line' , verb,'Enqueue audio for playing' )),
	assertIfNotAlreadyThere(related('audio' , verb,'Enqueue audio for playing' )),
	assertIfNotAlreadyThere(related('open' , verb,'Enqueue audio for playing' )),
	assertIfNotAlreadyThere(related('music' , verb,'Enqueue audio for playing' )),
	assertIfNotAlreadyThere(related('song' , verb,'Enqueue audio for playing' )),
	assertIfNotAlreadyThere(related('listen to' , verb,'Enqueue audio for playing' )),
	assertIfNotAlreadyThere(related('hear' , verb,'Enqueue audio for playing' )),
	assertIfNotAlreadyThere(related('sound' , verb,'Enqueue audio for playing' )),


	assertRelationship(verb, 'Play video', [multimedia, play, display, open, video, movie, watch, look, see, show,
						show, view]),
	assertRelationship(verb, 'Play movie with subtitle file',
			   [multimedia, play, display, open, video, movie, watch, look, see, show,
			    show, view, subtitle, sub, srt]),
	assertRelationship(verb, 'Extract video track from OGM movie',
			   [multimedia, extract, track, video,  movie, 
			    ogm, ogg, processing, multimedia]),
	assertRelationship(verb, 'Extract audio track from OGM movie',
			   [multimedia, extract, track, video,  movie, 
			    ogm, ogg, processing, multimedia, sound, audio, music, song]),
	assertRelationship(verb, 'Set sound volume',
			   [multimedia, play, sound, volume, adjust, song, options, customize,
			    set, settings, kde, gnome, setup, configure, music, audio]),
	assertRelationship(verb, 'Enqueue video for playing', [multimedia, play, display, open, video, movie, watch, look, see, show,
						show, view]),
	assertRelationship(verb,'Configure mouse clicking behaviour',
			   [mouse, setup,set, press, button, kde, configure, options, enable, disable,
			    customize, adjust, behaviour, settings, peripherals, hardware, device,
			    modify, change, click, double]),
								       
	assertRelationship(verb,'View free space on disk',
			   [view, show, see, blank, display, space, free, disc, empty,  peripherals, hardware, device]),
								       
%	assertRelationship(verb, 'Manage the directories watched by segusoLand',
%			   [import, directory, category, feature, assign, associate, concept, idea, metadata, attribute, feature,
%			    add, remove, delete,  open, see, view, show, display, watch,
%			    change, modify, unwatch, segusoLand, settings, setup, set, configure,
%			    options, customize, folder, management, look]),



	assertIfNotAlreadyThere(related('setup' , verb,'Set mouse speed' )),
	assertIfNotAlreadyThere(related('set' , verb,'Set mouse speed' )),
	assertIfNotAlreadyThere(related('hardware' , verb,'Set mouse speed' )),
	assertIfNotAlreadyThere(related('change' , verb,'Set mouse speed' )),
	assertIfNotAlreadyThere(related('modify' , verb,'Set mouse speed' )),
	assertIfNotAlreadyThere(related('behaviour' , verb,'Set mouse speed' )),
	assertIfNotAlreadyThere(related('peripherals' , verb,'Set mouse speed' )),
	assertIfNotAlreadyThere(related('x window system' , verb,'Set mouse speed' )),
	assertIfNotAlreadyThere(related(kde , verb,'Set mouse speed' )),
	assertIfNotAlreadyThere(related('speed' , verb,'Set mouse speed' )),
	assertIfNotAlreadyThere(related('acceleration' , verb,'Set mouse speed' )),
	assertIfNotAlreadyThere(related('configure' , verb,'Set mouse speed' )),
	assertIfNotAlreadyThere(related('customize' , verb,'Set mouse speed' )),
	assertIfNotAlreadyThere(related('adjust' , verb,'Set mouse speed' )),	
	assertIfNotAlreadyThere(related('settings' , verb,'Set mouse speed' )),
	assertIfNotAlreadyThere(related('options' , verb,'Set mouse speed' )),
	assertIfNotAlreadyThere(related('mouse' , verb,'Set mouse speed' )),
	assertIfNotAlreadyThere(related('feel' , verb,'Set mouse speed' )),
	assertIfNotAlreadyThere(related('velocity' , verb,'Set mouse speed' )),
	assertIfNotAlreadyThere(related('pointer' , verb,'Set mouse speed' )),
	assertIfNotAlreadyThere(related('cursor' , verb,'Set mouse speed' )),
	assertIfNotAlreadyThere(related('arrow' , verb,'Set mouse speed' )),

	
	assertIfNotAlreadyThere(related('setup' , verb,'Set mouse cursor theme' )),
	assertIfNotAlreadyThere(related('set' , verb,'Set mouse cursor theme' )),
	assertIfNotAlreadyThere(related('change' , verb,'Set mouse cursor theme' )),
	assertIfNotAlreadyThere(related('modify' , verb,'Set mouse cursor theme' )),
	assertIfNotAlreadyThere(related('configure' , verb,'Set mouse cursor theme' )),
	assertIfNotAlreadyThere(related('colour' , verb,'Set mouse cursor theme' )),
	assertIfNotAlreadyThere(related('peripherals' , verb,'Set mouse cursor theme' )),
	assertIfNotAlreadyThere(related('decorations' , verb,'Set mouse cursor theme' )),
	assertIfNotAlreadyThere(related('options' , verb,'Set mouse cursor theme' )),
	assertIfNotAlreadyThere(related('settings' , verb,'Set mouse cursor theme' )),	
	assertIfNotAlreadyThere(related('customize' , verb,'Set mouse cursor theme' )),
	assertIfNotAlreadyThere(related('adjust' , verb,'Set mouse cursor theme' )),
	assertIfNotAlreadyThere(related(kde , verb,'Set mouse cursor theme' )),
	assertIfNotAlreadyThere(related('x window system' , verb,'Set mouse cursor theme' )),
	assertIfNotAlreadyThere(related('appearance' , verb,'Set mouse cursor theme' )),
	assertIfNotAlreadyThere(related('look' , verb,'Set mouse cursor theme' )),
	assertIfNotAlreadyThere(related('theme' , verb,'Set mouse cursor theme' )),
	assertIfNotAlreadyThere(related('mouse' , verb,'Set mouse cursor theme' )),
	assertIfNotAlreadyThere(related('cursor' , verb,'Set mouse cursor theme' )),
	assertIfNotAlreadyThere(related('pointer' , verb,'Set mouse cursor theme' )),
	assertIfNotAlreadyThere(related('hardware' , verb,'Set mouse cursor theme' )),
	assertIfNotAlreadyThere(related('arrow' , verb,'Set mouse cursor theme' )),

	assertIfNotAlreadyThere(related('see' , verb,'Toggle scrolling buttons always visible' )),
	assertRelationship(verb, 'Toggle scrolling buttons always visible',
			   [ segusoland, setup, set, configure, always, settings, options, change, modify,
			     customize, scrolling, adjust,
			     button, hide, display, view, show, see, hidden, visibility]),

	assertIfNotAlreadyThere(related('archive' , verb,'Extract archive' )),
	assertIfNotAlreadyThere(related('archiving' , verb,'Extract archive' )),
	assertIfNotAlreadyThere(related('compression' , verb,'Extract archive' )),
	assertIfNotAlreadyThere(related('open' , verb,'Extract archive' )),
	assertIfNotAlreadyThere(related('extract' , verb,'Extract archive' )),
	assertIfNotAlreadyThere(related('unpack' , verb,'Extract archive' )),
	assertIfNotAlreadyThere(related('unpacked' , verb,'Extract archive' )),
	assertIfNotAlreadyThere(related('decompress' , verb,'Extract archive' )),
	assertIfNotAlreadyThere(related('unzip' , verb,'Extract archive' )),
	assertIfNotAlreadyThere(related('file' , verb,'Extract archive' )),

	assertIfNotAlreadyThere(related('view' , verb,'View picture' )),	
	assertIfNotAlreadyThere(related('image' , verb,'View picture' )),
	assertIfNotAlreadyThere(related('display' , verb,'View picture' )),
	assertIfNotAlreadyThere(related('see' , verb,'View picture' )),	
	assertIfNotAlreadyThere(related('look' , verb,'View picture' )),
	assertIfNotAlreadyThere(related('open' , verb,'View picture' )),
	assertIfNotAlreadyThere(related('graphics' , verb,'View picture' )),
	assertIfNotAlreadyThere(related('multimedia' , verb,'View picture' )),
	assertIfNotAlreadyThere(related('picture' , verb,'View picture' )),
	assertIfNotAlreadyThere(related('show' , verb,'View picture' )),
	assertIfNotAlreadyThere(related('file' , verb,'View picture' )),

	assertRelationship(verb, 'Edit picture',
			   [edit, processing, image,
			    editor,look, show, open, display, view, modify, file, change, graphics, see,
			    multimedia, picture]),
	assertRelationship(verb, 'Open terminal in the directory of this file',
			   [open, terminal,folder, console, cli, directory, file, command, line, display,see,view, show]),
	assertRelationship(verb, 'Print picture', [print, printer, file, picture, graphics, image, multimedia]),
	assertRelationship(verb, 'Configure printer', [print, printer, options, enable, disable,
						       hardware, device, setup, set, settings, configure,
						       customize, adjust]),

	assertRelationship(verb, 'Sort files by date', [sort, file, date, segusoland, view, show, see, display, order]),
	assertRelationship(verb, 'Sort files by size', [sort, file, size, segusoland, view, show, see, display, order]),

	assertRelationship(verb, 'Sort files by name', [sort, file, name, segusoland, view, show, see, display, order]),

	assertRelationship(verb, 'Show scheduled tasks', [display,list, show, view, see, tasks, queue, scheduled,
							  schedule, segusoland, delete, remove, management]),
	assertRelationship(verb, 'Install RPM',
			   [rpm, precompiled, binary, compiled, install,binary, program, package, management, system, administration, management]),

	assertRelationship(verb, 'Install DEB',
			   [deb, precompiled, install, binary, compiled, binary, program, package, management, system, administration, management]),

	assertRelationship(verb, 'Show/hide quick-search boxes',
			   [display, show, hide, hidden, visibility, view, enable, disable,
			    see,quick, string, substring, search, filter,
			    box, entry,options, text , delete, remove, segusoland, configure, setup, settings, set,
			    customize, adjust]),
	assertRelationship(verb, 'Show/hide concepts below files',
			   [display, show, hide, hidden, visibility, view, enable, disable,
			    see,concept, idea, attribute, metadata, category, feature, file, panel,
			    options, text , delete, remove, segusoland, configure, setup, settings, set,
			    customize, adjust]),
	assertRelationship(verb, 'Show/hide concepts below programs',
			   [display, show, hide, hidden, visibility, view, enable, disable,
			    see,concept, idea, attribute, metadata, category, feature, program, panel,
			    options, text , delete, remove, segusoland, configure, setup, settings, set,
			    customize, adjust]),

	assertRelationship(verb, 'View free space on disk',
			   [display,list, show, statistics, system, administration, monitoring,
			    usage, percentage, view, see,monitor,
			    free, empty, space, disc, device, hardware, peripherals]),

	assertRelationship(verb, 'View CPU usage',
			   [display,list, show, view, monitoring, monitor,
			    system,administration, see,cpu, statistics, usage, percentage,  device, hardware, peripherals]),
	assertRelationship(verb, 'View or terminate active processes',
			   [display,list, show, view, terminate, kill, see, process,
			    system, administration, monitor,
			    monitoring, management, statistics, usage, percentage,  software]),
	assertRelationship(verb, 'View memory usage',
			   [display, monitoring, system, management, administration, swap,monitor,
			    list, show, memory, ram, view, see, statistics, usage, percentage,
			    device, hardware, peripherals]),



	assertRelationship(verb, 'Take a snapshot of your screen',
			   [snapshot, image, file, hardware, screenshot, device, screen, monitor,picture, graphics,
			    multimedia, peripherals]),

	assertRelationship(verb, 'Change resolution',
			   [kde, gnome, change, modify, screen, monitor, resolution, device, hardware, peripherals,
			    system, management, administration, setup, configure, settings,
			    customize, options, set, width, height, size, adjust]),

	assertRelationship(verb, 'Show files/programs/actions/devices when no concept is selected',
			   [segusoland, always, hidden, settings, set, enable, disable, configure,adjust, customize,
			    behaviour, panel, hide, list, show, view,visibility,  see, display,options, setup]),

	assertRelationship(verb, 'Do not show files/programs/actions/devices when no concept is selected',
			   [segusoland,hidden, always, visibility,  hide, settings, set, enable, disable, configure,
			    adjust, customize,
			    behaviour, panel, list, show, view, see, display,options, setup]),

	assertRelationship(verb, 'Browse the directory of this file',
			   [display,list, show, view, see, file, directory,
			    browse, folder, navigate, management]),

	assertRelationship(verb, 'Refresh file list',
			   [display, segusoland, list, refresh, update, show, view, see, file, 
			    browse, visibility, management, synchronize]),


	assertRelationship(verb,  'Automatically deselect concepts when a nonconcept is selected',
			   [segusoland, setup, settings,set, options, configure, customize, adjust, concept, idea,
			    disable, enable, metadata, attribute, feature, category,deselect,
			     file, action, program, device]),

%	assertRelationship(device, 'My Mouse', [mouse, hardware, device, pointer, peripherals, cursor]),
%	assertRelationship(device, 'My monitor', [monitor, screen, device, peripherals, hardware]),
%	assertRelationship(device, 'My printer', [printer, device, peripherals, hardware]),
%	assertRelationship(device, 'My CD reader', ['CD', device, peripherals, hardware]),%
%	assertRelationship(device, 'Floppy Drive', [floppy, disc, drive, device, peripherals, hardware]),

		
	% END
	checkForIntegrityOfRelations,
	printTheVerbsThatHaveNotBeenCataloguedYet.

assertExtensionsAutomatically:-
	findall(V, verb(V), Verbs),
	sort(Verbs, V2),
	forall(member(V, V2),
	       ifTheVerbRequiresAMimeTypeThenAssertTheIDeaAndTheRelation(V)).

ifTheVerbRequiresAMimeTypeThenAssertTheIDeaAndTheRelation(V):-
	verbRequiresAtLeastOneObjectSatisfying(V, file, ext(E)),
	assertIfNotAlreadyThere(idea(verb, E)),
	assertIfNotAlreadyThere(related(E, verb, V)).
ifTheVerbRequiresAMimeTypeThenAssertTheIDeaAndTheRelation(V):-
	verbRequiresAtLeastOneObjectSatisfying(V, file, P),
	Head =.. [P, X],
	forall(clause(Head, ext(E, X)),
	       (assertIfNotAlreadyThere(idea(verb, E)),
		   assertIfNotAlreadyThere(related(E, verb, V)))).
ifTheVerbRequiresAMimeTypeThenAssertTheIDeaAndTheRelation(V):-
	\+verbRequiresAtLeastOneObjectSatisfying(V, file, _).

printTheVerbsThatHaveNotBeenCataloguedYet:-
	findall(V, (verb(V) , \+related(_, verb, V)), Verbs),
	sort(Verbs, V2),
	writeln('Verbs not yet catalogued: '),
	(V2 \= [] -> writeln(V2); writeln(none)).
		
assertIdeas(L):-
	forall(member(M, L),
	       assertIfNotAlreadyThere(idea(verb, M))).
assertRelationship(Kind, Verb, ListOfConcepts):-
	checkThatAllConceptsAreAsserted(ListOfConcepts),
	forall(member(M, ListOfConcepts),
	       assertIfNotAlreadyThere(related(M, Kind, Verb))).

checkThatAllConceptsAreAsserted(L):-
	findall(I, (member(I, L), \+idea(verb, I)), Ideas),
	(Ideas=[]->true;
	    (writeln('ERROR: Concepts in related/3 but not in idea/2 : '),
		writeln(Ideas))).

checkForIntegrityOfRelations:-
	ignore(checkThatSynonimsAreAllThereOrNotAtAll([see, view, show /*  not browse: view pdf */])),
	ignore(checkThatSynonimsAreAllThereOrNotAtAll([terminal, console, cli])),
	ignore(checkThatSynonimsAreAllThereOrNotAtAll([keys, key])),
	ignore(checkThatSynonimsAreAllThereOrNotAtAll([newsgroups, usenet])),
	ignore(checkThatSynonimsAreAllThereOrNotAtAll([compiled, precompiled, binary])),	
	ignore(checkThatSynonimsAreAllThereOrNotAtAll([scheduled, schedule])),
	ignore(checkThatSynonimsAreAllThereOrNotAtAll([sound, audio])),
	ignore(checkThatSynonimsAreAllThereOrNotAtAll([empty, blank, free])),
	ignore(checkThatSynonimsAreAllThereOrNotAtAll([refresh , update])),
	ignore(checkThatSynonimsAreAllThereOrNotAtAll(['listen to' , hear])),
	ignore(checkThatSynonimsAreAllThereOrNotAtAll([associate, assign])),
	ignore(checkThatSynonimsAreAllThereOrNotAtAll([deassociate, unassociate])),
	ignore(checkThatSynonimsAreAllThereOrNotAtAll([attach, append])),
	ignore(checkThatSynonimsAreAllThereOrNotAtAll([attachment, attach])), 
	%ignore(checkThatSynonimsAreAllThereOrNotAtAll([image, picture])), no: burn cd image
	ignore(checkThatSynonimsAreAllThereOrNotAtAll([sort, order])),
	ignore(checkThatSynonimsAreAllThereOrNotAtAll([folder, directory])),
	%ignore(checkThatSynonimsAreAllThereOrNotAtAll([memory, ram])),
	ignore(checkThatSynonimsAreAllThereOrNotAtAll([mail, email])),
	ignore(checkThatSynonimsAreAllThereOrNotAtAll([video, movie])),
	ignore(checkThatSynonimsAreAllThereOrNotAtAll([international, nationality])),
	ignore(checkThatSynonimsAreAllThereOrNotAtAll([archive, archiving])),
	ignore(checkThatSynonimsAreAllThereOrNotAtAll([delete, remove])),
	ignore(checkThatSynonimsAreAllThereOrNotAtAll([idea, category, attribute, concept, feature, metadata])),
	ignore(checkThatSynonimsAreAllThereOrNotAtAll([song, music])),
	ignore(checkThatSynonimsAreAllThereOrNotAtAll([settings, /*not set: set concepts associated to files ,*/
						      setup, options, configure, customize, adjust])),
	ignore(checkThatSynonimsAreAllThereOrNotAtAll([modify, change])), /* "edit" is not a synonym for "modify". Edit means: to
									     prepare a text or film for printing or viewing by
									     correcting mistakes and deciding what will
									     be removed and what will be kept in, etc:
									     */

	
	ignore(checkThatSynonimsAreAllThereOrNotAtAll([unpack, decompress, unzip])),
	ignore(checkThatSynonimsAreAllThereOrNotAtAll([hide, hidden])),
	ignore(checkThatSynonimsAreAllThereOrNotAtAll([speed, velocity])),
	ignore(checkThatSynonimsAreAllThereOrNotAtAll([appearance,theme /* not look. it is also a verb*/])),
	ignore(checkThatSynonimsAreAllThereOrNotAtAll([pack, compress /*not zip, because it can be a noun */])),

	% check that, if I have related(burn K, O), then I also have related(write, K, O)
	ignore(checkImplication(burn, write)),
	ignore(checkImplication(visibility, view)), % not viceversa: view movie does not imply visibility
	ignore(checkImplication(video, multimedia)),
	ignore(checkImplication(audio, multimedia)),
	ignore(checkImplication(sound, multimedia)),
	ignore(checkImplication(cd, disc)),
	ignore(checkImplication(dvd, disc)),
	ignore(checkImplication(floppy, disc)),
	ignore(checkImplication(enqueue, queue)),
	ignore(checkImplication(browse , see)),
	ignore(checkImplication(application , program)),
	ignore(checkImplication(terminal, command)),
	ignore(checkImplication(picture, image)),
	ignore(checkImplication(calendar, schedule)),
	ignore(checkImplication(pack, packed)), /*pack  = create a packed X */
	ignore(checkImplication(import, add)),
	ignore(checkImplication(url, internet)),
	%ignore(checkImplication(hide, filter)), % no: hide file sizes is not a filtering
	ignore(checkImplication(www, internet)),
	ignore(checkImplication(schedule, tasks)),
	ignore(checkImplication(settings, set)),
	ignore(checkImplication(picture, file)),
	ignore(checkImplication(ram, memory)),
	ignore(checkImplication(pdf, file)),
	ignore(checkImplication(trashcan, delete)),
	%ignore(checkImplication(view, look)), false: always view scrolling buttons
	%ignore(checkImplication(display, look)), false: always display scrolling buttons
	ignore(checkImplication(queue, list)),
	ignore(checkImplication(document, file)),
	% ignore(checkImplication(view, open)), false "always view the scroll buttons"
	ignore(checkImplication(edit, open)),
	ignore(checkImplication(edit, modify)),  
	ignore(checkImplication(song, audio)),
	ignore(checkImplication(show, display)), % not viceversa: display is also a noun
	ignore(checkImplication(picture, graphics)),
	ignore(checkImplication(graphics, multimedia)),
	ignore(checkImplication(unpack, extract)), % but not viceversa : extract audio track
	ignore(checkImplication(speech, audio)),
	ignore(checkImplication(shutdown, power)),
	ignore(checkImplication(shutdown, 'turn off')),
	ignore(checkImplication(read, see)),
	ignore(checkImplication(watch, see)),
	ignore(checkImplication(mouse, hardware)),
	%ignore(checkImplication(monitor, screen)), no: monitor is also a verb
	ignore(checkImplication(screen, monitor)),
	ignore(checkImplication(mouse, peripherals)),
	ignore(checkImplication(compression, archiving)),
	ignore(checkImplication(partition, disc)),
	ignore(checkImplication(disc, peripherals)),
	ignore(checkImplication(peripherals, hardware)),
	ignore(checkImplication(repartition, partition)),
	ignore(checkImplication(theme, appearance)),
	ignore(checkImplication(theme, look)),
	ignore(checkImplication(keyboard, hardware)),
	ignore(checkImplication(keyboard, key)),
	ignore(checkImplication(keyboard, peripherals)),


	ignore(checkThatIfAVerbHasAMetadataThenNoProgramHasThatMetadata),
	true.

checkThatIfAVerbHasAMetadataThenNoProgramHasThatMetadata:-
	related(I, verb, V),
	related(I, program, P),
	writeln(error-checkThatIfAVerbHasAMetadataThenNoProgramHasThatMetadata-P-and-V-have-the-same-concept-I).
checkThatSynonimsAreAllThereOrNotAtAll(L):-
	member(M, L),
	member(M2, L),
	M \= M2,
	related(M, K, O),
	\+related(M2, K, O),
	writeln(error-related(M, K, O)-but-not-related(M2, K, O)).

checkImplication(A, B):-
	related(A, K, O),
	\+related(B, K, O),
	writeln(error-related(A, K, O)-but-not-related(B, K, O)).


readLoop(Db):-
	read(Db, T),
	readLoopAux(Db, T).

readLoopAux(_, end_of_file):-
	writeln(end_of_file).
readLoopAux(Db, T):-
	T=theQuickSearchBoxesAreVisible,
	% REGION we cannot show the quick search boxes now, because they don't exist yet. So we simply
	% set a variable which will be read at the end of run/1
	assertIfNotAlreadyThere(theQuickSearchBoxesAreVisible),
	% END
	% REGION read next
	read(Db, T2),
	!,readLoopAux(Db, T2).
        % END
readLoopAux(Db, T):-
	T=showPanelsWhenNoConceptIsSelected,
	assertIfNotAlreadyThere(showPanelsWhenNoConceptIsSelected),
	% REGION read next
	read(Db, T2),
	!,readLoopAux(Db, T2).
        % END
readLoopAux(Db, T):-
	T=recentConcept(C),
	assertz(recentConcept(C)),
	% REGION read next
	read(Db, T2),
	!,readLoopAux(Db, T2).
        % END
readLoopAux(Db, T):-
	T=recentAction(Rep, Act),
	assertz(recentAction(Rep, Act)),
	% REGION read next
	read(Db, T2),
	!,readLoopAux(Db, T2).
        % END
readLoopAux(Db, T):-
	T=scrollingButtonsAlwaysVisible,
	% REGION we cannot show the quick search boxes now, because they don't exist yet. So we simply
	% set a variable which will be read at the end of run/1
	assertIfNotAlreadyThere(scrollingButtonsAlwaysVisible),
	% END
	% REGION read next
	read(Db, T2),
	!,
	readLoopAux(Db, T2).
        % END
readLoopAux(Db, T):-
	T=autoDeselectConcepts,
	assertIfNotAlreadyThere(autoDeselectConcepts),
	% REGION read next
	read(Db, T2),
	!,
	readLoopAux(Db, T2).
        % END
readLoopAux(Db, T):-
	T=panelStateMinimized(K),
	assertIfNotAlreadyThere(panelStateMinimized(K)),
	% REGION read next
	read(Db, T2),
	!,
	readLoopAux(Db, T2).
        % END
readLoopAux(Db, T):-
	T=table(_, _, real),
	concat_atom(['WARNING: Reading database: Discarded old style predicate table/3.'], Text),
	writeln(Text),
	% REGION read next
	read(Db, T2),
	!,
	readLoopAux(Db, T2).
        % END
readLoopAux(Db, T):-
	T= idea(_),
	concat_atom(['WARNING: Reading database: Discarded old style predicate idea/1.'], Text),
	writeln(Text),
	% REGION read next
	read(Db, T2),
	!,
	readLoopAux(Db, T2).
        % END
readLoopAux(Db, T):-
	T= idea(K, N),
	downcase_atom(N, N2),
	N=N2,
	assertIfNotAlreadyThere(idea(K, N)),
	% REGION read next
	read(Db, T2),
	!,
	readLoopAux(Db, T2).
        % END
readLoopAux(Db, T):-
	T= idea(_, N),
	downcase_atom(N, N2),
	N\=N2,
	concat_atom(['WARNING: Reading database: Discarded invalid idea ' , N, ' because it is not lowercase'], Text),
	writeln(Text),
	% REGION read next
	read(Db, T2),
	!,
	readLoopAux(Db, T2).
        % END
readLoopAux(Db, T):-
	T= idea(K, N),
	assertIfNotAlreadyThere(idea(K, N)),
	% REGION read next
	read(Db, T2),
	!,
	readLoopAux(Db, T2).
        % END


readLoopAux(Db, T):-
	T=belongsTo(Table, N, Kind),
	assertObjectBelongsToTable(N, Kind, Table),
	% REGION read next
	read(Db, T2),
	!,
	readLoopAux(Db, T2).
        % END
readLoopAux(Db, T):-
	T=segusoDir(D),
	expand_file_name(D, [DExp]),
	exists_directory(DExp),
	assert(segusoDir(DExp)),
	% REGION read next
	read(Db, T2),
	!,
	readLoopAux(Db, T2).
        % END
readLoopAux(Db, T):-
	T=segusoDir(D),
	expand_file_name(D, [DExp]),
	\+exists_directory(DExp),
	concat_atom(['WARNING: Reading database: Discarded directory ', DExp, ' because it does not seem to exist.'], Text),
	writeln(Text),
	% REGION read next
	read(Db, T2),
	!,
	readLoopAux(Db, T2).
        % END
readLoopAux(Db, T):-
	T=recentViewPredefinedTable(_K, _Table, _ReadableNameOfPredicate),
	%	assertRecentViewReadFromFile(K, Table, ReadableNameOfPredicate),
	writeln(obsolete-recent-view-ignored),
	% REGION next
	read(Db, T2),
	!, readLoopAux(Db, T2).
        % END



readLoopAux(_, T):-
	write('TRAGIC FAILURE: did not match: '),
	writeln(T).
/*
assertRecentViewReadFromFile(K, Table, ReadableNameOfPredicate):-
	atom(ReadableNameOfPredicate),
	assert(recentView(K, Table, ReadableNameOfPredicate)).
assertRecentViewReadFromFile(K, _Table, ReadableNameOfPredicate):-
	\+atom(ReadableNameOfPredicate),
	ReadableNameOfPredicate=[T],
	(table(K, T, real)-> % this requires that the recentView predicates be at the end of the database file.
	    assert(recentView(K, T, ReadableNameOfPredicate));
	    writeln(recent-view-discarded-because-table-T-does-not-seem-to-exist-anymore)).
assertRecentViewReadFromFile(K, _Table, ReadableNameOfPredicate):-
	\+atom(ReadableNameOfPredicate),
	ReadableNameOfPredicate \=[_],
	is_list(ReadableNameOfPredicate),
	assertNewClauseWithDerivedTable(K, ReadableNameOfPredicate, and, _, _). % @@ ignored the last two...
*/

assertObjectBelongsToTable(ObjName, file, _Table):-
	\+exists_file(ObjName),
	concat_atom(['Discarding info about f
		    ile ', ObjName, ' because it does not exist.'], Text),
	writeln(Text).
assertObjectBelongsToTable(ObjName, file, Table):-
	exists_file(ObjName),
	file_directory_name(ObjName, Dir),
	iAmWatchingTheDir(Dir),
	assertIfNotAlreadyThere(related(Table, file, ObjName)).
assertObjectBelongsToTable(ObjName, file, _Table):-
	exists_file(ObjName),
	file_directory_name(ObjName, Dir),
	\+iAmWatchingTheDir(Dir),
	concat_atom(['Discarding info about file ', ObjName, ' because I am not watching its dir.'], Text),
	writeln(Text).
assertObjectBelongsToTable(ObjName, Kind,Table):-
	Kind\=file,
	% REGION if the object is valid
	call(Kind, ObjName),
	% END
	assertIfNotAlreadyThere(related(Table,Kind, ObjName)).
assertObjectBelongsToTable(ObjName, Kind,_Table):-
	Kind\=file,
	% REGION if the object is not valid
	\+call(Kind, ObjName),
	% END
	concat_atom(['Discarding info about  ', Kind, ' ', ObjName, ' because it is not a valid ', Kind], Text),
	writeln(Text).

	



convertToRealFileIfItIsALink(F, Target):-
	read_link(F, _, Target).
convertToRealFileIfItIsALink(F, F):-
	\+read_link(F, _, _).
	

% iAmWatchingTheDir/1. This is not so easy  because the dir could
% be ~ and there could be segusoDir('/home/gesucristo/'). More in general,
% 1) one might have an unexpanded ~, and the other one could have it expanded.
% 2) one might end in /, the other one may not.
% 3) one or both could be links to the same physical dir.
% ALGO: 1)  expand them both 2) resolve links of both, if any 3) add slash if needed
% 4) do a syntactic equality check (unification).

iAmWatchingTheDir(D):-
	% REGION expand file names
	expand_file_name(D, [DExp]),
	segusoDir(SegDir),
	expand_file_name(SegDir, [SegDirExp]),
	% END
	% REGION resolve links
	convertToRealFileIfItIsALink(DExp, DExpNoLink),
	convertToRealFileIfItIsALink(SegDirExp, SegDirExpNoLink),
	% END
	% REGION add slash if there isn't any
	addSlashToDirIfNeeded(SegDirExpNoLink, SegDirExpSlashed),
	addSlashToDirIfNeeded(DExpNoLink, DExpSlashed),
	% END
	DExpSlashed = SegDirExpSlashed.
% @@ possible bug: this does not detect equality between /home/mau and /home//mau


assertIfNotAlreadyThere(_, _, _):-
	writeln('unmanaged case'),
	trace.

onMainWindowDestroyed:-
	writeln('onMainWindowDestroyed: saving database'),
	% save database -- write database
	expand_file_name('~', [HomeDir]),
	concat_atom([HomeDir, '/.segusoLand-database'], DatabaseFileNamewithFullPath),
	open(DatabaseFileNamewithFullPath, write, DB),
	% REGION write segusodirs
	forall(segusoDir(D), writeWithNewline(DB, segusoDir(D))),
	% END
	% REGION save each table together with its records
	findall(idea(K, Name),idea(K, Name),RealTables),
	forall(member(M, RealTables), dumpTable(M, DB)),
	% END
	% REGION save recent views
	%forall(recentView(K, Table, ReadableNameOfTable),
	%	   writeWithNewline(DB, recentViewPredefinedTable(K, Table, ReadableNameOfTable))),
	% END

	% REGION save the quick-search boxes visibility
	(theQuickSearchBoxesAreVisible->writeWithNewline(DB, theQuickSearchBoxesAreVisible); true),
	% END
	% REGION save the panel states
	(panelStateMinimized(file)->writeWithNewline(DB, panelStateMinimized(file)); true),
	(panelStateMinimized(directory)->writeWithNewline(DB, panelStateMinimized(directory)); true),
	(panelStateMinimized(time)->writeWithNewline(DB, panelStateMinimized(time)); true),
	(panelStateMinimized(program)->writeWithNewline(DB, panelStateMinimized(program)); true),
	(panelStateMinimized(device)->writeWithNewline(DB, panelStateMinimized(device)); true),
	% END
	(autoDeselectConcepts->writeWithNewline(DB, autoDeselectConcepts); true),
	(scrollingButtonsAlwaysVisible->writeWithNewline(DB, scrollingButtonsAlwaysVisible); true),
	(showPanelsWhenNoConceptIsSelected->writeWithNewline(DB, showPanelsWhenNoConceptIsSelected); true),
	forall(recentConcept(X),
	       writeWithNewline(DB, recentConcept(X))),
	forall(recentAction(Rep, Ac),
	       writeWithNewline(DB, recentAction(Rep, Ac))),
	close(DB),
	writeln('database saved').


% REGION dumpTable/2

dumpTable(idea(K, N), Db):-
	writeWithNewline(Db, idea(K, N)),
	findall((K, O), related(N,K, O), Items),  % better not use setof, findall helps discovering repetitions
	forall(member((K, O), Items), writeWithNewline(Db, belongsTo(N, O, K))).
% END



verbIdeaToIcon(action, 'img/action.png').
verbIdeaToIcon(adjust, 'img/action.png').
verbIdeaToIcon(appointments, 'img/korganizer.png').
verbIdeaToIcon(archiving, 'img/archive.png').
verbIdeaToIcon(archive, 'img/archive.png').
verbIdeaToIcon(audio, 'img/audio.png').
verbIdeaToIcon(behaviour, 'img/action.png').
verbIdeaToIcon(bytes, 'img/size.png').
verbIdeaToIcon(burn, 'img/flame.gif').
verbIdeaToIcon(box, 'img/archive.png').
verbIdeaToIcon(browse, 'img/glasses.png').
verbIdeaToIcon(cancel, 'img/cross.png').
verbIdeaToIcon(compress, 'img/archive.png').
verbIdeaToIcon(compression, 'img/archive.png').
verbIdeaToIcon(command, 'img/gnome-terminal.png').
verbIdeaToIcon(compose, 'img/text-file.png').
verbIdeaToIcon(computer, 'img/computer.png').
verbIdeaToIcon(configure, 'img/options.png').
verbIdeaToIcon(console, 'img/gnome-terminal.png').
verbIdeaToIcon(customize, 'img/options.png').
verbIdeaToIcon(date, 'img/korganizer.png').
verbIdeaToIcon(delete, 'img/cross.png').
verbIdeaToIcon(directory, 'img/folder.png').
verbIdeaToIcon(display, 'img/monitor.png').
verbIdeaToIcon(email, 'img/email.png').
verbIdeaToIcon(folder, 'img/folder.png').
verbIdeaToIcon(filter, 'img/mbuto.png').
verbIdeaToIcon(graphics, 'img/picture.png').
verbIdeaToIcon(hear, 'img/audio.png').
verbIdeaToIcon(hide, 'img/hidden.png').
verbIdeaToIcon(hidden, 'img/hidden.png').
verbIdeaToIcon(internet, 'img/internet.png').
verbIdeaToIcon(international, 'img/italian-flag.png').
verbIdeaToIcon(language, 'img/italian-flag.png').
verbIdeaToIcon(kde, 'img/kde.png').
verbIdeaToIcon(keyboard, 'img/keyboard.png').
verbIdeaToIcon(keys, 'img/keyboard.png').
verbIdeaToIcon(kill, 'img/cross.png').
verbIdeaToIcon('listen to', 'img/audio.png').
verbIdeaToIcon(mail, 'img/email.png').
verbIdeaToIcon(megabytes, 'img/size.png').
verbIdeaToIcon(monitor, 'img/monitor.png').
verbIdeaToIcon(multimedia, 'img/video.png').
verbIdeaToIcon(music, 'img/audio.png').
verbIdeaToIcon(newsgroups, 'img/pan.png').
verbIdeaToIcon(play, 'img/audio.png').
verbIdeaToIcon(picture, 'img/picture.png').
verbIdeaToIcon(quit, 'img/turnoff.png').
verbIdeaToIcon(mouse, 'img/mouse.png').
verbIdeaToIcon(movie, 'img/video.png').
verbIdeaToIcon(office, 'img/korganizer.png').
verbIdeaToIcon(options, 'img/options.png').
verbIdeaToIcon(pack, 'img/archive.png').
verbIdeaToIcon(packed, 'img/archive.png').
verbIdeaToIcon(pdf, 'img/pdf.png').
verbIdeaToIcon(refresh, 'img/refresh.png').
verbIdeaToIcon(resolution, 'img/resolution.png').
verbIdeaToIcon(remove, 'img/cross.png').
verbIdeaToIcon(rename, 'img/name.png').
verbIdeaToIcon(screen, 'img/monitor.png').
verbIdeaToIcon(scrolling, 'img/scroll-buttons.png').
verbIdeaToIcon(stop, 'img/turnoff.png').
verbIdeaToIcon(set, 'img/options.png').
verbIdeaToIcon(settings, 'img/options.png').
verbIdeaToIcon(see, 'img/glasses.png').
verbIdeaToIcon(show, 'img/glasses.png').
verbIdeaToIcon(setup, 'img/options.png').
verbIdeaToIcon(string, 'img/name.png').
verbIdeaToIcon(substring, 'img/name.png').
verbIdeaToIcon(segusoland, 'img/icon.png').
verbIdeaToIcon(shutdown, 'img/cross.png').
verbIdeaToIcon(select, 'img/deselect-all.png').
verbIdeaToIcon(search, 'img/mbuto.png').
verbIdeaToIcon(size, 'img/size.png').
verbIdeaToIcon(site, 'img/internet.png').
verbIdeaToIcon(song, 'img/audio.png').
verbIdeaToIcon(sxw, 'img/ooowriter.png').
verbIdeaToIcon(sound, 'img/audio.png').
verbIdeaToIcon(snapshot, 'img/gnome-screenshot.png').
verbIdeaToIcon(screenshot, 'img/gnome-screenshot.png').
verbIdeaToIcon(synchronize, 'img/refresh.png').
verbIdeaToIcon(terminal, 'img/gnome-terminal.png').
verbIdeaToIcon(terminate, 'img/cross.png').
verbIdeaToIcon(time, 'img/1.png').
verbIdeaToIcon(trashcan, 'img/trash.png').
verbIdeaToIcon('turn off', 'img/turnoff.png').
verbIdeaToIcon(url, 'img/internet.png').
verbIdeaToIcon(unpack, 'img/archive.png').
verbIdeaToIcon(unpacked, 'img/archive.png').
verbIdeaToIcon(update, 'img/refresh.png').
verbIdeaToIcon(usenet, 'img/pan.png').
verbIdeaToIcon(view, 'img/glasses.png').
verbIdeaToIcon(visibility, 'img/glasses.png').
verbIdeaToIcon(watch, 'img/glasses.png').
verbIdeaToIcon(write, 'img/text-file.png').
verbIdeaToIcon(www, 'img/internet.png').
verbIdeaToIcon(web, 'img/internet.png').

verbIdeaToIcon(calendar, 'img/korganizer.png').
verbIdeaToIcon(_, 'img/bulb.png').


typeToIcon(txt, 'img/text-file.png').
typeToIcon(pdf, 'img/pdf.png').
typeToIcon(doc, 'img/word-26x26.png').
typeToIcon(rpm, 'img/rpm.png').
typeToIcon(mp3, 'img/audio.png').
typeToIcon(flac, 'img/audio.png').
typeToIcon(ogg, 'img/audio.png').
typeToIcon(wav, 'img/audio.png').
typeToIcon(mpc, 'img/audio.png').
typeToIcon(flac, 'img/audio.png').
typeToIcon(ogm, 'img/video.png').
typeToIcon(avi, 'img/video.png').
typeToIcon(mpg, 'img/video.png').
typeToIcon(mpeg, 'img/video.png').
typeToIcon(sxw, 'img/ooowriter.png').
typeToIcon(wma, 'img/audio.png').
typeToIcon(wmv, 'img/video.png').
typeToIcon(spec, 'img/rpm.png').
typeToIcon(asf, 'img/video.png').
typeToIcon(zip, 'img/archive.png').
typeToIcon(rar, 'img/archive.png').
typeToIcon(gz , 'img/archive.png').
typeToIcon(bz2, 'img/archive.png').
typeToIcon(tar, 'img/archive.png').
typeToIcon(arj, 'img/archive.png').
typeToIcon(ace, 'img/archive.png').
typeToIcon(htm, 'img/internet.png').
typeToIcon(html, 'img/internet.png').
typeToIcon(png, 'img/picture.png').
typeToIcon(gif, 'img/picture.png').
typeToIcon(jpg, 'img/picture.png').
typeToIcon(ppm, 'img/picture.png').
typeToIcon(bmp, 'img/picture.png').
typeToIcon(xpm, 'img/picture.png').
typeToIcon(blend, 'img/picture.png').
typeToIcon(_, 'img/generic.png').


verbTooltip('Associate concepts to files', 'Associating concepts to files is useful because when you select those concepts you only see those files.').
verbTooltip('Create a new concept for organizing files', 'Associating concepts to files is a useful way to organize your files, because when you select those concepts you only see those files. It is similar to organizing files in directories, but better.').
verbTooltip('Do not show files/programs/actions/devices when no concept is selected', 'This is the default. At the beginning you only see the Index of Concepts. This is to prevent newbies from panicking before the many panels.').
verbTooltip('Show files/programs/actions/devices when no concept is selected', 'This is not the default. Only for experts!').
verbTooltip('Import directory and all its files', 'If you import a directory, all its files will appear in segusoLand\'s file panel. segusoLand tries to hide the concept of directory: after you import a directory, you can forget directories exist. You can concentrate on WHAT it is, not on WHERE it is.').
verbTooltip('Show scheduled tasks', 'segusoLand\'s time panel allows you to schedule any action to be executed in the future. If you schedule an action to be executed in the future, sometimes you may want to see the list of scheduled actions, or to delete one of them. Click here to see the list of scheduled actions.').
verbTooltip('Import directory and all its files (recursively)', 'If you import a directory, all its files will appear in segusoLand\'s file panel. This imports a directory and all its subdirectories, recursively.').
verbTooltip('Compress files', 'Creates a compressed archive containing all the selected files.').
verbTooltip('Automatically deselect concepts when a nonconcept is selected', 'This option is probably useless. :-) You can turn it on and off.').
verbTooltip('Show/hide concepts below files', 'By default, segusoLand displays below each file the list of concepts associated to that file, in green colour. You may want to hide the concepts if they take too much space. Click here to toggle this settings on and off.').
verbTooltip('Show/hide concepts below programs', 'By default, segusoLand displays below each program the list of concepts associated to that program, in green colour. You may want to hide the concepts if they take too much space. Click here to toggle this settings on and off.').

verbTooltip('Show/hide file sizes', 'You can choose to display the size of files directly in the file panel, below the files. Click here to turn this option on and off').
verbTooltip('Show/hide file dates', 'You can choose to display the modification date of files directly in the file panel, below the files. Click here to turn this option on and off').
verbTooltip('Show/hide directories below the files', 'You can choose to display the directories of each file, along with the file. Click here to turn this option on and off').
verbTooltip('Delete a concept', 'Delete a concept, so it will not appear in the index anymore. It also deletes any associations between the concept and files.').
verbTooltip('Compress files separately', 'Creates one archive for each selected file, separately. The new compressed files have the same metadata as the original ones.').
verbTooltip('Stop indexing this directory', 'This is the opposite of importing a directory. If you stop indexing a directories, its files will disappear from the file panel. You will also loose any associations between the files and concepts.').
verbTooltip('Toggle scrolling buttons always visible', 'Scrolling buttons are those buttons that appear above and below a panel, allowing you to scroll. The current implementation sometimes causes annoying "jumps" in the panel content. So you can decide to let these buttons always visible, defeating the problem. But it takes more space. The default is "do not show".').
verbTooltip('Sort files by date', 'Click here to sort the files by date. You often invoke this command from the "recent actions" panel.').
verbTooltip('Sort files by size', 'Click here to sort the files by size. You often invoke this command from the "recent actions" panel.').
verbTooltip('Sort files by name', 'Click here to sort the files by name. You often invoke this command from the "recent actions" panel.').
verbTooltip('Show/hide directories below the files', 'Although segusoLand tries to hide the concept of "directory" as much as possible (preferring folders, which are different), sometimes it\'s useful to see the directory of a file, especially if you are trying to free disk space.').

verbTooltip('Show/hide quick-search boxes', 'If you click here, quick-search boxes will appear. Quick search-boxes are text entries where you can type a search string in order to restrict the visible files, verbs, etc.').
verbTooltip('Refresh file list', 'Synchronizes segusoLand\'s internal database with the disk. You should click this if you created, deleted or moved some file with an external program. Newly created files will appear on the bottom of the file list.').

%verbTooltip( 'Merge two or more file folders', 'Merge the selected folders into one. This is useful, for example, if you downloaded a file that contained the information "singer=Elvis". This information becomes a folder in segusoLand, called "Elvis (singer)". This could be annoying if you ALREADY had a folder named, for example, "E. Presley (singer)". It is bad to have two folders with the same meaning: if you are looking for all songs by Elvis, you risk to select only one folder and to forget to select the other. So it\'s better if you merge the two folders.').
verbTooltip(_, '(No help yet)').

verbToIcon('Create a new concept for organizing files', 'img/bulb.png').
verbToIcon('Import directory and all its files', 'img/folder.png').
verbToIcon('Import directory and all its files (recursively)', 'img/folder.png').
verbToIcon('Delete directory', 'img/delete-folder.png').
verbToIcon('Stop indexing this directory', 'img/delete-folder.png').
verbToIcon('Delete a concept', 'img/bulb-delete.png').
verbToIcon('Automatically deselect concepts when a nonconcept is selected', 'img/bulb-deselect.png').
verbToIcon('Associate concepts to files', 'img/bulb.png').
verbToIcon('Set mouse speed', 'img/mouse.png').
verbToIcon('Set mouse cursor theme', 'img/mouse.png').
verbToIcon('Toggle scrolling buttons always visible', 'img/scroll-buttons.png').
verbToIcon('Browse the directory of this file', 'img/folder.png').


verbToIcon('Save database now', 'img/floppy.png').
%verbToIcon('Manage the directories watched by segusoLand', 'img/folder.png').
verbToIcon('Show scheduled tasks', 'img/1.png').
verbToIcon('Show/hide directories below the files', 'img/folder.png').
verbToIcon('Show/hide log', 'img/text-file.png').
verbToIcon('Show/hide tooltips', 'img/name.png').
verbToIcon('Persistent arrow boxes on/off', 'img/name.png').
verbToIcon('Show/hide concepts below files', 'img/bulb.png').
verbToIcon('Show/hide concepts below programs', 'img/bulb.png').
verbToIcon('Show/hide hidden files (.*)', 'img/hidden.png').
verbToIcon('Show/hide quick-search boxes', 'img/mbuto.png').
verbToIcon('Show/hide file sizes', 'img/size.png').
verbToIcon('Show/hide file dates', 'img/1.png').
verbToIcon('Sort files by date', 'img/1.png').
verbToIcon('Sort files by name', 'img/name.png').
verbToIcon('Sort files by size', 'img/size.png').
verbToIcon('Refresh file list', 'img/refresh.png').
verbToIcon('Rename file', 'img/name.png').
verbToIcon('Encrypt', 'img/encrypt.png').
verbToIcon('Mail files as attachments', 'img/mailto.png').
verbToIcon('Compose a new email', 'img/email.png').
verbToIcon('Setup K3B', 'img/k3b.png').
verbToIcon('Rename multiple files', 'img/name.png').
verbToIcon('Create new PDF file', 'img/pdf.png').
verbToIcon( 'Create new SXW file', 'img/ooowriter.png').
verbToIcon('Create new DOC file', 'img/word-26x26.png').
verbToIcon('Edit DOC file', 'img/word-26x26.png').
verbToIcon('View PDF', 'img/glasses.png').
verbToIcon('Shutdown', 'img/turnoff.png').
verbToIcon('Print picture', 'img/printer.png').
verbToIcon('Print PDF', 'img/printer.png').
verbToIcon('Play movie with subtitle file', 'img/video.png').
verbToIcon('Play video', 'img/video.png').
verbToIcon('Browse the internet', 'img/internet.png').
verbToIcon('View HTML file', 'img/glasses.png').
verbToIcon('View DVI file', 'img/glasses.png').
verbToIcon('View contents of archive', 'img/glasses.png').
verbToIcon('Calculate size', 'img/size.png').
verbToIcon('Delete files', 'img/delete.png').
verbToIcon('Play audio', 'img/audio.png').
verbToIcon('Enqueue audio for playing', 'img/audio.png').
verbToIcon('Enqueue video for playing', 'img/video.png').
verbToIcon('Set sound volume', 'img/volume.png').
verbToIcon('Burn audio CD', 'img/cdrom.png').
verbToIcon('Burn data CD', 'img/cdrom.png').
verbToIcon('Burn from ISO', 'img/cdrom.png').
verbToIcon('Burn video CD', 'img/cdrom.png').
verbToIcon('Restore file from trashcan', 'img/restore-trash.png').
verbToIcon('Move to trashcan', 'img/trash.png').
verbToIcon('Empty trashcan', 'img/empty-trash.png').
verbToIcon('Erase CD-RW', 'img/cdrom.png').
verbToIcon('Burn CD from audio and CUE files', 'img/cdrom.png').
verbToIcon('Burn CD from BIN and CUE files', 'img/cdrom.png').
verbToIcon('Compress files separately', 'img/archive.png').
verbToIcon('Compress files', 'img/archive.png').
verbToIcon('Extract archive', 'img/archive.png').
verbToIcon('Create new text file', 'img/text-file.png').
verbToIcon('Edit plain text file', 'img/text-file.png').
verbToIcon('Edit SXW file', 'img/ooowriter.png').
verbToIcon('Share and download files over the internet', 'img/share-files.png').
verbToIcon('Print text file', 'img/printer.png').
verbToIcon('Transfer files to and from an FTP site', 'img/share-files.png').
verbToIcon('Edit picture', 'img/picture.png').
verbToIcon('View picture', 'img/glasses.png').
verbToIcon('Install RPM', 'img/rpm.png').
verbToIcon('Install DEB', 'img/rpm.png').
verbToIcon('View calendar and schedule', 'img/schedule.png').
verbToIcon('Check mail', 'img/email.png').
verbToIcon('Setup mouse', 'img/mouse.png').
verbToIcon('Configure mouse clicking behaviour', 'img/mouse.png').
verbToIcon('Change keyboard layout', 'img/keyboard.png').
verbToIcon('Change keyboard repeat speed', 'img/keyboard.png').
verbToIcon('Change resolution', 'img/resolution.png').
verbToIcon('Configure printer', 'img/printer.png').
verbToIcon('View print queue', 'img/printer.png').
verbToIcon('Close program automatically X minutes from now', 'img/turnoff.png').
verbToIcon('Read newsgroups', 'img/pan.png').
verbToIcon('Edit TM file', 'img/texmacs.png').
verbToIcon('View CPU usage', 'img/procman.png').
verbToIcon('View memory usage', 'img/procman.png').
verbToIcon('View SWAP use', 'img/procman.png').
verbToIcon('View files contained in RPM', 'img/glasses.png').
verbToIcon('View files contained in DEB', 'img/glasses.png').
verbToIcon('View or terminate active processes', 'img/procman.png').
verbToIcon('Take a snapshot of your screen', 'img/gnome-screenshot.png').
verbToIcon('Open terminal in the directory of this file', 'img/gnome-terminal.png').
verbToIcon('Edit 3D picture', 'img/picture.png').

verbToIcon(_, 'img/generic.png').

programToIcon('Korganizer', 'img/korganizer.png').
programToIcon('Kde control center', 'img/kcontrol.png').
programToIcon('Sylpheed', 'img/sylpheed.png').
programToIcon('Ksnapshot', 'img/ksnapshot.png').
programToIcon('Downloader for X', 'img/d4x.png').
programToIcon('Konqueror', 'img/konqueror.png').
programToIcon('OpenOfficeWriter', 'img/ooowriter.png').
programToIcon('Pan', 'img/pan.png').
programToIcon('Opera', 'img/opera.png').
programToIcon('Galeon', 'img/galeon.png').
programToIcon('K3b', 'img/k3b.png').
programToIcon('KBear', 'img/kbear.png').
programToIcon('Nautilus', 'img/nautilus.png').
programToIcon('Kuickshow', 'img/kuickshow.png').
programToIcon('Kprinter', 'img/kprinter.png').
programToIcon('FileRoller', 'img/fileroller.png').
programToIcon('Kdiskfree', 'img/kdiskfree.png').
programToIcon('Xcdroast', 'img/xcdroast.png').
programToIcon('abiWord', 'img/abiword.png').
programToIcon('Gedit', 'img/gedit.png').
programToIcon('KJobViewer', 'img/kprinter.png').
programToIcon('Totem', 'img/totem.png').
programToIcon('Kde system guard', 'img/kdesystemguard.png').
programToIcon('Gpdf', 'img/gpdf.png').
programToIcon('The gimp', 'img/gimp.png').
programToIcon('Evolution', 'img/evolution.png').
programToIcon('Kmail', 'img/kmail.png').
programToIcon('Knode', 'img/knode.png').
programToIcon('Kget', 'img/kget.png').
programToIcon('Kghostview', 'img/kghostview.png').
programToIcon('Xmule', 'img/xmule.png').
programToIcon('Xine', 'img/xine.png').
programToIcon('Gxine', 'img/xine.png').
programToIcon('Mplayer', 'img/mplayer.png').
programToIcon('Kdvi', 'img/kdvi.png').
programToIcon('Ark', 'img/ark.png').
programToIcon('Xmms', 'img/xmms.png').
programToIcon('Rpmdrake', 'img/rpmdrake.png').
programToIcon('Diskdrake', 'img/diskdrake.png').
programToIcon('Eye of Gnome', 'img/eog.png').
programToIcon('Mozilla', 'img/mozilla2.png').
programToIcon('Mozilla Firebird', 'img/mozilla.png').
programToIcon('Xpdf', 'img/xpdf.png').
programToIcon('Emacs', 'img/emacs.png').
programToIcon('Aumix', 'img/aumix.png').
programToIcon('Kmix', 'img/kmix.png').
programToIcon('Trashcan', 'img/trash.png').
programToIcon('segusoLand', 'img/icon.png').
programToIcon('gThumb', 'img/gthumb.png').
programToIcon('TeXmacs', 'img/texmacs.png').
programToIcon('Epiphany', 'img/epiphany.png').
programToIcon('Gnome system monitor', 'img/procman.png').
programToIcon('Gnome screenshot', 'img/gnome-screenshot.png').
programToIcon('Gnome terminal', 'img/gnome-terminal.png').
programToIcon('gFTP', 'img/gftp.png').

programToIcon(_, 'img/generic.png').


deviceToIcon('Floppy Drive', 'img/floppy.png').
deviceToIcon('My monitor', 'img/monitor.png').
deviceToIcon('My Keyboard', 'img/keyboard.png').
deviceToIcon('My Mouse', 'img/mouse.png').
deviceToIcon('My printer', 'img/printer.png').
deviceToIcon('Primary Hard disk', 'img/diskdrake.png').
deviceToIcon('Secondary Hard disk', 'img/diskdrake.png').
deviceToIcon('My Computer', 'img/computer.png').
deviceToIcon('My sound card', 'img/audio.png').
deviceToIcon('My CD writer', 'img/cdrom.png').
deviceToIcon('My CD reader', 'img/cdrom.png').
deviceToIcon(X, 'img/diskdrake.png'):-
	diskPartition(X).
deviceToIcon(_, 'img/generic.png').

writeWithNewline(Database, Term):-
	writeq(Database, Term),
	put(Database, '.'),
	put(Database, '\n').

assertExtensionIfAny(B):-
	file_name_extension(_, Ext, B),
	Ext\='',
	nonvar(Ext),
	atom(Ext),
	%verbRequiresAtLeastOneObjectSatisfying(_, ext(Ext)),  % if the extension is useful.... @@ doesn't work because there
	% areindirect extensions, such as videoFile... We could transform videoFile/1 into ext(videoFile, _), and it would work.
	downcase_atom(Ext, Ext_lowercase),
	assert(ext(Ext_lowercase, B)).
assertExtensionIfAny(B):-
	file_name_extension(_, '', B).

% @@ to be written. For example. if F =/usr/local/doc/thesis/thesis----by-mau.pdf, it creates the list
% [usr, local, doc, thesis, by, mau, pdf] and asserts related(usr, file, F), related(local, file, F), etc.
assertConcepts(F):-
	theFileHasAValidUtf8Name(F),
	splitStringIntoWords(F, L),
	forall((member(Idea, L),theFileHasAValidUtf8Name(Idea)),
	       (downcase_atom(Idea, Idea2),
		   
		   (theFileHasAValidUtf8Name(Idea2)->
		       (assertIfNotAlreadyThere(idea(file, Idea2)),
			   /*
			   (checkListBoxItemExists(checkListBoxOfKind(idea), pair(file, Idea2))->true;
			       checkListBoxAddItem(checkListBoxOfKind(idea), Idea2,
						   Idea2,
						   ''/*tooltip*/, [], ''/*dir*/,
						   '', '', ''/*icon*/)),
	       */
			   assertIfNotAlreadyThere(related(Idea2, file, F)))
		   ; writeln(skippedIdeaNotUTF-Idea2)))).
assertConcepts(F):-
	\+theFileHasAValidUtf8Name(F),
	concat_atom(['Assertion of file ', F, ' discarded because its name is not UTF8 compatible'], T),
	writeln(T).
	

/*
% REGION sortByNameInsensitiveNotCountingPath
sortByNameInsensitiveNotCountingPath(List, Sorted):-
	addBasenameInFront(List, List2),
	sort(List2, List3),
	removeBasenameInFront(List3, Sorted).
addBasenameInFront([], []).
addBasenameInFront([H|T], [L-H|T2]):-
	file_base_name(H, B),
	downcase_atom(B, L),
	addBasenameInFront(T, T2).
removeBasenameInFront([], []).
removeBasenameInFront([_-H|T], [H|T2]):-
	removeBasenameInFront(T, T2).
% END
*/


% REGION sortByNameInsensitive
sortByNameInsensitive(List, Sorted):-
	addKeys(List, Keys),
	sort(Keys, S),
	removeKeys(S, Sorted).
addKeys([], []).
addKeys([H|T], [L-H|T1]):-
	downcase_atom(H, L),
	addKeys(T, T1).
removeKeys([], []).
removeKeys([_-H|T], [H|T1]):-
	removeKeys(T, T1).
% END

/*
onSortByFileNameButtonToggled:-
	toggleButtonIsPressed(sortByFileNameButton),
	toggleButtonSetPressedState(sortByFileDateButton, released),
	toggleButtonSetPressedState(sortByFileSizeButton, released), 
	retractall(fileSortMode(_)),
	assert(fileSortMode(name)),
	checkListBoxSortByName(checkListBoxOfKind(file)).
onSortByFileNameButtonToggled:-
	\+toggleButtonIsPressed(sortByFileNameButton).

onSortByFileDateButtonToggled:-
	toggleButtonIsPressed(sortByFileDateButton),
	toggleButtonSetPressedState(sortByFileNameButton, released),
	toggleButtonSetPressedState(sortByFileSizeButton, released),  
	retractall(fileSortMode(_)),
	assert(fileSortMode(date)),
	checkListBoxSortByDate(checkListBoxOfKind(file)).
onSortByFileDateButtonToggled:-
	\+toggleButtonIsPressed(sortByFileDateButton).


onSortByFileSizeButtonToggled:-
	toggleButtonIsPressed(sortByFileSizeButton),
	toggleButtonSetPressedState(sortByFileNameButton, released),
	toggleButtonSetPressedState(sortByFileDateButton, released),
	retractall(fileSortMode(_)),
	assert(fileSortMode(size)),
	checkListBoxSortBySize(checkListBoxOfKind(file)).
onSortByFileSizeButtonToggled:-
	\+toggleButtonIsPressed(sortByFileSizeButton).

*/

/*
sortFilesAccordingToMode(F, Sorted):-
	fileSortMode(name),
	sortByNameInsensitiveNotCountingPath(F, Sorted).
sortFilesAccordingToMode(F, Sorted):-
	fileSortMode(date),
	predsort(sortByDate, F, Sorted).
sortFilesAccordingToMode(F, Sorted):-
	fileSortMode(size),
	predsort(sortBySize, F, Sorted).



onSortFileAux:-
	% REGION empty checklistbox 
	checkListBoxOfKind(file, C),
	% END
	% REGION sort selected objects of that kind (even not in current table)
	findall(O2, selected(file, O2), SelectedFiles),
	sortObjectsAccordingToKindAndMode(file, SelectedFiles, SortedSelectedFiles),
	% END

	% REGION 2) add all the objects in current table which
	             %are not selected (selected ones have already been added)
	curView(file, T),
	findall(O, (related(T,file, O), \+selected(file, O)), Objs),   % do not use setof or bagof! THey can fail.
	sortFilesAccordingToMode(Objs, SortedFiles).
	% END

	  after sorting  you don't need to do narrowing...
	  	selectedObjects(Sel)
	% REGION narrow verbs
	curView(verb, TableVerbs),
	forall(related(TableVerbs,verb, V), decideVerbVisibility(V)),
	% END
	% REGION narrow objects to only those objects that are required by a visible verb!
	findall(V, theVerbIsVisible(V, Sel), VisibleVerbs),
	forall(objectKind(K2),
	       (curView(K2, T2),
		   forall(related(T2,K2, O2), decideObjectVisibility(O2,K2, VisibleVerbs)))).
*/


	
onRecentFilterSelected(recentView(Kind, Predicate, ReadableNameOfPredicate)):-
	retractall(curView(Kind, _)),
	assert(curView(Kind, Predicate)),

	% REGION retract and assert again so this filter is listed first next time
	retract(recentView(Kind, Predicate, ReadableNameOfPredicate)),
	asserta(recentView(Kind, Predicate, ReadableNameOfPredicate)),
	% END

	% REGION trim all but the 20 most recent views
	find20MostRecentViews(MostRecent), % MostRecent is in reverse order, so ...
	retractall(recentView(_, _, _)),
	forall(member(M, MostRecent),
	       asserta(M)), % .. i must use asserta.
	% END
	commonShit(Kind, Predicate, ReadableNameOfPredicate),
	dialogClose(recentFileFiltersDialog),
	setGoButton.

find20MostRecentViews(L):-
	findall(recentView(X, Y, Z), recentView(X, Y, Z), AllViews),
	find20MostRecentViewsAux(1, [], AllViews, L).
find20MostRecentViewsAux(20, L, _Remaining, L).
find20MostRecentViewsAux(_, SoFar, [], SoFar).
find20MostRecentViewsAux(N, SoFar, [H|T], L):-
	N<20,
	!,
	N2 is N+1,
	find20MostRecentViewsAux(N2, [H|SoFar], T, L).




assertIfNotAlreadyThere(X):-
	X.
assertIfNotAlreadyThere(X):-
	\+X,
	assert(X).
assertIfNotAlreadyThere(_):-
	writeln('assertIfNotAlreadyThere failed...'),
	throw('tragic error').



addNewlinesToString(Position, % if Position=5, I wrap every 5 letters*/
		    String, % in
		    NewString):-  % out
	atom_chars(String, List),
	addNewlinesToStringAux(Position, 0, [], List, ListWithNewlinesInserted),
	concat_atom(ListWithNewlinesInserted, NewString).
addNewlinesToStringAux(_, _Count, CurList, [], CurList).
addNewlinesToStringAux(Position, Position, CurList, [H|T], NewList):-
	H\=[],
	append(CurList, [H, '\n'], C2),
	!,
	addNewlinesToStringAux(Position, 0, C2, T, NewList).
addNewlinesToStringAux(Position, Count, CurList, [H|T], NewList):-
	H\=[],
	Count < Position,
	NewCount is Count+1,
	append(CurList, [H], C2),
	!,
	addNewlinesToStringAux(Position, NewCount, C2, T, NewList).
	
:-dynamic(nextFreeConst/1).
nextFreeConst(1).

newconst(C):-
	nextFreeConst(F),
	concat_atom([p, F], C),
	retract(nextFreeConst(F)),
	D is F+1,
	assert(nextFreeConst(D)).

/*

  	forall(member(M, FoldersSorted),
	       ((selected(idea, M) ; theIdeaIntersectAllSelectedIdeas(M))->checkListBoxAddItem(checkListBoxOfKind(idea), M, )
		   ; true)),

	forall(selected(idea, I),
	       checkListBoxSetItemState(checkListBoxOfKind(idea), I, selected)),

	
  this won't work! I can't simply hide all idea that don't intersect
the currently selected ideas.

  use case:

  I want to play a song by Michael Jackson.

  I want to select the PLAY idea, then the MICHAL JACKSON idea. But as
soon as I select PLAY, MICHAEL JACKSON disappears. Because the two
idea don't intersect: there is no item that is EXPLICITLY in
both. MICHAEL JACKSON contains only files, whereas PLAY contains only
verbs.


the solution is that MICHAEL JACKSON must not disappear, because?


  */

  
redrawFolderPanelHidingAndReshowing:-
	\+selectedLetter(_),
	checkListBoxHideAllItems(checkListBoxOfKind(idea)),
	forall(selected(idea-K, I),
	       ( checkListBoxSetItemState(checkListBoxOfKind(idea), pair(K,I), selected))),
	checkListBoxDefeatBug(checkListBoxOfKind(idea)).
redrawFolderPanelHidingAndReshowing:-
	writeln(debug-1),
	selectedLetter(L),
	checkListBoxHideAllItems(checkListBoxOfKind(idea)),
	forall(selected(idea-K, I),
	       (  checkListBoxSetItemState(checkListBoxOfKind(idea), pair(K,I), selected))),
	forall((  idea(K, I),
		  \+selected(idea-K, I),
		  theIdeaIntersectAllSelectedIdeasOfThatKind(I, K),
		  theIdeaIsCompatibleWithAllSelectedIdeasOfOtherKinds(I, K),
		  theNameMatchesFilter(idea, I),
		  theAtomBeginsWithLetter(I, L)),
	       checkListBoxSetItemState(checkListBoxOfKind(idea), pair(K, I), deselected)),
	checkListBoxDefeatBug(checkListBoxOfKind(idea)),
	writeln(debug-exit).
% only use this if you just added or removed a concept
redrawFolderPanelClearingAndReaddingEverything:-
	checkListBoxClear(checkListBoxOfKind(idea)),
	findall(pair(I, K)/* first I, then K. Because we want to sort by I */,
		idea(K, I), Ideas),
	sort(Ideas, Ideas2),
	writeln(redrawFolderPanelClearingAndReaddingEverything),
	forall(member(pair(I, K), Ideas2),
	       ( (thereIsAnotherIdeaWithThisName(K, I)->
		     (K=verb -> ReadableName = I;
		     	 concat_atom([I, ' [in ', K, 's]'], ReadableName))
		 ; ReadableName = I),
		   (K\=verb-> Icon = ''; 
		   verbIdeaToIcon(I, Icon)),
		   checkListBoxAddItem(checkListBoxOfKind(idea),
				       pair(K, I) /*reversing order. Previously it was pair(I, K). */,
				       ReadableName,
				       ''/*tooltip*/, [], ''/*dir*/,
				       '', '', Icon))),
       	writeln(debug-aaa),

       redrawFolderPanelHidingAndReshowing,
       writeln(redrawFolderPanelClearingAndReaddingEverything-quit).

thereIsAnotherIdeaWithThisName(K, I):-
	idea(K2, I),
	K2\=K.
	

theAtomBeginsWithLetter(A, L):-
	L\=0,
	downcase_atom(A, A2),
	downcase_atom(L, L2),
	atom_chars(A2, List),
	List = [L2|_].
theAtomBeginsWithLetter(A, L):-
	L=0,
	atom_chars(A, List),
	List = [L2|_],
	char_type(L2, digit).


theIdeaIsCompatibleWithAllSelectedIdeasOfOtherKinds(_, file):-
	\+selected(idea-verb, _).
theIdeaIsCompatibleWithAllSelectedIdeasOfOtherKinds(IF, file):-
	forall(selected(idea-verb, IV),
	       ( related(IV, verb, V),
		   verbRequiresAtLeastOneObjectSatisfying(V, file, P),
		   related(IF, file, F),
		   call(P, F))).
	
theIdeaIsCompatibleWithAllSelectedIdeasOfOtherKinds(_, verb):-
	\+selected(idea-file, _).
theIdeaIsCompatibleWithAllSelectedIdeasOfOtherKinds(I, verb):-
	selected(idea-file, IF),
	related(I, verb, V),
	verbRequiresAtLeastOneObjectSatisfying(V, file, P),
	related(IF, file, F),
	call(P, F).
	       
theIdeaIntersectAllSelectedIdeasOfThatKind(I, K):-
	forall((selected(idea-K, I2), I2\=I),
	       ( related(I, K, CommonItem),
		   related(I2, K, CommonItem))).

presentTime('Now').

